#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "tpu2program4xi.py">
#<abstract4xi abstract = "TPU program management"/>
#<refer4xi reference = "program4xi.py"/>
#<verify4xi verification = "10" target = "tpu2program4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================
# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'TPU2Program4Xi',                      # ACK 5.6     p132  

            'add2programwithbinary4tpu',           # ACK 5.6.1   p132            
            'add2programwithsource4tpu',           # ACK 5.6.1   p133  
                                
            'build2program4tpu',                   # ACK 5.6.2   p137      

            'compile2program4tpu',                 # ACK 5.6.3   p139            
            'create2programwithsource4tpu',        # ACK 5.6.1   p132 
            'create2programwithbinary4tpu',        # ACK 5.6.1   p133 
            'create2programwithbuiltinkernels4tpu',# NACK 5.6.1  p135   
                       
            'delete2programwithbinary4tpu',        # ACK  
            'delete2programwithsource4tpu',        # ACK               
            
            'fetch2programbinary4tpu',             # ACK
            'fetch2programsource4tpu',             # ACK
            
            'get2programinfo4tpu',                 # ACK 5.6.7   p150 
            'get2programbuildinfo4tpu',            # NACK 5.6.7  p153
            
            'link2program4tpu',                    # ACK 5.6.3   p142                          
            'release2program4tpu',                 # ACK 5.6.1   p136                                         
            'retain2program4tpu',                  # ACK 5.6.1   p136
 
            'set2programwithsource4tpu',           # ACK
            'set2programwithbinary4tpu',           # ACK            
            'set2programcallback4tpu',             # ACK 
                              
            'trace2program4tpu',                   # ACK
            'unload2compiler4tpu'                  # NACK 5.6.6  p149 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi
from version4xi import VersionKB4Xi

# -------------------------------------------------------------------------------------------------
# Buffer                           
# -------------------------------------------------------------------------------------------------
from tpu2buffer4xi import get2bufferinfo4tpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from tpu2commandqueue4xi import TPU2CommandQueue4Xi
from tpu2commandqueue4xi import create2commandqueue4tpu 
from tpu2commandqueue4xi import flush2commandqueue4tpu
from tpu2commandqueue4xi import finish2commandqueue4tpu
from tpu2commandqueue4xi import get2commandqueueinfo4tpu
from tpu2commandqueue4xi import release2commandqueue4tpu
from tpu2commandqueue4xi import retain2commandqueue4tpu
from tpu2commandqueue4xi import set2commandqueue4tpu
from tpu2commandqueue4xi import trace2commandqueue4tpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from tpu2context4xi import TPU2Context4Xi 
from tpu2context4xi import create2context4tpu
from tpu2context4xi import get2context4tpu
from tpu2context4xi import release2context4tpu
from tpu2context4xi import retain2context4tpu
from tpu2context4xi import set2context4tpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from tpu2device4xi import TPU4Xi
from tpu2device4xi import init2tpu4xi 
from tpu2device4xi import set2tpu4xi
from tpu2device4xi import get2tpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from tpu2event4xi import TPU2Event4Xi
from tpu2event4xi import create2event4tpu
from tpu2event4xi import get2eventinfo4tpu
from tpu2event4xi import release2event4tpu
from tpu2event4xi import retain2event4tpu
from tpu2event4xi import set2eventcallback4tpu 
from tpu2event4xi import set2event4tpu
from tpu2event4xi import trace2event4tpu
from tpu2event4xi import wait2event4tpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from process4xi import Process4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                         
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global tpu2program4xi                                                   # Program of TPU
global tpu2build4xi                                                     # Build of TPU
global tpu2compile4xi                                                   # Compile of TPU
global tpu2trace4xi                                                     # TRACE

global dir2binary4tpu                                                   # Directory for binary files
global dir2source4tpu                                                   # Directory for source files

global fetch2code4source                                                # Fetched source code
global fetch2code4binary                                                # Fetched binary code

global import2use4tpu                                                   # Imports of target function for Process4Xi

global kb2fetch4tpu                                                     # Knowledgebase for fetching

global folder2lib4tpu                                                   # Folder of TPU library

global profile2fetch4source                                             # Extension for source files
global keys2profile4source                                                
global profile2fetch4binary                                             # Extension for binary files
global keys2profile4binary                                               

global target2function4tpu                                              # Target function for Process4Xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "
temp2tpu = 'tpu'
key2dir = temp2tpu[0]                                                                                               # MORE COMPLIATED WAY DUE TO AUTOMATIC SOFTWARE GENERATION
folder2lib4tpu = VersionKB4Xi[key2dir] 
dir2binary4tpu = folder2lib4tpu + '__pycache__//'                                                                   # Default directory is Python cache for binary files - C++ exe files are in //bin
dir2source4tpu = folder2lib4tpu                                                                                     # Directory for source files

profile2fetch4source = {}
profile2fetch4source ['.py']  = 'ACK'                                                                               # Example extensions: '.txt'= text '.py'=Python, '.cpp'=C++, '.hpp'=headers, '.c'=C
profile2fetch4source ['.cpp'] = 'ACK'
profile2fetch4source ['.c']   = 'ACK'
profile2fetch4source ['.cl']  = 'ACK'
keys2profile4source = profile2fetch4source.keys()

profile2fetch4binary = {}
profile2fetch4binary ['.pyc'] = 'ACK'                                                                              
profile2fetch4binary ['.exe'] = 'ACK' 
keys2profile4binary = profile2fetch4binary.keys()

kb2fetch4tpu = {}                                                                                                   # Program to be fetched - always define both source and binary pairs                               
kb2fetch4tpu[1] = folder2lib4tpu + 'tpu2sampler4xi5test.py'                                                         # This will be fetched if no parameters are given to create function
kb2fetch4tpu[2] = folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'
kb2fetch4tpu[3] = folder2lib4tpu + 'tpu2dummy4xi5test.cpp'                                                          # C++ 
kb2fetch4tpu[4] = folder2lib4tpu + 'bin//tpu2dummy4xi5test.exe'



# kb2fetch4tpu[1]  = folder2lib4tpu + 'tpu2sampler4xi5test.py'                                                      
# kb2fetch4tpu[2]  = folder2lib4tpu + '__pycache__//tpu2sampler4xi.cpython-33.pyc'                                  

# kb2fetch4tpu[3]  = folder2lib4tpu + 'tpu2image4xi5test.py'                                                        # Note for an unresolved reason Python compiler does NOT always generate cache file!
# kb2fetch4tpu[4]  = folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'
# kb2fetch4tpu[5]  = folder2lib4tpu + 'tpu3image4xi5test.py'
# kb2fetch4tpu[6]  = folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'
# kb2fetch4tpu[7]  = folder2lib4tpu + 'tpu2commandqueue4xi5test.py'
# kb2fetch4tpu[8]  = folder2lib4tpu + '__pycache__//tpu2commandqueue4xi.cpython-33.pyc'
# kb2fetch4tpu[9]  = folder2lib4tpu + 'tpu2event4xi5test.py'
# kb2fetch4tpu[10] = folder2lib4tpu + '__pycache__//tpu2event4xi.cpython-33.pyc'
# kb2fetch4tpu[11]  = folder2lib4tpu + 'tpu2kernel4xi5test.py'
# kb2fetch4tpu[12] = folder2lib4tpu + '__pycache__//tpu2kernel4xi.cpython-33.pyc'
# kb2fetch4tpu[13]  = folder2lib4tpu + 'tpu2dummy4xi5test.cpp'                                                      # C++ 
# kb2fetch4tpu[14] = folder2lib4tpu + 'bin//tpu2dummy4xi5test.exe'

fetch2code4source = {} 
fetch2code4binary = {} 

#"/>

# -------------------------------------------------------------------------------------------------
# Command queue module of TPU imports for target function                           
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2commandqueue4tpu" usage = " importing target function for Process4Xi">
def importcreate2commandqueue4tpu():
   global target2function4tpu
   from tpu2commandqueue4xi import create2commandqueue4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importflush2commandqueue4tpu" usage = " importing target function for Process4Xi">
def importflush2commandqueue4tpu():
   global target2function4tpu
   from tpu2commandqueue4xi import flush2commandqueue4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importfinish2commandqueue4tpu" usage = " importing target function for Process4Xi">
def importfinish2commandqueue4tpu():
   global target2function4tpu
   from tpu2commandqueue4xi import finish2commandqueue4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importget2commandqueueinfo4tpu" usage = " importing target function for Process4Xi">
def importget2commandqueueinfo4tpu():
   global target2function4tpu
   from tpu2commandqueue4xi import get2commandqueueinfo4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importrelease2commandqueue4tpu" usage = " importing target function for Process4Xi">
def importrelease2commandqueue4tpu():
   global target2function4tpu
   from tpu2commandqueue4xi import release2commandqueue4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importretain2commandqueue4tpu" usage = " importing target function for Process4Xi">
def importretain2commandqueue4tpu():
   global target2function4tpu
   from tpu2commandqueue4xi import retain2commandqueue4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importset2commandqueue4tpu" usage = " importing target function for Process4Xi">
def importset2commandqueue4tpu():
   global target2function4tpu
   from tpu2commandqueue4xi import set2commandqueue4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importtrace2commandqueue4tpu" usage = " importing target function for Process4Xi">
def importtrace2commandqueue4tpu():
   global target2function4tpu
   from tpu2commandqueue4xi import trace2commandqueue4tpu as target2function4tpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Context module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2context4tpu" usage = " importing target function for Process4Xi">
def importcreate2context4tpu():
   global target2function4tpu
   from tpu2context4xi import create2context4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importget2context4tpu" usage = " importing target function for Process4Xi">
def importget2context4tpu():
   global target2function4tpu
   from tpu2context4xi import get2context4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importrelease2context4tpu" usage = " importing target function for Process4Xi">
def importrelease2context4tpu():
   global target2function4tpu
   from tpu2context4xi import release2context4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importretain2context4tpu" usage = " importing target function for Process4Xi">
def importretain2context4tpu():
   global target2function4tpu
   from tpu2context4xi import retain2context4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importset2context4tpu" usage = " importing target function for Process4Xi">
def importset2context4tpu():
   global target2function4tpu
   from tpu2context4xi import set2context4tpu as target2function4tpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Device module of TPU imports for target function                           
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importinit2tpu4xi" usage = " importing target function for Process4Xi">
def importinit2tpu4xi():
   global target2function4tpu
   from tpu2device4xi import init2tpu4xi as target2function4tpu 
#</function4xi> 

#<function4xi function = "importset2tpu4xi" usage = " importing target function for Process4Xi">
def importset2tpu4xi():
   global target2function4tpu
   from tpu2device4xi import set2tpu4xi   as target2function4tpu
#</function4xi>
 
#<function4xi function = "importget2tpu4xi" usage = " importing target function for Process4Xi">
def importget2tpu4xi():
   global target2function4tpu
   from tpu2device4xi import get2tpu4xi   as target2function4tpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Event module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2event4tpu" usage = " importing target function for Process4Xi">
def importcreate2event4tpu():
   global target2function4tpu
   from tpu2event4xi import create2event4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importget2eventinfo4tpu" usage = " importing target function for Process4Xi">
def importget2eventinfo4tpu():
   global target2function4tpu
   from tpu2event4xi import get2eventinfo4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importrelease2event4tpu" usage = " importing target function for Process4Xi">
def importrelease2event4tpu():
   global target2function4tpu
   from tpu2event4xi import release2event4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importretain2event4tpu" usage = " importing target function for Process4Xi">
def importretain2event4tpu():
   global target2function4tpu
   from tpu2event4xi import retain2event4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importset2eventcallback4tpu" usage = " importing target function for Process4Xi">
def importset2eventcallback4tpu():
   global target2function4tpu
   from tpu2event4xi import set2eventcallback4tpu as target2function4tpu 
#</function4xi> 

#<function4xi function = "importset2event4tpu" usage = " importing target function for Process4Xi">
def importset2event4tpu():
   global target2function4tpu
   from tpu2event4xi import set2event4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importtrace2event4tpu" usage = " importing target function for Process4Xi">
def importtrace2event4tpu():
   global target2function4tpu
   from tpu2event4xi import trace2event4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importwait2event4tpu" usage = " importing target function for Process4Xi">
def importwait2event4tpu():
   global target2function4tpu
   from tpu2event4xi import wait2event4tpu as target2function4tpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# 2D Image module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2image4tpu" usage = " importing target function for Process4Xi">
def importcreate2image4tpu():
   global target2function4tpu
   from tpu2image4xi import create2image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importget2imageinfo4tpu" usage = " importing target function for Process4Xi">
def importget2imageinfo4tpu():
   global target2function4tpu
   from tpu2image4xi import get2imageinfo4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importget2supportedimageformats4tpu" usage = " importing target function for Process4Xi">
def importget2supportedimageformats4tpu():
   global target2function4tpu
   from tpu2image4xi import get2supportedimageformats4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importmanoeuvre2copyimage4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copyimage4tpu():
   global target2function4tpu
   from tpu2image4xi import manoeuvre2copyimage4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2copyimagetobuffer4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copyimagetobuffer4tpu():
   global target2function4tpu
   from tpu2image4xi import manoeuvre2copyimagetobuffer4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2copybuffertoimage4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copybuffertoimage4tpu():
   global target2function4tpu
   from tpu2image4xi import manoeuvre2copybuffertoimage4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importmanoeuvre2readimage4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2readimage4tpu():
   global target2function4tpu
   from tpu2image4xi import manoeuvre2readimage4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2writeimage4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2writeimage4tpu():
   global target2function4tpu
   from tpu2image4xi import manoeuvre2writeimage4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2mapimage4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2mapimage4tpu():
   global target2function4tpu
   from tpu2image4xi import manoeuvre2mapimage4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importcopy2image4tpu" usage = " importing target function for Process4Xi">
def importcopy2image4tpu():
   global target2function4tpu
   from tpu2image4xi import copy2image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importcopy2imagetobuffer4tpu" usage = " importing target function for Process4Xi">
def importcopy2imagetobuffer4tpu():
   global target2function4tpu
   from tpu2image4xi import copy2imagetobuffer4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importcopy2buffertoimage4tpu" usage = " importing target function for Process4Xi">
def importcopy2buffertoimage4tpu():
   global target2function4tpu
   from tpu2image4xi import copy2buffertoimage4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importread2image4tpu" usage = " importing target function for Process4Xi">
def importread2image4tpu():
   global target2function4tpu
   from tpu2image4xi import read2image4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importmap2image4tpu" usage = " importing target function for Process4Xi">
def importmap2image4tpu():
   global target2function4tpu
   from tpu2image4xi import map2image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importrelease2image4tpu" usage = " importing target function for Process4Xi">
def importrelease2image4tpu():
   global target2function4tpu
   from tpu2image4xi import release2image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importretain2image4tpu" usage = " importing target function for Process4Xi">
def importretain2image4tpu():
   global target2function4tpu
   from tpu2image4xi import retain2image4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importset2imagecallback4tpu" usage = " importing target function for Process4Xi">
def importset2imagecallback4tpu():
   global target2function4tpu
   from tpu2image4xi import set2imagecallback4tpu as target2function4tpu 
#</function4xi>
 
#<function4xi function = "importset2image4tpu" usage = " importing target function for Process4Xi">
def importset2image4tpu():
   global target2function4tpu
   from tpu2image4xi import set2image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importtrace2image4tpu" usage = " importing target function for Process4Xi">
def importtrace2image4tpu():
   global target2function4tpu
   from tpu2image4xi import trace2image4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importwait2image4tpu" usage = " importing target function for Process4Xi">
def importwait2image4tpu():
   global target2function4tpu
   from tpu2image4xi import wait2image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importwrite2image4tpu" usage = " importing target function for Process4Xi">
def importwrite2image4tpu():
   global target2function4tpu
   from tpu2image4xi import write2image4tpu as target2function4tpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# 3D Image module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate3image4tpu" usage = " importing target function for Process4Xi">
def importcreate3image4tpu():
   global target2function4tpu
   from tpu3image4xi import create3image4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importget3imageinfo4tpu" usage = " importing target function for Process4Xi">
def importget3imageinfo4tpu():
   global target2function4tpu
   from tpu3image4xi import get3imageinfo4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importget3supportedimageformats4tpu" usage = " importing target function for Process4Xi">
def importget3supportedimageformats4tpu():
   global target2function4tpu
   from tpu3image4xi import get3supportedimageformats4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3copyimage4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copyimage4tpu():
   global target2function4tpu
   from tpu3image4xi import manoeuvre3copyimage4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3copyimagetobuffer4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copyimagetobuffer4tpu():
   global target2function4tpu
   from tpu3image4xi import manoeuvre3copyimagetobuffer4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importmanoeuvre3copybuffertoimage4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copybuffertoimage4tpu():
   global target2function4tpu
   from tpu3image4xi import manoeuvre3copybuffertoimage4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3readimage4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3readimage4tpu():
   global target2function4tpu
   from tpu3image4xi import manoeuvre3readimage4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importmanoeuvre3writeimage4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3writeimage4tpu():
   global target2function4tpu
   from tpu3image4xi import manoeuvre3writeimage4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importmanoeuvre3mapimage4tpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3mapimage4tpu():
   global target2function4tpu
   from tpu3image4xi import manoeuvre3mapimage4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importcopy3image4tpu" usage = " importing target function for Process4Xi">
def importcopy3image4tpu():
   global target2function4tpu
   from tpu3image4xi import copy3image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importcopy3imagetobuffer4tpu" usage = " importing target function for Process4Xi">
def importcopy3imagetobuffer4tpu():
   global target2function4tpu
   from tpu3image4xi import copy3imagetobuffer4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importcopy3buffertoimage4tpu" usage = " importing target function for Process4Xi">
def importcopy3buffertoimage4tpu():
   global target2function4tpu
   from tpu3image4xi import copy3buffertoimage4tpu as target2function4tpu
#</function4xi>

#<function4xi function = "importread3image4tpu" usage = " importing target function for Process4Xi">
def importread3image4tpu():
   global target2function4tpu
   from tpu3image4xi import read3image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importmap3image4tpu" usage = " importing target function for Process4Xi">
def importmap3image4tpu():
   global target2function4tpu
   from tpu3image4xi import map3image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importrelease3image4tpu" usage = " importing target function for Process4Xi">
def importrelease3image4tpu():
   global target2function4tpu
   from tpu3image4xi import release3image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importretain3image4tpu" usage = " importing target function for Process4Xi">
def importretain3image4tpu():
   global target2function4tpu
   from tpu3image4xi import retain3image4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importset3imagecallback4tpu" usage = " importing target function for Process4Xi">
def importset3imagecallback4tpu():
   global target2function4tpu
   from tpu3image4xi import set3imagecallback4tpu as target2function4tpu 
#</function4xi> 

#<function4xi function = "importset3image4tpu" usage = " importing target function for Process4Xi">
def importset3image4tpu():
   global target2function4tpu
   from tpu3image4xi import set3image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importtrace3image4tpu" usage = " importing target function for Process4Xi">
def importtrace3image4tpu():
   global target2function4tpu
   from tpu3image4xi import trace3image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importwait3image4tpu" usage = " importing target function for Process4Xi">
def importwait3image4tpu():
   global target2function4tpu
   from tpu3image4xi import wait3image4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importwrite3image4tpu" usage = " importing target function for Process4Xi">
def importwrite3image4tpu():
   global target2function4tpu
   from tpu3image4xi import write3image4tpu as target2function4tpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Sampler module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2sampler4tpu" usage = " importing target function for Process4Xi">
def importcreate2sampler4tpu():
   global target2function4tpu
   from tpu2sampler4xi import create2sampler4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importget2samplerinfo4tpu" usage = " importing target function for Process4Xi">
def importget2samplerinfo4tpu():
   global target2function4tpu
   from tpu2sampler4xi import get2samplerinfo4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importrelease2sampler4tpu" usage = " importing target function for Process4Xi">
def importrelease2sampler4tpu():
   global target2function4tpu
   from tpu2sampler4xi import release2sampler4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importretain2sampler4tpu" usage = " importing target function for Process4Xi">
def importretain2sampler4tpu():
   global target2function4tpu
   from tpu2sampler4xi import retain2sampler4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importset2samplercallback4tpu" usage = " importing target function for Process4Xi">
def importset2samplercallback4tpu():
   global target2function4tpu
   from tpu2sampler4xi import set2samplercallback4tpu as target2function4tpu 
#</function4xi>
 
#<function4xi function = "importset2sampler4tpu" usage = " importing target function for Process4Xi">
def importset2sampler4tpu():
   global target2function4tpu
   from tpu2sampler4xi import set2sampler4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importtrace2sampler4tpu" usage = " importing target function for Process4Xi">
def importtrace2sampler4tpu():
   global target2function4tpu
   from tpu2sampler4xi import trace2sampler4tpu as target2function4tpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Kernel module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importadd2kernel4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importadd2kernel4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import add2kernel4tpu2kernel4tpu as target2function4tpu
#</function4xi>

#<function4xi function = "importexecute2kernel4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importexecute2kernel4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import execute2kernel4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importexecute2task4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importexecute2task4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import execute2task4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importexecute2nativekernel4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importexecute2nativekernel4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import execute2nativekernel4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importcreate2kernel4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importcreate2kernel4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import create2kernel4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importcreate2kernelsinprogram4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importcreate2kernelsinprogram4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import create2kernelsinprogram4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importdelete2kernel4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importdelete2kernel4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import delete2kernel4tpu2kernel4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importfetch2kernel4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importfetch2kernel4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import fetch2kernel4tpu2kernel4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importget2kernelinfo4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importget2kernelinfo4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import get2kernelinfo4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importget2kernelworkgroupinfo4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importget2kernelworkgroupinfo4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import get2kernelworkgroupinfo4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importget2kernelarginfo4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importget2kernelarginfo4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import get2kernelarginfo4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importrelease2kernel4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importrelease2kernel4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import release2kernel4tpu2kernel4tpu as target2function4tpu
#</function4xi> 

#<function4xi function = "importretain2kernel4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importretain2kernel4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import retain2kernel4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importset2kernel4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importset2kernel4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import set2kernel4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importset2kernelcallback4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importset2kernelcallback4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import set2kernelcallback4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importset2kernelarg4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importset2kernelarg4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import set2kernelarg4tpu2kernel4tpu as target2function4tpu
#</function4xi>
 
#<function4xi function = "importtrace2kernel4tpu2kernel4tpu" usage = " importing target function for Process4Xi">
def importtrace2kernel4tpu2kernel4tpu():
   global target2function4tpu
   from tpu2kernel4xi import trace2kernel4tpu2kernel4tpu as target2function4tpu
#</function4xi> 
 
# -------------------------------------------------------------------------------------------------
# Test modules of TPU imports for target function                             
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importtpu2commandqueue4xi5test" usage = " importing target function for Process4Xi">
def importtpu2commandqueue4xi5test():
   global target2function4tpu
   from tpu2commandqueue4xi5test import tpu2commandqueue4xi5test as target2function4tpu
#</function4xi>
 
#<function4xi function = "importtpu2event4xi5test" usage = " importing target function for Process4Xi">
def importtpu2event4xi5test():
   global target2function4tpu
   from tpu2event4xi5test import tpu2event4xi5test as target2function4tpu
#</function4xi>
 
#<function4xi function = "importtpu2image4xi5test" usage = " importing target function for Process4Xi">
def importtpu2image4xi5test():
   global target2function4tpu
   from tpu2image4xi5test import tpu2image4xi5test as target2function4tpu
#</function4xi> 

#<function4xi function = "importtpu3image4xi5test" usage = " importing target function for Process4Xi">
def importtpu3image4xi5test():
   global target2function4tpu
   from tpu3image4xi5test import tpu3image4xi5test as target2function4tpu
#</function4xi>
 
#<function4xi function = "importtpu2sampler4xi5test" usage = " importing target function for Process4Xi">
def importtpu2sampler4xi5test():
   global target2function4tpu
   from tpu2sampler4xi5test import tpu2sampler4xi5test as target2function4tpu
#</function4xi> 

#<function4xi function = "importtpu2kernel4xi5test" usage = " importing target function for Process4Xi">
def importtpu2kernel4xi5test():
   global target2function4tpu
   from tpu2kernel4xi5test import tpu2kernel4xi5test as target2function4tpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Error handler of TPU imports for target function                             
# -------------------------------------------------------------------------------------------------
#<function4xi function = "handle2error4tpu" usage = " Error handling for importing target function for Process4Xi">
def handle2error4tpu():
   global import2use4tpu
   print ("HANDLE2ERROR4TPU OF TPU2PROGRAM4XI.PY: *** FAILED TO IMPORT TARGET FUNCTION ***")
   keys2import4tpu = import2use4tpu.keys()
   for key2import4tpu in keys2import4tpu:
       print(key2import4tpu,import2use4tpu[key2import4tpu]) 
#</function4xi> 


import2use4tpu = {}
# -------------------------------------------------------------------------------------------------
# Command queue module of TPU imports for target function                           
# -------------------------------------------------------------------------------------------------
import2use4tpu["create2commandqueue4tpu"] = importcreate2commandqueue4tpu        
import2use4tpu["flush2commandqueue4tpu"] = importflush2commandqueue4tpu 
import2use4tpu["finish2commandqueue4tpu"] = importfinish2commandqueue4tpu 
import2use4tpu["get2commandqueueinfo4tpu"] = importget2commandqueueinfo4tpu 
import2use4tpu["release2commandqueue4tpu"] = importrelease2commandqueue4tpu 
import2use4tpu["retain2commandqueue4tpu"] = importretain2commandqueue4tpu 
import2use4tpu["set2commandqueue4tpu"] = importset2commandqueue4tpu 
import2use4tpu["trace2commandqueue4tpu"] = importtrace2commandqueue4tpu 

# -------------------------------------------------------------------------------------------------
# Context module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4tpu["create2context4tpu"] = importcreate2context4tpu 
import2use4tpu["get2context4tpu"] = importget2context4tpu 
import2use4tpu["release2context4tpu"] = importrelease2context4tpu 
import2use4tpu["retain2context4tpu"] = importretain2context4tpu 
import2use4tpu["set2context4tpu"] = importset2context4tpu 

# -------------------------------------------------------------------------------------------------
# Device module of TPU imports for target function                           
# -------------------------------------------------------------------------------------------------
import2use4tpu["init2tpu4xi"] = importinit2tpu4xi 
import2use4tpu["set2tpu4xi"] = importset2tpu4xi 
import2use4tpu["get2tpu4xi"] = importget2tpu4xi 

# -------------------------------------------------------------------------------------------------
# Event module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4tpu["create2event4tpu"] = importcreate2event4tpu 
import2use4tpu["get2eventinfo4tpu"] = importget2eventinfo4tpu 
import2use4tpu["release2event4tpu"] = importrelease2event4tpu 
import2use4tpu["retain2event4tpu"] = importretain2event4tpu 
import2use4tpu["set2eventcallback4tpu"] = importset2eventcallback4tpu 
import2use4tpu["set2event4tpu"] = importset2event4tpu 
import2use4tpu["trace2event4tpu"] = importtrace2event4tpu 
import2use4tpu["wait2event4tpu"] = importwait2event4tpu 

# -------------------------------------------------------------------------------------------------
# 2D Image module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4tpu["create2image4tpu"] = importcreate2image4tpu 
import2use4tpu["get2imageinfo4tpu"] = importget2imageinfo4tpu 
import2use4tpu["get2supportedimageformats4tpu"] = importget2supportedimageformats4tpu 
import2use4tpu["manoeuvre2copyimage4tpu"] = importmanoeuvre2copyimage4tpu 
import2use4tpu["manoeuvre2copyimagetobuffer4tpu"] = importmanoeuvre2copyimagetobuffer4tpu 
import2use4tpu["manoeuvre2copybuffertoimage4tpu"] = importmanoeuvre2copybuffertoimage4tpu 
import2use4tpu["manoeuvre2readimage4tpu"] = importmanoeuvre2readimage4tpu 
import2use4tpu["manoeuvre2writeimage4tpu"] = importmanoeuvre2writeimage4tpu 
import2use4tpu["manoeuvre2mapimage4tpu"] = importmanoeuvre2mapimage4tpu 
import2use4tpu["copy2image4tpu"] = importcopy2image4tpu 
import2use4tpu["copy2imagetobuffer4tpu"] = importcopy2imagetobuffer4tpu 
import2use4tpu["copy2buffertoimage4tpu"] = importcopy2buffertoimage4tpu 
import2use4tpu["read2image4tpu"] = importread2image4tpu 
import2use4tpu["map2image4tpu"] = importmap2image4tpu 
import2use4tpu["release2image4tpu"] = importrelease2image4tpu 
import2use4tpu["retain2image4tpu"] = importretain2image4tpu 
import2use4tpu["set2imagecallback4tpu"] = importset2imagecallback4tpu 
import2use4tpu["set2image4tpu"] = importset2image4tpu 
import2use4tpu["trace2image4tpu"] = importtrace2image4tpu 
import2use4tpu["wait2image4tpu"] = importwait2image4tpu 
import2use4tpu["write2image4tpu"] = importwrite2image4tpu 

# -------------------------------------------------------------------------------------------------
# 3D Image module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4tpu["create3image4tpu"] = importcreate3image4tpu 
import2use4tpu["get3imageinfo4tpu"] = importget3imageinfo4tpu 
import2use4tpu["get3supportedimageformats4tpu"] = importget3supportedimageformats4tpu 
import2use4tpu["manoeuvre3copyimage4tpu"] = importmanoeuvre3copyimage4tpu 
import2use4tpu["manoeuvre3copyimagetobuffer4tpu"] = importmanoeuvre3copyimagetobuffer4tpu 
import2use4tpu["manoeuvre3copybuffertoimage4tpu"] = importmanoeuvre3copybuffertoimage4tpu 
import2use4tpu["manoeuvre3readimage4tpu"] = importmanoeuvre3readimage4tpu 
import2use4tpu["manoeuvre3writeimage4tpu"] = importmanoeuvre3writeimage4tpu 
import2use4tpu["manoeuvre3mapimage4tpu"] = importmanoeuvre3mapimage4tpu 
import2use4tpu["copy3image4tpu"] = importcopy3image4tpu 
import2use4tpu["copy3imagetobuffer4tpu"] = importcopy3imagetobuffer4tpu 
import2use4tpu["copy3buffertoimage4tpu"] = importcopy3buffertoimage4tpu 
import2use4tpu["read3image4tpu"] = importread3image4tpu 
import2use4tpu["map3image4tpu"] = importmap3image4tpu 
import2use4tpu["release3image4tpu"] = importrelease3image4tpu 
import2use4tpu["retain3image4tpu"] = importretain3image4tpu 
import2use4tpu["set3imagecallback4tpu"] = importset3imagecallback4tpu 
import2use4tpu["set3image4tpu"] = importset3image4tpu 
import2use4tpu["trace3image4tpu"] = importtrace3image4tpu 
import2use4tpu["wait3image4tpu"] = importwait3image4tpu 
import2use4tpu["write3image4tpu"] = importwrite3image4tpu 

# -------------------------------------------------------------------------------------------------
# Sampler module of TPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4tpu["create2sampler4tpu"] = importcreate2sampler4tpu 
import2use4tpu["get2samplerinfo4tpu"] = importget2samplerinfo4tpu 
import2use4tpu["release2sampler4tpu"] = importrelease2sampler4tpu 
import2use4tpu["retain2sampler4tpu"] = importretain2sampler4tpu 
import2use4tpu["set2samplercallback4tpu"] = importset2samplercallback4tpu 
import2use4tpu["set2sampler4tpu"] = importset2sampler4tpu 
import2use4tpu["trace2sampler4tpu"] = importtrace2sampler4tpu 

# -------------------------------------------------------------------------------------------------
# Test modules of TPU imports for target function                             
# -------------------------------------------------------------------------------------------------
import2use4tpu["tpu2commandqueue4xi5test"] = importtpu2commandqueue4xi5test 
import2use4tpu["tpu2event4xi5test"] = importtpu2event4xi5test 
import2use4tpu["tpu2image4xi5test"] = importtpu2image4xi5test 
import2use4tpu["tpu3image4xi5test"] = importtpu3image4xi5test 
import2use4tpu["tpu2sampler4xi5test"] = importtpu2sampler4xi5test 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2program4tpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF TPU2PROGRAM4XI5TEST - TRYING TO TRACE TPU: ', time4xi())      
#   print('=' *33)      
#   trace2tpu = None
#   try:
#       trace2tpu = trace2program4tpu() 
#       print('(Step 15B/25) OF TPU2PROGRAM4XI5TEST - TRACE2TPU: ', trace2tpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2program4tpu-failed {}; time: 04:32:25 11/07/11 Pacific Standard Time}', 'CLASS': 'trace2program4tpu', 'METHOD': 'trace2program4tpu'} 
#   except:
#       print('(Step 15Z1/25) OF TPU2PROGRAM4XI5TEST *** FAILED TO TRACE TPU: ', trace2tpu) 
#      
#">
# -------------------------------------------------------------------------------------------------   
def trace2program4tpu(task4trace2tpu=None):  

    '''
    creates callback to tpu2sourceprogram4xi
    '''
    
    global tpu2trace4xi


    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2program4tpu" 
    method2print = "trace2program4tpu"

    message2print = "{(Step 1A/33) started" + "; trace2program4tpu-started: " + str(trace2program4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function trace2program4tpu at 0x0000000003A202C8> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2tpu-to-trace2program4tpu: " + str(task4trace2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # 

    code2ack  = ' '
    code2nack = ' '    
    
    tpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2tpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY TPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2tpu = {}                                                                                            
            
        task4trace2tpu['CLASS']   = "trace2program4tpu"                                                               # TPUIZE ZPU TRACE!
        task4trace2tpu['METHOD']  = "trace2program4tpu" 
        task4trace2tpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2program4tpu-failed " + str(tpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2tpu-set-by-trace2program4tpu to: " + str(task4trace2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY TPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2tpu-try-by-trace2program4tpu: " + str(task4trace2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    # 

    try:
        tpu2trace4xi ['CLASS']  = task4trace2tpu['CLASS']                                                      # TPUIZE TPU TRACE!
        tpu2trace4xi ['METHOD']  = task4trace2tpu['METHOD']
        tpu2trace4xi ['MESSAGE'] = task4trace2tpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; tpu2trace4xi-set-by-trace2program4tpu to: " + str(tpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; tpu-trace-failed-by-trace2program4tpu: " + str(task4trace2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE TPU TRACE
        # =================================
        class2print   = task4trace2tpu['CLASS']                                                                # TPUIZE TPU TRACE!
        method2print  = task4trace2tpu['METHOD']
        
        message2print = task4trace2tpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # TPUIZED TRACE FOR OPENCL CALLBACK!
        # 
        
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2program4tpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2program4tpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-tpu-trace-failed-by-trace2program4tpu: " + str(task4trace2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2program4tpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2program4tpu"                
        message2print = "{(Step 3D/33) performed" + "; tpu-trace-performed-by-trace2program4tpu: " + str(task4trace2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; tpu-trace-failed-by-trace2program4tpu: " + str(task4trace2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; tpu2trace4xi-returning-by-trace2program4tpu: " + str(tpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2program4tpu-failed {}; time: 04:40:23 11/16/11 Pacific Standard Time}', 
    #  'CLASS': 'trace2program4tpu', 
    #  'METHOD': 'trace2program4tpu'}; time: 04:40:23 11/16/11 Pacific Standard Time}

    
    return tpu2trace4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "unload2compiler4tpu" usage = "
#
#   response4unload2compiler = unload2compiler4tpu()
#      
#">
# -------------------------------------------------------------------------------------------------   
def unload2compiler4tpu(task4unload2tpu=None):  

    '''
    unloads compiler
    '''


            
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "unload2compiler4tpu" 
    method2print = "unload2compiler4tpu"

    message2print = "{(Step 1A/33) started" + "; unload2compiler4tpu-started: " + str(unload2compiler4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # 
  
          
    message2print = "{(Step 1B/33) passed" + "; task4unload2tpu-to-unload2compiler4tpu: " + str(task4unload2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                            # 
    # 
    status4unload2tpu = 'CL_SUCCESS'
    
    code2ack  = ' '
    code2nack = ' '    
    
                                                                                                                  #       
    if task4unload2tpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN - BUILD THEM
        # -----------------------------------------------------------------------------------------    
        task4unload2tpu = {}                                                                                            

        message2print = "{(Step 2A/33) set" + "; task4unload2tpu-set-by-unload2compiler4tpu to: " + str(task4unload2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # 
    # -----------------------------------------------------------------------------------------          
    status4unload2tpu = 'CL_SUCCESS'
    message2print = "{(Step 3A/33) set" + "; status4unload2tpu-set-by-unload2compiler4tpu to: " + str(status4unload2tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; status4unload2tpu-returning-by-unload2compiler4tpu: " + str(status4unload2tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    
    return status4unload2tpu
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2programcallback4tpu" usage = "
#
#   print('(Step 14A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO SET PROGRAM CALLBACK OF TPU ', time4xi())  
#   try:                               
#       response4setcallback2program = set2programcallback4tpu()                                                                                                 
#       print('(Step 14B/700) OF TPU2PROGRAM4XI5TEST - SET PROGRAM CALLBACK OF TPU TO: ', response4setcallback2program)   
#       # 
#   except:
#       print('(Step 14Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM CALLBACK OF TPU ', time4xi())  
# 
#">
# ------------------------------------------------------------------------------------------------- 
 
def set2programcallback4tpu(task4set2programwithsourcecallback=None):

    '''
    registers callback function to tpu2program4xi
    '''
    global tpu2program4xi

            
    class2print = "set2programcallback4tpu" 
    method2print = "set2programcallback4tpu"
    
    message2print = "{(Step 1A/70) started" + "; set2programcallback4tpu-started: " + str(set2programcallback4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # set2programcallback4tpu-started
    
    try:
    
        tpu2program4xi ['CALLBACK'] = trace2program4tpu
        
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-tpu2program4xi-returning-by-set2programcallback4tpu: " + str(tpu2program4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2program4tpu at 0x0000000003A985C8>
        
        return tpu2program4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2program-failed-by-set2programcallback4tpu: " + str(callback2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2program4tpu-returning-by-set2programcallback4tpu: " + str(trace2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2program4tpu
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "build2program4tpu" usage = "
#
#   task4build2program  = {}
#   task4build2program  ['PROGRAM'] = folder2lib4tpu + 'tpu2sampler4xi.py'
#   task4build2program  ['OPTIONS'] = {}
#   task4build2program  ['OPTIONS'] ['MATH']          = 'SINGLE-PRECISION-CONSTANT'
#   task4build2program  ['OPTIONS'] ['OPTIMIZATION']  = 'DISABLE'
#   task4build2program  ['OPTIONS'] ['VERSION']       = 'CL1.1'
#   task4build2program  ['OPTIONS'] ['WARNINGS']      = 'INHIBIT'
#   print('(Step 99A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO BUILD PROGRAM OF TPU DEVICE ', time4xi())    
#   # 
#   try:                               
#       response4build2program = build2program4tpu(task4build2program)
#       print('(Step 99B/700) OF TPU2PROGRAM4XI5TEST - BUILT PROGRAM OF TPU DEVICE ', time4xi())
#   except:
#       print('(Step 99Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO BUILD PROGRAM OF TPU DEVICE ', time4xi())  
#   try:                                                                                                                 
#       keys2response = response4build2program.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4build2program[key2response])
#           # ACK2RETURN  :   11A1-12A1-
#           # NACK2RETURN  :   13Z1-14Z1-
#           # OPTIONS  :  {'VERSION': 'CL1.1', 'OPTIMIZATION': 'DISABLE', 'MATH': 'SINGLE-PRECISION-CONSTANT', 'WARNINGS': 'INHIBIT'}
#           # PROGRAM  :  folder2lib4tpu + 'tpu2sampler4xi.py'
#       
#   except:
#       print('(Step 99Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4BUILD2PROGRAM ', response4build2program) 
#                 
#">
# -------------------------------------------------------------------------------------------------   
def build2program4tpu(task4build2tpu=None):

    '''
    building program to TPU 
    '''
    global tpu2build4xi

    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "build2program4tpu"
    method2print = "build2program4tpu" 

    message2print = "{(Step 1A/33) started" + "; build2program4tpu-started: " + str(build2program4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # build2program4tpu-started
    
    
    message2print = "{(Step 1B/33) passed" + "; task4build2tpu-to-build2program4tpu: " + str(task4build2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'PROGRAM': '...//tpu2sampler4xi.py', 
    #  'OPTIONS': {'VERSION': 'CL1.1', 
    #  'OPTIMIZATION': 'DISABLE', 
    #  'MATH': 'SINGLE-PRECISION-CONSTANT', 
    #  'WARNINGS': 'INHIBIT'}}
    # ; time: 07:57:00 11/18/11 Pacific Standard Time}

             
    code2ack  = ' '
    code2nack = ' '

    message2print = "{(Step 1A/70) try" + "; build-try-by-build2program4tpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    try:             
        if task4build2tpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO TPU! 
            # -----------------------------------------------------------------------------------------    
            task4build2tpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4build2tpu-created-by-build2program4tpu: " + str(task4build2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1) SOURCE PROGRAM - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            program2tpu = {}
            message2print = "{(Step 1A/70) trying" + "; program2tpu-trying-by-create2programwithsource4tpu: " + str(program2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                program2tpu = create2programwithsource4tpu()                                                                     
                
                message2print = "{(Step 1B/70) got" + "; program2tpu-got-by-create2programwithsource4tpu: " + str(program2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '1B-'         
            except:
                message2print = "{(Step 1Z/70) failed" + "; program2tpu-failed-by-create2programwithsource4tpu: " + str(program2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '1Z-'
                
            task4build2tpu ['PROGRAM'] = program2tpu
            
            # ---------------------------------------------------------------------------------------------
            # (2) OPTIONS - REQUIRED                         
            # --------------------------------------------------------------------------------------------- 
            task4build2program  ['OPTIONS'] = {}
            task4build2program  ['OPTIONS'] ['MATH']          = 'SINGLE-PRECISION-CONSTANT'
            task4build2program  ['OPTIONS'] ['OPTIMIZATION']  = 'DISABLE'
            task4build2program  ['OPTIONS'] ['VERSION']       = 'CL1.1'
            task4build2program  ['OPTIONS'] ['WARNINGS']      = 'INHIBIT'            
            
            # ---------------------------------------------------------------------------------------------
            # (3) CALLBACK - REQUIRED                         
            # --------------------------------------------------------------------------------------------- 
            function2callback4tpu = trace2program4tpu                                                               # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 3A/70) initialized" + "; function2callback4tpu-created-by-build2program4tpu: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '3A-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "build2program4tpu"                                                            # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "build2program4tpu"                                                            # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 3Z1/33) failed" + "; build2program4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 3B/70) initialized" + "; args2callback-created-by-build2program4tpu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '3B-'
            
           
            task4build2tpu['DATA2CALLBACK'] = args2callback                                                       # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4build2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                           # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '3C-'

            
            # ---------------------------------------------------------------------------------------------
            # (4) USERDATA - REQUIRED                         
            # ---------------------------------------------------------------------------------------------
            task4build2tpu['USERDATA'] = None 
            
            message2print = "{(Step 4A/70) initialized" + "; task4build2tpu-created-by-build2program4tpu: " + str(task4build2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
        else:
            pass                                                                  
    except:
        message2print = "{(Step 1Z1/70) yielded" + "; task4build2tpu-yielded-by-build2program4tpu: " + str(task4build2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-'  
                                                                                                 
    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4build2tpu.keys()        
    message2print = "{(Step 9/70) using" + "; keys2task-using-by-build2program4tpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['OPTIONS', 'PROGRAM']
  
    # ---------------------------------------------------------------------------------------------
    # (Step 10A/70) - CREATE BUILD FOR TPU ! 
    # ---------------------------------------------------------------------------------------------    
    tpu2build4xi = {}                                                                                                                                                                               
    message2print = "{(Step 10A/70) initialized" + "; tpu2build4xi-initialized-by-build2program4tpu to: " + str(tpu2build4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}
    
    message2print = "{(Step 11A0/70) try" + "; program-try-by-build2program4tpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 11A/70) - PROGRAM - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        tpu2build4xi['PROGRAM'] = task4build2tpu['PROGRAM']
        
        message2print = "{(Step 11A1/70) initialized" + "; program-created-by-build2program4tpu: " + str(tpu2build4xi['PROGRAM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # ...//tpu2sampler4xi.py
        
        code2ack  = code2ack + '11A1-'
    except: 
        message2print = "{(Step 11Z1/70) yielded" + "; program-tpu2build4xi-yielded-by-build2program4tpu: " + str(tpu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '11Z1-' 
           
    message2print = "{(Step 12A0/70) try" + "; options-try-by-build2program4tpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 12A/70) - OPTIONS - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        tpu2build4xi['OPTIONS'] = task4build2tpu['OPTIONS']
        
        message2print = "{(Step 12A1/70) initialized" + "; options-created-by-build2program4tpu: " + str(tpu2build4xi['OPTIONS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'VERSION': 'CL1.1', 'OPTIMIZATION': 'DISABLE', 'MATH': 'SINGLE-PRECISION-CONSTANT', 'WARNINGS': 'INHIBIT'}
        
        code2ack  = code2ack + '12A1-'
    except: 
        message2print = "{(Step 12Z1/70) yielded" + "; options-tpu2build4xi-yielded-by-build2program4tpu: " + str(tpu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '12Z1-' 
        
    message2print = "{(Step 13A0/70) try" + "; callback-try-by-build2program4tpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 13A/70) - CALLBACK - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        tpu2build4xi['CALLBACK'] = task4build2tpu['CALLBACK']
        
        message2print = "{(Step 13A1/70) initialized" + "; callback-created-by-build2program4tpu: " + str(tpu2build4xi['CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '13A1-'
    except: 
        message2print = "{(Step 13Z1/70) yielded" + "; callback-tpu2build4xi-yielded-by-build2program4tpu: " + str(tpu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '13Z1-' 
        
    message2print = "{(Step 14A0/70) try" + "; userdata-try-by-build2program4tpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 14A/70) - USERDATA - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        tpu2build4xi['USERDATA'] = task4build2tpu['USERDATA']
        
        message2print = "{(Step 14A1/70) initialized" + "; userdata-created-by-build2program4tpu: " + str(tpu2build4xi['USERDATA'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '14A1-'
    except: 
        message2print = "{(Step 14Z1/70) yielded" + "; userdata-tpu2build4xi-yielded-by-build2program4tpu: " + str(tpu2build4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '14Z1-'
                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2build4xi ['ACK2RETURN']  = code2ack
    tpu2build4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; tpu2build4xi-returning-by-build2program4tpu: " + str(tpu2build4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'ACK2RETURN': ' 11A1-12A1-', 
    #  'PROGRAM': '...//tpu2sampler4xi.py', 
    #  'NACK2RETURN': ' 13Z1-14Z1-', 
    #  'OPTIONS': {'VERSION': 'CL1.1', 
    #  'OPTIMIZATION': 'DISABLE', 
    #  'MATH': 'SINGLE-PRECISION-CONSTANT', 
    #  'WARNINGS': 'INHIBIT'}
    # }; time: 07:57:00 11/18/11 Pacific Standard Time}

    
    return tpu2build4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2programwithsource4tpu" usage = "
#     
#   print('(Step 2A1/700) OF TPU2PROGRAM4XI5TEST - TRYING TO CREATE PROGRAM SOURCE FOR TPU ', time4xi())       
#   try:
#       response4create2programwithsource = create2programwithsource4tpu()
#   except:
#       print('(Step 2Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM SOURCE FOR TPU *** ', time4xi())
#   try:                                                                                      
#       print('COUNT4SOURCE: ', response4create2programwithsource['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE: ', response4create2programwithsource['DIRECTORY4SOURCE'])
#       print('LANGUAGE4SOURCE: ', response4create2programwithsource['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE: ', response4create2programwithsource['SIZE4SOURCE'])
#       keys2source = response4create2programwithsource['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [1])
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [3])
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [4])
#           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [5])
#           
#       # COUNT4SOURCE:  1
#       # DIRECTORY4SOURCE:  folder2lib4tpu
#       # LANGUAGE4SOURCE:  Python
#       # SIZE4SOURCE:  {folder2lib4tpu + 'tpu2sampler4xi5test.py': 1337}
#       # SOURCE:  folder2lib4tpu + 'tpu2sampler4xi5test.py'
#       # ... profile = "11" />
#       # ... program = "tpu2sampler4xi5test.py">
#       # ... abstract = "tests tpu2sampler4xi.py"/>
#       # ... verification = "11" target = "tpu2sampler4xi.py"/>                
#   except:
#       print('(Step 2Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4create2programwithsource *** ', response4create2programwithsource)
#          
#">
# -------------------------------------------------------------------------------------------------   
def create2programwithsource4tpu(task4program2tpu=None):

    '''
    creates program for TPU
    '''
    global tpu2program4xi                                                                                           # Program of TPU    

    global dir2source4tpu                                                                                           # Directory for source files      


    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2programwithsource4tpu" 
    method2print = "create2programwithsource4tpu"

    message2print = "{(Step 1A/70) started" + "; create2programwithsource4tpu-started: " + str(create2programwithsource4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2programwithsource4tpu at 0x00000000040D77C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2tpu-to-create2programwithsource4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    tpu2program4xi = {}                                                                                              # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4program2tpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO TPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4program2tpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4program2tpu-created-by-create2programwithsource4tpu: " + str(task4program2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4tpu = trace2program4tpu                                                               # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4tpu-created-by-create2programwithsource4tpu: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2program4tpu at 0x0000000003AB12C8>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2programwithsource4tpu"                                               # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2programwithsource4tpu"                                               # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2programwithsource4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2programwithsource4tpu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4tpu-failed ; time: 06:57:23 11/15/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithsource4tpu', 
            #  'METHOD': 'create2programwithsource4tpu'}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4program2tpu['DATA2CALLBACK'] = args2callback                                                       # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4program2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                           # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4program2tpu-created-by-create2programwithsource4tpu: " + str(task4program2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2program4tpu at 0x0000000003AB12C8>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4tpu-failed ; time: 06:57:23 11/15/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithsource4tpu', 
            #  'METHOD': 'create2programwithsource4tpu'}}; time: 06:57:23 11/15/11 Pacific Standard Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2programwithsource4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2programwithsource4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 06:57:23 11/15/11 Pacific Standard Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2programwithsource4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4program2tpu['PLATFORM'] = platform2xi                                                              # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2tpu-trying-by-create2programwithsource4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2tpu-got-by-create2programwithsource4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ... 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2tpu-failed-by-create2programwithsource4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4program2tpu ['CONTEXT'] = context2tpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2tpu-trying-by-create2programwithsource4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2tpu-got-by-create2programwithsource4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2tpu-failed-by-create2programwithsource4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4program2tpu ['DEVICE'] = device2tpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4program2tpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2tpu-trying-by-create2programwithsource4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2tpu-got-by-create2programwithsource4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003ACCF98>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2tpu-failed-by-create2programwithsource4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4program2tpu ['QUEUE'] = queue2tpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2tpu-trying-by-create2programwithsource4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2tpu-got-by-create2programwithsource4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2tpu-failed-by-create2programwithsource4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4program2tpu ['PIPE'] = pipe2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2tpu-trying-by-create2programwithsource4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2tpu-got-by-create2programwithsource4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ... 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2tpu-failed-by-create2programwithsource4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4program2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2tpu-trying-by-create2programwithsource4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                               # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2tpu-got-by-create2programwithsource4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2tpu-failed-by-create2programwithsource4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4program2tpu ['BUFFER'] = bufferinfo2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2tpu-trying-by-create2programwithsource4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                 # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2tpu-got-by-create2programwithsource4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2tpu-failed-by-create2programwithsource4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4program2tpu ['EVENT'] = eventinfo2tpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) COUNT - REQUIRED                     
            # ---------------------------------------------------------------------------------------------             
            task4program2tpu ['COUNT'] = 0
            message2print = "{(Step 11A/70) trying" + "; count2tpu-trying-by-create2programwithsource4tpu: " + str(task4program2tpu['COUNT'])  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:            
                task4program2tpu ['COUNT'] = int(len(kb2fetch4tpu)/2)                                               # NUMBER OF SOURCE FILES TO BE READ AND EMBEDDED
                message2print = "{(Step 11B/70) set" + "; count2tpu-set-by-create2programwithsource4tpu to: " + str(task4program2tpu['COUNT'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 1                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; count2tpu-failed-by-create2programwithsource4tpu: " + str(task4program2tpu['COUNT']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
                                               
            # ---------------------------------------------------------------------------------------------
            # (12) DIRECTORY - REQUIRED - COMMON TO ALL KIND OF SOURCE FILES (Python, C++, C ...)                 
            # ---------------------------------------------------------------------------------------------             
            dir2tpu = dir2source4tpu                                                                                                                                                          
            message2print = "{(Step 12B/70) set" + "; dir2tpu-set-by-create2programwithsource4tpu to: " + str(dir2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #  
            code2ack  = code2ack + '12B-' 
        
            task4program2tpu ['DIRECTORY'] = dir2tpu
            
                        
            # ---------------------------------------------------------------------------------------------
            # (14) CODE AND SIZE - REQUIRED                 
            # ---------------------------------------------------------------------------------------------             
            message2print = "{(Step 14A/70) using" + "; DirectoryWalker-using-by-create2programwithsource4tpu: " + str(DirectoryWalker)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # <class 'tpu2program4xi.DirectoryWalker'>
            
            for file2walk in DirectoryWalker(dir2tpu):
                if os.path.isdir(file2walk):
                    message2print = "{(Step 14B/70) examining" + "; directory-examining-by-create2programwithsource4tpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                if os.path.isfile(file2walk):
                    message2print = "{(Step 14C/70) examining" + "; file-examining-by-create2programwithsource4tpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #folder2lib4tpu +  tpu2buffer4xi.py
                    
                    message2print = "{(Step 14D/70) calling" + "; fetch2programsource4tpu-calling-by-create2programwithsource4tpu: " + str(fetch2programsource4tpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # <function fetch2programsource4tpu at 0x0000000003FE18C8>
                    
                    message2print = "{(Step 14E/70) passing" + "; file2walk-passing-to-fetch2programsource4tpu-by-create2programwithsource4tpu: " + str(file2walk) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #folder2lib4tpu +  tpu2buffer4xi.py
                    
                    fetch2programsource4tpu(file2walk)                                                              # fetches program files into fetch2code4source dictionary 
                    
                    message2print = "{(Step 14E/70) performed" + "; fetch2programsource4tpu-performed-by-create2programwithsource4tpu: " + str(fetch2programsource4tpu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # 
                    
                    message2print = "{(Step 14F/70) got" + "; fetch2code4source-got-by-create2programwithsource4tpu: " + str(fetch2code4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # 
                                        
            task4program2tpu ['CODE'] = fetch2code4source                                                           # dictionary of source code(s)      
                    
            size2source4tpu  = {}
            keys2source4tpu = fetch2code4source.keys()                                                              # tpu2sampler4xi5test.py
            for key2source4tpu in keys2source4tpu:
                size2source4tpu[key2source4tpu] = len(fetch2code4source[key2source4tpu])                            # length of saved source   
                       
            message2print = "{(Step 14B/70) set" + "; size2source4tpu-set-by-create2programwithsource4tpu to: " + str(size2source4tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # {folder2lib4tpu + 'tpu2sampler4xi5test.py': 1337}
            code2ack  = code2ack + '14B-' 
        
            task4program2tpu ['SIZE'] = size2source4tpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) LANGUAGE - EXTRA                             
            # ---------------------------------------------------------------------------------------------             
            language2tpu  = "Python"  # C++; C
           
            message2print = "{(Step 15B/70) set" + "; language2tpu-set-by-create2programwithsource4tpu to: " + str(language2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '15B-' 
        
            task4program2tpu ['LANGUAGE'] = language2tpu     

                                    
            # ---------------------------------------------------------------------------------------------
            # (16) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            count2reference2tpu  = 1
           
            message2print = "{(Step 17B/70) set" + "; count2reference2tpu-set-by-create2programwithbinary4tpu to: " + str(count2reference2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '17B-' 
        
            task4program2tpu ['COUNT2REFERENCE'] = count2reference2tpu                                 
                                                                                                                      
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2tpu-yielded-by-create2programwithsource4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4program2tpu.keys()         
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2programwithsource4tpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'COUNT',
    #  'COUNT2REFERENCE',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'DIRECTORY',
    #  'EVENT',
    #  'FUNCTION2CALLBACK',
    #  'LANGUAGE',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'SIZE']
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE PROGRAM OBJECT FOR TPU ! 
    # ---------------------------------------------------------------------------------------------    
    tpu2program4xi = {}                                                                                             # CREATE INITIALIZES DICTIONARY!                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; tpu2program4xi-initialized-by-create2programwithsource4tpu to: " + str(tpu2program4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 

    if 'DATA2CALLBACK' in keys2task:    
        message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2programwithsource4tpu: {}" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 21A/70) - DATA2CALLBACK - EXTRA 
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['DATA2CALLBACK'] = task4program2tpu['DATA2CALLBACK']
            
            message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4tpu-failed ; time: 01:21:23 11/16/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithsource4tpu', 
            #  'METHOD': 'create2programwithsource4tpu'}; time: 01:21:23 11/16/11 Pacific Standard Time}
     
            
            code2ack  = code2ack + '21A1-'
        except: 
            message2print = "{(Step 21Z1/70) yielded" + "; data2callback-tpu2program4xi-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '21Z1-'

    if 'FUNCTION2CALLBACK' in keys2task:     
        message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['FUNCTION2CALLBACK'] = task4program2tpu['FUNCTION2CALLBACK']
            
            message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function trace2program4tpu at 0x000000000392E2C8>
            
            code2ack  = code2ack + '22B1-'
        except: 
            message2print = "{(Step 22Z2/70) yielded" + "; function2callback-tpu2program4xi-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z2-' 

    if 'PLATFORM' in keys2task:     
        message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22C/70) - PLATFORM - EXTRA
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['PLATFORM'] = task4program2tpu['PLATFORM']
            
            message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 01:21:23 11/16/11 Pacific Standard Time}
    
            code2ack  = code2ack + '22C1-'
        except: 
            message2print = "{(Step 22Z3/70) yielded" + "; platform-tpu2program4xi-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z3-' 
        

    if 'CONTEXT' in keys2task:         
          
        message2print = "{(Step 23A/70) try" + "; context2tpu-try-by-create2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 23/70) - CONTEXT - REQUIRED
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['CONTEXT'] = task4program2tpu['CONTEXT']
            
            message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...        
            
            code2ack  = code2ack + '23B-'
        except: 
            message2print = "{(Step 23Z1/70) yielded" + "; context-tpu2program4xi-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
        
            code2nack  = code2nack + '23Z1-' 

    if 'DEVICE' in keys2task:         
        message2print = "{(Step 24A/70) try" + "; device-try-by-create2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 24/70) - DEVICE - EXTRA
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['DEVICE'] = task4program2tpu['DEVICE']
            
            message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ... 
            
            code2ack  = code2ack + '24B-'
        except: 
            message2print = "{(Step 24Z1/70) yielded" + "; device-tpu2program4xi-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '24Z1-'

    if 'CODE2ERROR' in keys2task:                  
        message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 25/70) - ERROR CODE - REQUIRED
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['CODE2ERROR'] = task4program2tpu['CODE2ERROR']
            
            message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 0 
            
            code2ack  = code2ack + '25B-'
        except: 
            message2print = "{(Step 25Z1/70) yielded" + "; code2error-tpu2program4xi-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '25Z1-'                

    if 'QUEUE' in keys2task:                 
              
        message2print = "{(Step 26A/70) try" + "; queue-try-by-create2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 26/70) - QUEUE - EXTRA
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['QUEUE'] = task4program2tpu['QUEUE']
            
            message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <queue4xi.Queue4Xi object at 0x0000000003EEC978> 
            
            code2ack  = code2ack + '26B-'
        except: 
            message2print = "{(Step 26Z1/70) yielded" + "; queue-tpu2program4xi-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '26Z1-' 

    if 'PIPE' in keys2task:                                 
        message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                     
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 27/70) - PIPE - EXTRA
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['PIPE'] = task4program2tpu['PIPE']
            
            message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # (<read-write PipeConnection, handle 384>, <read-write PipeConnection, handle 388>) 
            
            code2ack  = code2ack + '27B-'
        except: 
            message2print = "{(Step 27Z1/70) yielded" + "; pipe-tpu2program4xi-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '27Z1-'   

    if 'COMMANDQUEUE' in keys2task: 
        message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            tpu2program4xi['COMMANDQUEUE']  = task4program2tpu['COMMANDQUEUE']  
     
            message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, ... 
                    
            code2ack  = code2ack + '28B-'
        except: 
            message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                  
            code2nack  = code2nack + '28Z1-'    

    if 'BUFFER' in keys2task: 
        message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            tpu2program4xi['BUFFER']  = task4program2tpu['BUFFER']  
     
            message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
            
            
            code2ack  = code2ack + '29B-'
        except: 
            message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '29Z1-'            

    if 'EVENT' in keys2task: 
        message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            tpu2program4xi['EVENT']  = task4program2tpu['EVENT']  
     
            message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK':  ... 
            
            code2ack  = code2ack + '30B-'
        except: 
            message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '30Z1-'            

    if 'COUNT' in keys2task:                 
        message2print = "{(Step 31A1/70) try" + "; task4program2tpu-try-by-create2programwithsource4tpu: " + str(task4program2tpu)  + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # {'COUNT': 1, 'CODE2ERROR': 0 ... 
        message2print = "{(Step 31A2/70) try" + "; count-try-by-create2programwithsource4tpu: " + str(task4program2tpu['COUNT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1 
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            tpu2program4xi['COUNT4SOURCE']  = task4program2tpu['COUNT']  
    
            message2print = "{(Step 31B/70) initialized" + "; count-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['COUNT4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '31B-'
        except: 
            message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z1-'     

    if 'DIRECTORY' in keys2task: 
        message2print = "{(Step 32A/70) try" + "; directory-try-by-create2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            tpu2program4xi['DIRECTORY4SOURCE']  = task4program2tpu['DIRECTORY']  
     
            message2print = "{(Step 32B/70) initialized" + "; directory-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['DIRECTORY4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #
            code2ack  = code2ack + '32B-'
        except: 
            message2print = "{(Step 32Z1/70) yielded" + "; DIRECTORY-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z1-'  

    if 'CODE' in keys2task:         
        message2print = "{(Step 33A/70) try" + "; codetask-try-by-create2programwithsource4tpu: " + str(task4program2tpu ['CODE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # {folder2lib4tpu + 'tpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: ...
        
        tpu2program4xi['CODE4SOURCE'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            tpu2program4xi['CODE4SOURCE'] = task4program2tpu ['CODE']
    
     
            message2print = "{(Step 33B/70) created" + "; code-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['CODE4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4tpu + 'tpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2:
             
            code2ack  = code2ack + '33B-'
        except: 
            message2print = "{(Step 33Z1/70) yielded" + "; source-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi['CODE4SOURCE']) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '33Z1-'     

    if 'SIZE' in keys2task:         
        message2print = "{(Step 34A/70) try" + "; size-try-by-create2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34) SIZE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            tpu2program4xi['SIZE4SOURCE']  = task4program2tpu['SIZE']  
     
            message2print = "{(Step 34B/70) initialized" + "; size-initialized-by-create2programwithsource4tpu to: " + str(tpu2program4xi['SIZE4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4tpu + 'tpu2sampler4xi5test.py': 1337}
            code2ack  = code2ack + '34B-'
        except: 
            message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z1-'     

    if 'LANGUAGE' in keys2task:         
        message2print = "{(Step 35A/70) try" + "; language-try-by-create2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (35) LANGUAGE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            tpu2program4xi['LANGUAGE4SOURCE']  = task4program2tpu['LANGUAGE']  
     
            message2print = "{(Step 35B/70) initialized" + "; language-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['LANGUAGE4SOURCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '35B-'
        except: 
            message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '35Z1-'     

    if 'COUNT2REFERENCE' in keys2task:                 
        message2print = "{(Step 36A/70) try" + "; count2reference-try-by-create2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (36) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
                            
            tpu2program4xi['COUNT2REFERENCE']  = task4program2tpu['COUNT2REFERENCE']  
     
            message2print = "{(Step 36B/70) initialized" + "; count2reference-created-by-create2programwithsource4tpu: " + str(tpu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '36B-'
        except: 
            message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-create2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '36Z1-'     
        
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)     
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack  
    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70/70) returning" + "; key2program4tpu-returning-by-create2programwithsource4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # COUNT4SOURCE
        # SIZE4SOURCE
        # DATA2CALLBACK
        # BUFFER
        # FUNCTION2CALLBACK
        # ACK2RETURN
        # DIRECTORY4SOURCE
        # QUEUE
        # PLATFORM
        # COUNT2REFERENCE
        # NACK2RETURN
        # CONTEXT
        # COMMANDQUEUE
        # DEVICE
        # CODE4SOURCE
        # EVENT
        # PIPE

    return tpu2program4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "add2programwithbinary4tpu" usage = "
#     
#   print('(Step 3A2/700) OF TPU2PROGRAM4XI5TEST - TRYING TO ADD BINARY INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4add2program = {}
#       task4add2program ['CODE4BINARY'] = {}
#       # NOT OK - EXISTS
#       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'] = {}
#       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
#       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
#       task4add2program ['SIZE4BINARY'] = {}
#       task4add2program ['SIZE4BINARY'] [folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'])
#                
#       # OK
#       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] = {}
#       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
#       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
#       task4add2program ['SIZE4BINARY'] = {}
#       task4add2program ['SIZE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'])
#       
#       response4add2programwithbinary = add2programwithbinary4tpu(task4add2program)
#       print('(Step 3A3/700) OF TPU2PROGRAM4XI5TEST - ADDED BINARY INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 3Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4add2programwithbinary: ', response4add2programwithbinary)
#   try:                                                                                          
#       keys2response = response4add2programwithbinary.keys()
#       print('COUNT2REFERENCE:', response4add2programwithbinary ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4add2programwithbinary ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4add2programwithbinary ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4add2programwithbinary ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4add2programwithbinary ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4add2programwithbinary ['SIZE4BINARY'])                
#       # 
#       keys2source = response4add2programwithbinary['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4add2programwithbinary ['CODE4BINARY'] [key2source] [1])
#           # 
#           # COUNT2REFERENCE: 1
#           # COUNT4BINARY: 2
#           # DIRECTORY4BINARY: ...//__pycache__//
#           # FUNCTION2CALLBACK: <function trace2program4tpu at 0x0000000003F96AC8>
#           # LANGUAGE4BINARY: Python
#           # SIZE4BINARY: {'...//__pycache__//tpu2image4xi.cpython-33.pyc': 1006, '...//__pycache__//tpu3image4xi.cpython-33.pyc': 2}
#           # BINARY:  ...//__pycache__//tpu2image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'
#           # BINARY:  ...//__pycache__//tpu3image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'                       
#   except:
#       print('(Step 3Z3/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4add2programwithbinary : ', response4add2programwithbinary)
#          
#">
# -------------------------------------------------------------------------------------------------   
def add2programwithbinary4tpu(task4program2tpu):

    '''
    adds program for TPU
    '''
    global tpu2program4xi                                                                                           # Program of TPU    
    
  
    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "add2programwithbinary4tpu" 
    method2print = "add2programwithbinary4tpu"

    message2print = "{(Step 10A/70) started" + "; add2programwithbinary4tpu-started: " + str(add2programwithbinary4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function add2programwithbinary4tpu at 0x000000000418DC48>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2tpu-to-add2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE4BINARY': {folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}, 
    #                  folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}}, 
    #  'SIZE4BINARY': {folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc': 2}
    # }
    
    code2ack  = ' '
    code2nack = ' ' 
    try:
        keys2binary4task = task4program2tpu ['CODE4BINARY'].keys()
        message2print = "{(Step 10C/70) using" + "; keys2binary4task-using-by-add2programwithbinary4tpu: " + str(keys2binary4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # //__pycache__//tpu2image4xi.cpython-33.pyc
        # //__pycache__//tpu3image4xi.cpython-33.pyc          
    except: 
        message2print = "{(Step 10Z1/70) failed" + "; keys2binary4task-failed-by-add2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
    try:
        keys2exist4task =  tpu2program4xi ['CODE4BINARY'].keys() 
        message2print = "{(Step 10D/70) using" + "; keys2exist4task-using-by-add2programwithbinary4tpu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    except:
        try:
            response2createwb4tpu = create2programwithbinary4tpu()
            keys2createwb4tpu = response2createwb4tpu.keys()
            for key2createwb4tpu in keys2createwb4tpu:
                if key2createwb4tpu == 'CODE4BINARY':
                    message2print = "{(Step 10E/70) created" + "; key2createwb4tpu-created-by-add2programwithbinary4tpu: " + str(key2createwb4tpu) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)
                    # {'... //__pycache__//tpu2image4xi.cpython-33.pyc'
            
            keys2exist4task =  tpu2program4xi ['CODE4BINARY'].keys() 
            message2print = "{(Step 10F/70) using" + "; keys2exist4task-using-by-add2programwithbinary4tpu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # //__pycache__//tpu2image4xi.cpython-33.pyc'
        
        except: 
            message2print = "{(Step 10Z2/70) failed" + "; tpu2program4xi-failed-by-add2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - EXPECTED - SEVERAL BINARY CODE CAN BE ADDED INTO ONE PROGRAM OBJECT                              
        # ---------------------------------------------------------------------------------------------             
        for key2binary4task in keys2binary4task:
            try:
                message2print = "{(Step 33A/70) using" + "; key2binary4task-using-by-add2programwithbinary4tpu: " + str(key2binary4task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # //__pycache__//tpu2image4xi.cpython-33.pyc
                            
                if key2binary4task in keys2exist4task:
                    message2print = "{(Step 33Z0/70) yielded" + "; key2binary4task-yielded-by-add2programwithbinary4tpu to: " + str(key2binary4task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #folder2lib4tpu +  __pycache__//tpu2image4xi.cpython-33.pyc
                     
                    code2nack  = code2nack + '33Z0-'
                else: 
                    try:
                        tpu2program4xi ['CODE4BINARY'] [key2binary4task] = task4program2tpu ['CODE4BINARY'] [key2binary4task]
                    except:
                        message2print = "{(Step 33Z1/70) failed" + "; task4program2tpu-failed-by-add2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z1-' 
                    try:             
                        message2print = "{(Step 33B/70) added" + "; key2binary4task-added-by-add2programwithbinary4tpu to: " + str(key2binary4task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        #folder2lib4tpu +  __pycache__//tpu3image4xi.cpython-33.pyc
                    except:
                        message2print = "{(Step 33Z2/70) failed" + "; task4program2tpu-failed-by-add2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z2-' 
                    try:             
                        message2print = "{(Step 33C/70) added" + "; code-added-by-add2programwithbinary4tpu to: " + str(tpu2program4xi ['CODE4BINARY'] [key2binary4task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}
                    except:
                        message2print = "{(Step 33Z3/70) failed" + "; task4program2tpu-failed-by-add2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        code2nack  = code2nack + '33Z3-'                 
                    try:
                        tpu2program4xi ['COUNT4BINARY'] = tpu2program4xi ['COUNT4BINARY'] + 1
                        message2print = "{(Step 33D/70) added" + "; count4binary-added-by-add2programwithbinary4tpu to: " + str(tpu2program4xi ['COUNT4BINARY'])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 2 
                    except:
                        message2print = "{(Step 33Z4/70) failed" + "; task4program2tpu-failed-by-add2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z4-' 
                    try:                            
                        tpu2program4xi ['SIZE4BINARY'] [key2binary4task]  = task4program2tpu['SIZE4BINARY'] [key2binary4task]  
                        message2print = "{(Step 33E/70) set" + "; size4binary-set-by-add2programwithbinary4tpu to: " + str(tpu2program4xi ['SIZE4BINARY'] [key2binary4task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 2
                        code2ack  = code2ack + '33D-'
                    except:
                        message2print = "{(Step 33Z5/70) failed" + "; task4program2tpu-failed-by-add2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z5-'    
            except:
                message2print = "{(Step 33Z6/70) failed" + "; task4program2tpu-failed-by-add2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                code2nack  = code2nack + '33Z6-'                     
    except: 
        message2print = "{(Step 33Z7/70) yielded" + "; binary-yielded-by-add2programwithbinary4tpu: " + str(tpu2program4xi ['CODE4BINARY']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '33Z7-'       
          
                                                                                                                   
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2binaryprograms = tpu2program4xi ['CODE4BINARY'].keys()
    for key2binaryprogram in sorted(keys2binaryprograms):
        message2print = "{(Step 70A/70) got" + "; key2binaryprogram-got-by-add2programwithbinary4tpu: " + str(key2binaryprogram) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        #folder2lib4tpu +  __pycache__//tpu2image4xi.cpython-33.pyc
        #folder2lib4tpu +  __pycache__//tpu3image4xi.cpython-33.pyc

    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70B/70) returning" + "; key2program4tpu-returning-by-add2programwithbinary4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
        
    return tpu2program4xi
            
#</function4xi>



# -------------------------------------------------------------------------------------------------
#<function4xi function = "add2programwithsource4tpu" usage = "
#     
#   print('(Step 2A3/700) OF TPU2PROGRAM4XI5TEST - TRYING TO ADD SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())          
#   try:
#       task4add2program = {}
#       # OK
#       task4add2program ['CODE4SOURCE'] = {}
#       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] = {}
#       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] [1] = '... profile = "33" />\n'
#       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] [2] = '... program = "tpu2buffer4xi.py" />\n'
#       task4add2program ['SIZE4SOURCE'] = {}
#       task4add2program ['SIZE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'])
#       # NOT OK - EXISTS
#       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2sampler4xi5test.py'] = {}
#       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2sampler4xi5test.py'] [1] = '... profile = "33" />\n'
#       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2sampler4xi5test.py'] [2] = '... program = "tpu2sampler4xi5test.py" />\n'
#       
#       task4add2program ['SIZE4SOURCE'] = {}
#       task4add2program ['SIZE4SOURCE'] [folder2lib4tpu + 'tpu2sampler4xi5test.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2sampler4xi5test.py'])            
#       task4add2program ['SIZE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'])            
#       
#       response4add2programwithsource = add2programwithsource4tpu(task4add2program)
#       print('(Step 2A4/700) OF TPU2PROGRAM4XI5TEST - ADDED SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())
#       #                                                                                   
#   except:
#       print('(Step 2Z3/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO ADD SOURCE INTO PROGRAM OBJECT WITH FUNCTION *** ', time4xi())
#   try:                                                                                      
#       print('COUNT4SOURCE: ', response4add2programwithsource['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE: ', response4add2programwithsource['DIRECTORY4SOURCE'])
#       print('LANGUAGE4SOURCE: ', response4add2programwithsource['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE: ', response4add2programwithsource['SIZE4SOURCE'])
#       keys2source = response4add2programwithsource['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4add2programwithsource ['CODE4SOURCE'] [key2source] [1])
#           # COUNT4SOURCE:  2
#           # DIRECTORY4SOURCE:  folder2lib4tpu
#           # LANGUAGE4SOURCE:  Python
#           # SIZE4SOURCE:  {'folder2lib4tpu + tpu2buffer4xi.py': 2, 'folder2lib4tpu + tpu2sampler4xi5test.py': 1337}
#           # SOURCE:  folder2lib4tpu + tpu2buffer4xi.py
#           # ... profile = "33" />
#           # 
#           # SOURCE:  folder2lib4tpu + tpu2sampler4xi5test.py
#           # ... profile = "11" />           
#   except:
#       print('(Step 2Z4/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4add2programwithsource *** ', response4add2programwithsource)     
#          
#">
# -------------------------------------------------------------------------------------------------   
def add2programwithsource4tpu(task4program2tpu): 

    '''
    adds program for TPU
    '''
    global tpu2program4xi                                                                                           # Program of TPU    

    global dir2source4tpu                                                                                           # Directory for source files      

    
        
    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "add2programwithsource4tpu" 
    method2print = "add2programwithsource4tpu"

    message2print = "{(Step 10A/70) started" + "; add2programwithsource4tpu-started: " + str(add2programwithsource4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function add2programwithsource4tpu at 0x000000000418DC48>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2tpu-to-add2programwithsource4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': {1: '... profile = "33" />\n', 2: '... program = "tpu2buffer4xi.py" />\n'}, 
    #                  folder2lib4tpu + 'tpu2sampler4xi5test.py': {1: '... profile = "33" />\n', 2: '... program = "tpu2sampler4xi5test.py" />\n'}}, 
    #  'SIZE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': 2, 
    #                  folder2lib4tpu + 'tpu2sampler4xi5test.py': 2}}
    code2ack  = ' '
    code2nack = ' ' 
    try:
        keys2source4task = task4program2tpu ['CODE4SOURCE'].keys()
        message2print = "{(Step 10C/70) using" + "; keys2source4task-using-by-add2programwithsource4tpu: " + str(keys2source4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
              
    except: 
        message2print = "{(Step 10Z1/70) failed" + "; keys2source4task-failed-by-add2programwithsource4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
    try:
        keys2exist4task =  tpu2program4xi ['CODE4SOURCE'].keys() 
        message2print = "{(Step 10D/70) using" + "; keys2exist4task-using-by-add2programwithsource4tpu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    except:
        try:
            response2createwb4tpu = create2programwithsource4tpu()
            keys2createwb4tpu = response2createwb4tpu.keys()
            for key2createwb4tpu in keys2createwb4tpu:
                if key2createwb4tpu == 'CODE4SOURCE':
                    message2print = "{(Step 10E/70) created" + "; key2createwb4tpu-created-by-add2programwithsource4tpu: " + str(key2createwb4tpu) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)
                    # CODE4SOURCE 
            
            keys2exist4task =  tpu2program4xi ['CODE4SOURCE'].keys() 
            message2print = "{(Step 10F/70) using" + "; keys2exist4task-using-by-add2programwithsource4tpu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # dict_keys(['...//tpu2sampler4xi5test.py'])
        
        except: 
            message2print = "{(Step 10Z2/70) failed" + "; tpu2program4xi-failed-by-add2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - EXPECTED - SEVERAL SOURCE CODE CAN BE ADDED INTO ONE PROGRAM OBJECT                              
        # ---------------------------------------------------------------------------------------------             
        for key2source4task in keys2source4task:
            if key2source4task in keys2exist4task:
                message2print = "{(Step 33Z0/70) yielded" + "; key2source4task-yielded-by-add2programwithsource4tpu to: " + str(key2source4task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #folder2lib4tpu +  tpu2sampler4xi5test.py
                 
                code2nack  = code2nack + '33Z0-'
            else: 
                tpu2program4xi ['CODE4SOURCE'] [key2source4task] = task4program2tpu ['CODE4SOURCE'] [key2source4task]
 
                message2print = "{(Step 33B/70) added" + "; key2source4task-added-by-add2programwithsource4tpu to: " + str(key2source4task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #folder2lib4tpu +  tpu2buffer4xi.py
 
                message2print = "{(Step 33C/70) added" + "; code-added-by-add2programwithsource4tpu to: " + str(tpu2program4xi ['CODE4SOURCE'] [key2source4task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {1: '... profile = "33" />\n', 2: '... program = "tpu2buffer4xi.py" />\n'}
         
            
                tpu2program4xi ['COUNT4SOURCE'] = tpu2program4xi ['COUNT4SOURCE'] + 1
                message2print = "{(Step 33D/70) added" + "; count4source-added-by-add2programwithsource4tpu to: " + str(tpu2program4xi ['COUNT4SOURCE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 2
                
                tpu2program4xi ['SIZE4SOURCE'] [key2source4task]  = task4program2tpu['SIZE4SOURCE'] [key2source4task]  
                message2print = "{(Step 33E/70) set" + "; size4source-set-by-add2programwithsource4tpu to: " + str(tpu2program4xi ['SIZE4SOURCE'] [key2source4task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 2
                
                         
                code2ack  = code2ack + '33D-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; source-yielded-by-add2programwithsource4tpu: " + str(tpu2program4xi ['CODE4SOURCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # tpu2sampler4xi5test.py
        
        code2nack  = code2nack + '33Z1-'     
        

                                                                                                                   
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2sourceprograms = tpu2program4xi ['CODE4SOURCE'].keys()
    for key2sourceprograms in sorted(keys2sourceprograms):
        message2print = "{(Step 70A/70) got" + "; key2sourceprograms-got-by-add2programwithsource4tpu: " + str(key2sourceprograms) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        #folder2lib4tpu +  tpu2buffer4xi.py
        #folder2lib4tpu +  tpu2sampler4xi5test.py
          
     
    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70B/70) returning" + "; key2program4tpu-returning-by-add2programwithsource4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # COUNT4SOURCE
        # SIZE4SOURCE
        # DATA2CALLBACK
        # BUFFER
        # FUNCTION2CALLBACK
        # ACK2RETURN
        # DIRECTORY4SOURCE
        # QUEUE
        # PLATFORM
        # COUNT2REFERENCE
        # NACK2RETURN
        # CONTEXT
        # COMMANDQUEUE
        # DEVICE
        # CODE4SOURCE
        # EVENT
        # PIPE 

    return tpu2program4xi
            
#</function4xi>


# -------------------------------------------------------------------------------------------------
#<function4xi function = "compile2program4tpu" usage = "
#     
#   print('(Step 2A2/700) OF TPU2PROGRAM4XI5TEST - TRYING TO COMPILE SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())           
#   try:
#       response4compile = compile2program4tpu()
#   except:
#       print('(Step 2Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO COMPILE SOURCE INTO PROGRAM OBJECT WITH FUNCTION *** ', time4xi())                                                                      
#   try:
#       print('(Step 2B2/700) OF TPU2PROGRAM4XI5TEST - RESPONSE4COMPILE: ', time4xi())                                                                                      
#       keys2response4compile = response4compile.keys()
#       for key2response4compile in keys2response4compile:
#           print('KEY2RESPONSE4COMPILE: ', key2response4compile)
#           # tpu2sampler4xi5test.py            
#   except:
#       print('(Step 2Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4compile *** ', response4compile)
#   tailor2trace4program(t2xi)                                                                              # TAILOR TRACE AGAIN - TARGET FUNCTIONS OF ACTIVATED PROCESSES MIGHT HAVE RESET TRACE ! 
#          
#">
# -------------------------------------------------------------------------------------------------   
def compile2program4tpu(task4program2tpu=None):

    '''
    compiles program for TPU
    '''
    global tpu2program4xi                                                                                           # Program of TPU    
    global import2use4tpu                                                                                           # Imports of TPU
    global target2function4tpu                                                                                      # Target function of Process4Xi
    

    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "compile2program4tpu" 
    method2print = "compile2program4tpu"

    message2print = "{(Step 1A/70) started" + "; compile2program4tpu-started: " + str(compile2program4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function compile2program4tpu at 0x00000000039D5E48>
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2tpu-to-compile2program4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    source2program4tpu = {}                                                                                       # INITIALIZES TO EMPTY DICTIONARY
    message2print = "{(Step 1C/70) initialized" + "; source2program4tpu-initialized-by-compile2program4tpu to: " + str(source2program4tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #                
    try:             
        if task4program2tpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER
            # -----------------------------------------------------------------------------------------    
            pass                                                                       
        else:
            # -----------------------------------------------------------------------------------------
            # PARAMETERS GIVEN ARE OMITTED 
            # -----------------------------------------------------------------------------------------         
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2tpu-yielded-by-compile2program4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
       
        code2nack  = code2nack + '1Z1-' 

    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - COMPILE SOURCES INSIDE PROGRAM OBJECT FOR TPU ! 
    # ---------------------------------------------------------------------------------------------    
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) - SOURCE CODE WITHOUT EOL CHARACTERS                            
        # ---------------------------------------------------------------------------------------------             
        keys2source4tpu = tpu2program4xi['CODE4SOURCE'].keys()
        message2print = "{(Step 30A/70) using" + "; keys2source4tpu-using-by-compile2program4tpu: " + str(keys2source4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # dict_keys([folder2lib4tpu + 'tpu2sampler4xi5test.py'])
        
        unit4processing = "tpu"
        
        for key2source4tpu in keys2source4tpu:
            index2search=str.find(key2source4tpu,unit4processing) 
            if index2search == -1:
                pass
            else:
                target2compile = key2source4tpu[index2search:]                                                          # tpu2sampler4xi5test.py
                index2search=str.find(target2compile,".py")
                if index2search == -1:
                    pass
                else:
                    target2compile4tpu = target2compile[:index2search]                                                  
                    message2print = "{(Step 32A/70) set" + "; target2compile4tpu-set-by-compile2program4tpu to: " + str(target2compile4tpu)  + "; time: " + time4xi() + "}"  
                    print4xi(class2print, method2print, message2print)
                    # tpu2sampler4xi5test

                    p2source4tpu = None    
                    try:
                        # ---------------------------------------------------------------------------------------------    
                        # IMPORT TARGET FUNCTION FROM ITS MODULE AS TARGET2FUNCTION4TPU 
                        # ---------------------------------------------------------------------------------------------    
             
                        try:
                            import2use4tpu.get(target2compile4tpu, handle2error4tpu) ()
                            
                            message2print = "{(Step 32B/70) imported" + "; target2compile4tpu-imported-by-compile2program4tpu: " + str(target2compile4tpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # tpu2sampler4xi5test
                            code2ack  = code2ack + '32B-'
                        except:
                            message2print = "{(Step 32Z1/70) failed" + "; import2use4tpu-failed-by-compile2program4tpu: " + str(import2use4tpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)                            
                            # 
                            message2print = "{(Step 32Z2/70) used" + "; target2compile4tpu-used-by-compile2program4tpu: " + str(target2compile4tpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # 
                            code2nack  = code2nack + '32Z2-'
                                    
                        # ---------------------------------------------------------------------------------------------    
                        # CREATE PROCESS WITH TARGET2FUNCTION4TPU
                        # ---------------------------------------------------------------------------------------------                          
                        p2tpu = Process4Xi(target=target2function4tpu, args=())                                
                        message2print = "{(Step 33A/70) created" + "; p2tpu-created-by-compile2program4tpu: " + str(p2tpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # <Process4Xi(Process4Xi-1, initial)>
                        
                        code2ack  = code2ack + '33A-'
                        try:
                            # ---------------------------------------------------------------------------------------------    
                            # START PROCESS
                            # --------------------------------------------------------------------------------------------- 
                            p2tpu.start()
                            message2print = "{(Step 34A/70) started" + "; p2tpu-started-by-compile2program4tpu: " + str(p2tpu.name) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # Process4Xi-1
                            
                            try:
                                # ---------------------------------------------------------------------------------------------    
                                # STATUS OF PROCESS
                                # ---------------------------------------------------------------------------------------------                             
                                status2process4tpu = p2tpu.is_alive()
                                message2print = "{(Step 35A/70) got" + "; status-got-by-compile2program4tpu: " + str(status2process4tpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                # True
                                
                                try:
                                    # ---------------------------------------------------------------------------------------------    
                                    # RUN PROCESS
                                    # ---------------------------------------------------------------------------------------------                                  
                                    p2tpu.run()
                                    message2print = "{(Step 36A/70) run" + "; p2tpu-run-by-compile2program4tpu: " + str(p2tpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)
                                    # <Process4Xi(Process4Xi-1, stopped[1])>
                                except:
                                    message2print = "{(Step 36Z/70) failed" + "; run-failed-by-compile2program4tpu: " + str(p2tpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)  
                                    code2nack  = code2nack + '36Z-'            
                                    # 
                            except:
                                message2print = "{(Step 35Z/70) failed" + "; status-failed-by-compile2program4tpu: " + str(p2tpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)  
                                # 
                                code2nack  = code2nack + '35Z-'                                   
                                
                        except:
                            message2print = "{(Step 33Z/70) failed" + "; start-failed-by-compile2program4tpu: " + str(p2tpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)  
                            code2nack  = code2nack + '33Z-'     
                        
                    except:
                        message2print = "{(Step 32Z/70) failed" + "; p2source4tpu-failed-by-compile2program4tpu: " + str(p2source4tpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        code2nack  = code2nack + '32Z-' 
                                                                                    
            # ---------------------------------------------------------------------------------------------
            # SAVE SOURCE WITHOUT END OF LINES
            # ---------------------------------------------------------------------------------------------        
            keys2lines = tpu2program4xi['CODE4SOURCE'][key2source4tpu].keys()     
            source2program4tpu[key2source4tpu] = ""
            for key2line in keys2lines:
                code2add4tpu = tpu2program4xi ['CODE4SOURCE'] [key2source4tpu] [key2line]
                source2program4tpu[key2source4tpu] = source2program4tpu[key2source4tpu] + code2add4tpu.replace("\n","")
        
            message2print = "{(Step 34A/70) saved" + "; lenght-source4tpu-saved-by-compile2program4tpu to: " + str(len(source2program4tpu[key2source4tpu]))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #     
                
 
        message2print = "{(Step 38A/70) set" + "; lenght-source2program4tpu-set-by-compile2program4tpu to: " + str(len(source2program4tpu))  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '38A-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; source-yielded-by-compile2program4tpu: " + str(tpu2program4xi['CODE4SOURCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'     

                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2source4tpu = source2program4tpu.keys()
    for key2source4tpu in keys2source4tpu:
        message2print = "{(Step 70/70) returning" + "; key2source4tpu-returning-by-compile2program4tpu: " + str(key2source4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        # 
    
    return source2program4tpu
            
#</function4xi> 
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "delete2programwithbinary4tpu" usage = "
#     
#   print('(Step 3A3/700) OF TPU2PROGRAM4XI5TEST - TRYING TO DELETE BINARY INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4delete2program = {}
#       task4delete2program ['CODE4BINARY'] = {}
#       task4delete2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] = 'Yes'
#       task4delete2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
#            task4delete2program ['SIZE4BINARY'] = {}
#       task4delete2program ['SIZE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] = 'Yes'
#       task4delete2program ['SIZE4BINARY'] [folder2lib4tpu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
#       
#       response4delete2programwithbinary = delete2programwithbinary4tpu(task4delete2program) 
#       print('(Step 3A3/700) OF TPU2PROGRAM4XI5TEST - DELETED BINARY INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 3Z3/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO DELETE BINARY INSIDE PROGRAM OBJECT WITH FUNCTION *** ', time4xi()) 
#   try:                                                                                          
#       keys2response = response4delete2programwithbinary.keys()
#       print('COUNT2REFERENCE:', response4delete2programwithbinary ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4delete2programwithbinary ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4delete2programwithbinary ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4delete2programwithbinary ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4delete2programwithbinary ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4delete2programwithbinary ['SIZE4BINARY'])                
#       # 
#       keys2source = response4delete2programwithbinary['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4delete2programwithbinary ['CODE4BINARY'] [key2source] [1])
#       # COUNT2REFERENCE: 1
#       # COUNT4BINARY: 1
#       # DIRECTORY4BINARY: ...//__pycache__//
#       # FUNCTION2CALLBACK: <function trace2program4tpu at 0x0000000003B46AC8>
#       # LANGUAGE4BINARY: Python
#       # SIZE4BINARY: {'...//__pycache__//tpu2image4xi.cpython-33.pyc': 1006}
#       # BINARY:  ... //__pycache__//tpu2image4xi.cpython-33.pyc                       
#   except:
#       print('(Step 3Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithbinary : ', response4delete2programwithbinary)   
#          
#">

# -------------------------------------------------------------------------------------------------   
def delete2programwithbinary4tpu(task4program2tpu):

    '''
    deletes program for TPU
    '''
    global tpu2program4xi                                                                                     


    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "delete2programwithbinary4tpu" 
    method2print = "delete2programwithbinary4tpu"

    message2print = "{(Step 10A/70) started" + "; delete2programwithbinary4tpu-started: " + str(delete2programwithbinary4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function delete2programwithbinary4tpu at 0x000000000405CEC8>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2tpu-to-delete2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 


    code2ack  = ' '
    code2nack = ' '    
    #
    ack4key2programbinary = False
    try:
        # ---------------------------------------------------------------------------------------------
        # TASK MUST HAVE BEEN GIVEN BY USER - CONTAINS BINARY KEYS    
        # ---------------------------------------------------------------------------------------------
        keys2delete4binary = task4program2tpu ['CODE4BINARY'].keys()        
        message2print = "{(Step 19A/70) using" + "; keys2delete4binary-using-by-delete2programwithbinary4tpu: " + str(sorted(keys2delete4binary))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (Step 20A/70) - DELETE BINARY INSIDE PROGRAM OBJECT FOR TPU ! 
        # ---------------------------------------------------------------------------------------------    
        message2print = "{(Step 20A0/70) try" + "; key-try-by-delete2programwithbinary4tpu: None" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      

        for key2delete4binary in keys2delete4binary:
            try:            
                del tpu2program4xi ['CODE4BINARY'] [key2delete4binary]        
               
                message2print = "{(Step 20C/70) deleted" + "; code4binary-deleted-by-delete2programwithbinary4tpu: " + str(key2delete4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 
                
                code2ack  = code2ack + '20C-' 
                
                del tpu2program4xi ['SIZE4BINARY'] [key2delete4binary]        
               
                message2print = "{(Step 20D/70) deleted" + "; size4binary-deleted-by-delete2programwithbinary4tpu: " + str(key2delete4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 
                
                code2ack  = code2ack + '20D-' 
                
                tpu2program4xi ['COUNT4BINARY'] = tpu2program4xi ['COUNT4BINARY'] - 1
                message2print = "{(Step 20E/70) upgraded" + "; count4binary-upgraded-by-delete2programwithbinary4tpu: " + str(tpu2program4xi ['COUNT4BINARY'] ) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                #                                
                
            except:
                message2print = "{(Step 20Z1/70) yielded" + "; key2delete4binary-yielded-by-delete2programwithbinary4tpu: " + str(key2delete4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                
                code2nack  = code2nack + '20Z1-' 
    except: 
        message2print = "{(Step 20Z0/70) failed" + "; task4program2tpu-tpu2program4xi-failed-by-delete2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '20Z0-'    
                                                                                            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack  
    try:
        keys2binaryprograms = tpu2program4xi ['CODE4BINARY'] .keys()
        for key2binaryprograms in keys2binaryprograms:
            message2print = "{(Step 70A/70) got" + "; key2binaryprograms-got-by-delete2programwithbinary4tpu: " + str(key2binaryprograms) + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)                                                              
            #  
    except:
        pass 
               
     
    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70B/70) returning" + "; key2program4tpu-returning-by-delete2programwithbinary4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 


    return tpu2program4xi
            
#</function4xi> 
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "delete2programwithsource4tpu" usage = "
#     
#   print('(Step 2A4/700) OF TPU2PROGRAM4XI5TEST - TRYING TO DELETE SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4delete2program = {}
#       task4delete2program ['CODE4SOURCE'] = {}
#       task4delete2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] = 'Yes'
#       task4delete2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2dummy4xi.py'] = 'Yes'
#            task4delete2program ['SIZE4SOURCE'] = {}
#       task4delete2program ['SIZE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] = 'Yes'
#       task4delete2program ['SIZE4SOURCE'] [folder2lib4tpu + 'tpu2dummy4xi.py'] = 'Yes'
#       
#       response4delete2programwithsource = delete2programwithsource4tpu(task4delete2program)
#       print('(Step 2A5/700) OF TPU2PROGRAM4XI5TEST - DELETED SOURCE INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 2Z4/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO DELETE SOURCE INTO PROGRAM OBJECT WITH FUNCTION *** ', time4xi()) 
#   try:                                                                                      
#       print('COUNT4SOURCE: ', response4delete2programwithsource['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE: ', response4delete2programwithsource['DIRECTORY4SOURCE'])
#       print('LANGUAGE4SOURCE: ', response4delete2programwithsource['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE: ', response4delete2programwithsource['SIZE4SOURCE'])
#       keys2source = response4delete2programwithsource['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4delete2programwithsource ['CODE4SOURCE'] [key2source] [1])
#           # COUNT4SOURCE:  1
#           # DIRECTORY4SOURCE:  OK
#           # LANGUAGE4SOURCE:  Python
#           # SIZE4SOURCE:  {'... //tpu2sampler4xi5test.py': 1337}
#           # SOURCE:  ... //tpu2sampler4xi5test.py
#           # ... profile = "11" />       
#   except:
#       print('(Step 2Z5/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithsource *** ', response4delete2programwithsource) 
#          
#">

# -------------------------------------------------------------------------------------------------   
def delete2programwithsource4tpu(task4program2tpu):

    '''
    deletes program for TPU
    '''
    global tpu2program4xi                                                                                           # Program of TPU    

    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "delete2programwithsource4tpu" 
    method2print = "delete2programwithsource4tpu"

    message2print = "{(Step 10A/70) started" + "; delete2programwithsource4tpu-started: " + str(delete2programwithsource4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function delete2programwithsource4tpu at 0x000000000405CEC8>
  
    message2print = "{(Step 10B/70) passed" + "; task4program2tpu-to-delete2programwithsource4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': 'Yes'}, 
    #  'SIZE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': 'Yes'}
    # }; 


    code2ack  = ' '
    code2nack = ' '    
    #
    ack4key2programsource = False
    try:
        # ---------------------------------------------------------------------------------------------
        # TASK MUST HAVE BEEN GIVEN BY USER - CONTAINS SOURCE KEYS    
        # ---------------------------------------------------------------------------------------------
        keys2delete4source = task4program2tpu ['CODE4SOURCE'].keys()        
        message2print = "{(Step 19A/70) using" + "; keys2delete4source-using-by-delete2programwithsource4tpu: " + str(sorted(keys2delete4source))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # [folder2lib4tpu + 'tpu2buffer4xi.py']
        
        # ---------------------------------------------------------------------------------------------
        # (Step 20A/70) - DELETE SOURCE INSIDE PROGRAM OBJECT FOR TPU ! 
        # ---------------------------------------------------------------------------------------------    
        message2print = "{(Step 20A0/70) try" + "; key-try-by-delete2programwithsource4tpu: None" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      

        for key2delete4source in keys2delete4source:
            try:            
                del tpu2program4xi ['CODE4SOURCE'] [key2delete4source]        
               
                message2print = "{(Step 20C/70) deleted" + "; code4source-deleted-by-delete2programwithsource4tpu: " + str(key2delete4source) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                #folder2lib4tpu +  tpu2buffer4xi.py; tpu2dummy4xi5test.py
                
                code2ack  = code2ack + '20C-' 
                
                del tpu2program4xi ['SIZE4SOURCE'] [key2delete4source]        
               
                message2print = "{(Step 20D/70) deleted" + "; size4source-deleted-by-delete2programwithsource4tpu: " + str(key2delete4source) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                #folder2lib4tpu +  tpu2buffer4xi.py
                
                code2ack  = code2ack + '20D-' 
                
                tpu2program4xi ['COUNT4SOURCE'] = tpu2program4xi ['COUNT4SOURCE'] - 1
                message2print = "{(Step 20E/70) upgraded" + "; count4source-upgraded-by-delete2programwithsource4tpu: " + str(tpu2program4xi ['COUNT4SOURCE'] ) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 1                                  
                
            except:
                message2print = "{(Step 20Z1/70) yielded" + "; key2delete4source-yielded-by-delete2programwithsource4tpu: " + str(key2delete4source) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #folder2lib4tpu +  tpu2dummy4xi.py                    
                code2nack  = code2nack + '20Z1-' 
    except: 
        message2print = "{(Step 20Z0/70) failed" + "; task4program2tpu-tpu2program4xi-failed-by-delete2programwithsource4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '20Z0-'    
                                                                                            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack  
    try:
        keys2sourceprograms = tpu2program4xi ['CODE4SOURCE'] .keys()
        for key2sourceprograms in keys2sourceprograms:
            message2print = "{(Step 70A/70) got" + "; key2sourceprograms-got-by-delete2programwithsource4tpu: " + str(key2sourceprograms) + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)                                                              
            #folder2lib4tpu +  tpu2sampler4xi5test.py    
    except:
        pass 
               
     
    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70B/70) returning" + "; key2program4tpu-returning-by-delete2programwithsource4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # CODE2ERROR ...

    return tpu2program4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2programwithbinary4tpu" usage = "
#     
#   print('(Step 3A1/700) OF TPU2PROGRAM4XI5TEST - TRYING TO CREATE PROGRAM BINARY FOR TPU ', time4xi())          
#   try:
#       response4create2programwithbinary = create2programwithbinary4tpu()
#       print('(Step 3A2/700) OF TPU2PROGRAM4XI5TEST - CREATED PROGRAM BINARY FOR TPU ', time4xi())
#   except:
#       print('(Step 3Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM BINARY FOR TPU *** ', time4xi())
#   try:                                                                                          
#       keys2response = response4create2programwithbinary.keys()
#       print('COUNT2REFERENCE:', response4create2programwithbinary ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4create2programwithbinary ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4create2programwithbinary ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4create2programwithbinary ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4create2programwithbinary ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4create2programwithbinary ['SIZE4BINARY'])                
#       # COUNT2REFERENCE: 1
#       # COUNT4BINARY: 1
#       # DIRECTORY4BINARY: ...__pycache__//
#       # FUNCTION2CALLBACK: <function trace2program4tpu at 0x0000000003F53AC8>
#       # LANGUAGE4BINARY: Python
#       # SIZE4BINARY: {'...__pycache__//tpu2image4xi.cpython-33.pyc': 1006}
#       keys2source = response4create2programwithbinary['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4create2programwithbinary ['CODE4BINARY'] [key2source] [1])
#           # BINARY: ...//__pycache__//tpu2image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'    
#   except:
#       print('(Step 3Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4create2programwithbinary : ', response4create2programwithbinary)
#
#">

# -------------------------------------------------------------------------------------------------   
def create2programwithbinary4tpu(task4program2tpu=None):

    '''
    creates program with binary for TPU
    '''
    global tpu2program4xi                                                                                           # Program of TPU    

    global dir2binary4tpu                                                                                           # Directory for binary files      

    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2programwithbinary4tpu" 
    method2print = "create2programwithbinary4tpu"

    message2print = "{(Step 1A/70) started" + "; create2programwithbinary4tpu-started: " + str(create2programwithbinary4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2programwithbinary4tpu at 0x0000000003B19048>
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2tpu-to-create2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    tpu2program4xi = {}                                                                                             # INITIALIZES TO EMPTY DICTIONARY
    message2print = "{(Step 1E/70) initialized" + "; tpu2program4xi-initialized-by-create2programwithbinary4tpu to: " + str(tpu2program4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                 
    # {}
    try:              
        if task4program2tpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO TPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4program2tpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4program2tpu-created-by-create2programwithbinary4tpu: " + str(task4program2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4tpu = trace2program4tpu                                                               # CALLBACK FUNCTION FOR TPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4tpu-created-by-create2programwithbinary4tpu: " + str(function2callback4tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2program4tpu at 0x00000000039F12C8>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2programwithbinary4tpu"                                               # MODULE OF TPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2programwithbinary4tpu"                                               # FUNCTION OF TPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2programwithbinary4tpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR TPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2programwithbinary4tpu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4tpu-failed ; time: 22:50:41 11/25/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithbinary4tpu', 
            #  'METHOD': 'create2programwithbinary4tpu'}; time: 22:50:41 11/25/11 Pacific Standard Time}


            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4program2tpu['DATA2CALLBACK'] = args2callback                                                       # USER DATA PASSED TO TPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4program2tpu['FUNCTION2CALLBACK'] = function2callback4tpu                                           # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4program2tpu-created-by-create2programwithbinary4tpu: " + str(task4program2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2program4tpu at 0x0000000003B18AC8>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4tpu-failed ; time: 22:50:41 11/25/11 Pacific Standard Time}', 
            #  'CLASS': 'create2programwithbinary4tpu', 'METHOD': 'create2programwithbinary4tpu'}}; time: 22:50:41 11/25/11 Pacific Standard Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2programwithbinary4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2programwithbinary4tpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 22:50:41 11/25/11 Pacific Standard Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2programwithbinary4tpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4program2tpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2tpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2tpu-trying-by-create2programwithbinary4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2tpu = get2context4tpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2tpu-got-by-create2programwithbinary4tpu: " + str(context2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK':  ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2tpu-failed-by-create2programwithbinary4tpu: " + str(context2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4program2tpu ['CONTEXT'] = context2tpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2tpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2tpu-trying-by-create2programwithbinary4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2tpu = get2tpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2tpu-got-by-create2programwithbinary4tpu: " + str(device2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2tpu-failed-by-create2programwithbinary4tpu: " + str(device2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4program2tpu ['DEVICE'] = device2tpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4program2tpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2tpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2tpu-trying-by-create2programwithbinary4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2tpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2tpu-got-by-create2programwithbinary4tpu: " + str(queue2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003BD1E10>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2tpu-failed-by-create2programwithbinary4tpu: " + str(queue2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4program2tpu ['QUEUE'] = queue2tpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2tpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2tpu-trying-by-create2programwithbinary4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2tpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2tpu-got-by-create2programwithbinary4tpu: " + str(pipe2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2tpu-failed-by-create2programwithbinary4tpu: " + str(pipe2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4program2tpu ['PIPE'] = pipe2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2tpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2tpu-trying-by-create2programwithbinary4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2tpu = get2commandqueueinfo4tpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2tpu-got-by-create2programwithbinary4tpu: " + str(commandqueueinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2tpu-failed-by-create2programwithbinary4tpu: " + str(commandqueueinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4program2tpu ['COMMANDQUEUE'] = commandqueueinfo2tpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2tpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2tpu-trying-by-create2programwithbinary4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2tpu = get2bufferinfo4tpu()                                                                   # SYSTEM CREATES BUFFER FOR TPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2tpu-got-by-create2programwithbinary4tpu: " + str(bufferinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ... 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2tpu-failed-by-create2programwithbinary4tpu: " + str(bufferinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4program2tpu ['BUFFER'] = bufferinfo2tpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2tpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2tpu-trying-by-create2programwithbinary4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2tpu = get2eventinfo4tpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2tpu-got-by-create2programwithbinary4tpu: " + str(eventinfo2tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': ... 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2tpu-failed-by-create2programwithbinary4tpu: " + str(eventinfo2tpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4program2tpu ['EVENT'] = eventinfo2tpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) COUNT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            task4program2tpu ['COUNT'] = 0
            message2print = "{(Step 11A/70) trying" + "; count2tpu-trying-by-create2programwithbinary4tpu: " + str(task4program2tpu['COUNT'])  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 0            
            try:            
                task4program2tpu ['COUNT'] = int(len(kb2fetch4tpu)/2)                                               # number of binary files to be read and embedded
                message2print = "{(Step 11B/70) set" + "; count2tpu-set-by-create2programwithbinary4tpu to: " + str(task4program2tpu['COUNT'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 1                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; count2tpu-failed-by-create2programwithbinary4tpu: " + str(task4program2tpu['COUNT']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
                               
            # ---------------------------------------------------------------------------------------------
            # (12) DIRECTORY - EXTRA                       
            # ---------------------------------------------------------------------------------------------             
            dir2tpu = dir2binary4tpu                                                                                # default Python cache folder for binaries                                                                                                                                                                 
            
            message2print = "{(Step 12B/70) set" + "; dir2tpu-set-by-create2programwithbinary4tpu to: " + str(dir2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # folder2lib4tpu +  __pycache__//
            
            code2ack  = code2ack + '12B-' 
            task4program2tpu ['DIRECTORY'] = dir2tpu
            
                        
                        
            # ---------------------------------------------------------------------------------------------
            # (14) CODE AND SIZE - REQUIRED                   
            # ---------------------------------------------------------------------------------------------             
            message2print = "{(Step 14A/70) using" + "; DirectoryWalker-using-by-create2programwithbinary4tpu: " + str(DirectoryWalker)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # <class 'tpu2program4xi.DirectoryWalker'> 
            
            for file2walk in DirectoryWalker(dir2tpu):
                if os.path.isdir(file2walk):
                    message2print = "{(Step 14B/70) examining" + "; directory-examining-by-create2programwithbinary4tpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                      
                if os.path.isfile(file2walk):
                    message2print = "{(Step 14C/70) examining" + "; file-examining-by-create2programwithbinary4tpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # folder2lib4tpu +  __pycache__//tpu2buffer4xi.cpython-33.pyc
                    
                    message2print = "{(Step 14D/70) calling" + "; fetch2programbinary4tpu-calling-by-create2programwithbinary4tpu: " + str(fetch2programbinary4tpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # <function fetch2programbinary4tpu at 0x00000000040079C8>
                    
                    message2print = "{(Step 14E/70) passing" + "; file2walk-passing-to-fetch2programbinary4tpu-by-create2programwithbinary4tpu: " + str(file2walk) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # folder2lib4tpu +  __pycache__//tpu2buffer4xi.cpython-33.pyc
                    
                    fetch2programbinary4tpu(file2walk)                                                                 # fetches program files into fetch2code4binary dictionary 
                    
                    message2print = "{(Step 14F/70) performed" + "; fetch2programbinary4tpu-performed-by-create2programwithbinary4tpu: " + str(fetch2programbinary4tpu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # <function fetch2programbinary4tpu at 0x00000000040079C8>
                    
            task4program2tpu ['CODE'] = fetch2code4binary        
            size2binary4tpu  = {}
            keys2binary4tpu = fetch2code4binary.keys()                                                                 # tpu2sampler4xi5test.py
            for key2binary4tpu in keys2binary4tpu:
                size2binary4tpu[key2binary4tpu] = len(fetch2code4binary[key2binary4tpu])                               # length of saved binary   
                
                message2print = "{(Step 14H/70) using" + "; key2binary4tpu-using-by-create2programwithbinary4tpu: " + str(key2binary4tpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # folder2lib4tpu +  __pycache__//tpu2image4xi.cpython-33.pyc                       
                message2print = "{(Step 14I/70) set" + "; size2binary4tpu-set-by-create2programwithbinary4tpu to: " + str(len(fetch2code4binary[key2binary4tpu]))  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 1006
            
            code2ack  = code2ack + '14B-' 
        
            task4program2tpu ['SIZE'] = size2binary4tpu
            message2print = "{(Step 14J/70) set" + "; size-of-task4program2tpu-set-by-create2programwithbinary4tpu to: " + str(task4program2tpu ['SIZE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #  folder2lib4tpu +  __pycache__//tpu2image4xi.cpython-33.pyc': 1006}
                            
            # ---------------------------------------------------------------------------------------------
            # (15) LANGUAGE - EXTRA                             
            # ---------------------------------------------------------------------------------------------             
            language2tpu  = "Python"  # C++; C
           
            message2print = "{(Step 15B/70) set" + "; language2tpu-set-by-create2programwithbinary4tpu to: " + str(language2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '15B-' 
        
            task4program2tpu ['LANGUAGE'] = language2tpu     

                                    
            # ---------------------------------------------------------------------------------------------
            # (16) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            count2reference2tpu  = 1
           
            message2print = "{(Step 17B/70) set" + "; count2reference2tpu-set-by-create2programwithbinary4tpu to: " + str(count2reference2tpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '17B-' 
        
            task4program2tpu ['COUNT2REFERENCE'] = count2reference2tpu                                 
                                                                                                                      
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2tpu-yielded-by-create2programwithbinary4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4program2tpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2programwithbinary4tpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER', 'CODE' ... 
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE PROGRAM OBJECT FOR TPU ! 
    # ---------------------------------------------------------------------------------------------    
    tpu2program4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; tpu2program4xi-initialized-by-create2programwithbinary4tpu to: " + str(tpu2program4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}

    if 'DATA2CALLBACK' in keys2task:    
        message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 21A/70) - DATA2CALLBACK - EXTRA 
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['DATA2CALLBACK'] = task4program2tpu['DATA2CALLBACK']
            
            message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4tpu-failed ; time: 02:37:08 11/17/11 Pacific Standard Time}', 'CLASS': 'create2programwithbinary4tpu', 'METHOD': 'create2programwithbinary4tpu'}; time: 02:37:08 11/17/11 Pacific Standard Time}
            
            code2ack  = code2ack + '21A1-'
        except: 
            message2print = "{(Step 21Z1/70) yielded" + "; data2callback-tpu2program4xi-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '21Z1-'

    if 'FUNCTION2CALLBACK' in keys2task:       
        message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['FUNCTION2CALLBACK'] = task4program2tpu['FUNCTION2CALLBACK']
            
            message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function trace2program4tpu at 0x00000000039912C8>
            
            code2ack  = code2ack + '22B1-'
        except: 
            message2print = "{(Step 22Z2/70) yielded" + "; function2callback-tpu2program4xi-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z2-' 

    if 'PLATFORM' in keys2task:           
        message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22C/70) - PLATFORM - EXTRA
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['PLATFORM'] = task4program2tpu['PLATFORM']
            
            message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:37:08 11/17/11 Pacific Standard Time}
            
            code2ack  = code2ack + '22C1-'
        except: 
            message2print = "{(Step 22Z3/70) yielded" + "; platform-tpu2program4xi-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z3-' 
        
        

    if 'CONTEXT' in keys2task:         
        message2print = "{(Step 23A/70) try" + "; context2tpu-try-by-create2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 23/70) - CONTEXT - REQUIRED
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['CONTEXT'] = task4program2tpu['CONTEXT']
            
            message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK' ...       
            
            code2ack  = code2ack + '23B-'
        except: 
            message2print = "{(Step 23Z1/70) yielded" + "; context-tpu2program4xi-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
        
            code2nack  = code2nack + '23Z1-' 

    if 'DEVICE' in keys2task:           
        message2print = "{(Step 24A/70) try" + "; device-try-by-create2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 24/70) - DEVICE - EXTRA
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['DEVICE'] = task4program2tpu['DEVICE']
            
            message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'RESOLUTION2TIMER4PROFILING': 318 ... 
            
            code2ack  = code2ack + '24B-'
        except: 
            message2print = "{(Step 24Z1/70) yielded" + "; device-tpu2program4xi-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '24Z1-'

    if 'CODE2ERROR' in keys2task:                    
        message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 25/70) - ERROR CODE - REQUIRED
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['CODE2ERROR'] = task4program2tpu['CODE2ERROR']
            
            message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 0
            
            code2ack  = code2ack + '25B-'
        except: 
            message2print = "{(Step 25Z1/70) yielded" + "; code2error-tpu2program4xi-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '25Z1-'                
                

    if 'QUEUE' in keys2task:             
        message2print = "{(Step 26A/70) try" + "; queue-try-by-create2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 26/70) - QUEUE - EXTRA
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['QUEUE'] = task4program2tpu['QUEUE']
            
            message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <queue4xi.Queue4Xi object at 0x0000000003BEE630>
            
            code2ack  = code2ack + '26B-'
        except: 
            message2print = "{(Step 26Z1/70) yielded" + "; queue-tpu2program4xi-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '26Z1-' 

    if 'PIPE' in keys2task:                                   
        message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                     
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 27/70) - PIPE - EXTRA
            # -----------------------------------------------------------------------------------------               
            tpu2program4xi['PIPE'] = task4program2tpu['PIPE']
            
            message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)
            
            code2ack  = code2ack + '27B-'
        except: 
            message2print = "{(Step 27Z1/70) yielded" + "; pipe-tpu2program4xi-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '27Z1-'   

    if 'COMMANDQUEUE' in keys2task:   
        message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            tpu2program4xi['COMMANDQUEUE']  = task4program2tpu['COMMANDQUEUE']  
     
            message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
                    
            code2ack  = code2ack + '28B-'
        except: 
            message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                  
            code2nack  = code2nack + '28Z1-'    

    if 'BUFFER' in keys2task:   
        message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            tpu2program4xi['BUFFER']  = task4program2tpu['BUFFER']  
     
            message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ... 
            
            
            code2ack  = code2ack + '29B-'
        except: 
            message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '29Z1-'            

    if 'EVENT' in keys2task:   
        message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            tpu2program4xi['EVENT']  = task4program2tpu['EVENT']  
     
            message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '30B-'
        except: 
            message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '30Z1-'            
                

    if 'COUNT4BINARY' in keys2task:   
        message2print = "{(Step 31A/70) try" + "; count-try-by-create2programwithbinary4tpu: " + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            tpu2program4xi['COUNT4BINARY']  = task4program2tpu['COUNT']  
     
            message2print = "{(Step 31B/70) initialized" + "; count-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['COUNT4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '31B-'
        except: 
            message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z1-'     

    if 'DIRECTORY4BINARY' in keys2task:   
        message2print = "{(Step 32A/70) try" + "; DIRECTORY-try-by-create2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            tpu2program4xi['DIRECTORY4BINARY']  = task4program2tpu['DIRECTORY']  
     
            message2print = "{(Step 32B/70) initialized" + "; directory-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['DIRECTORY4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #folder2lib4tpu +  __pycache__//
            code2ack  = code2ack + '32B-'
        except: 
            message2print = "{(Step 32Z1/70) yielded" + "; DIRECTORY-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z1-'  

    if 'CODE4BINARY' in keys2task:               
        message2print = "{(Step 33A/70) try" + "; binary-try-by-create2programwithbinary4tpu: None" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)        
        tpu2program4xi['CODE4BINARY'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            tpu2program4xi['CODE4BINARY'] = task4program2tpu ['CODE']
    
     
            message2print = "{(Step 33B/70) created" + "; code-created-by-create2programwithsource4tpu: " + str(len(tpu2program4xi['CODE4BINARY']))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
             
            code2ack  = code2ack + '33B-'
        except: 
            message2print = "{(Step 33Z1/70) yielded" + "; binary-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '33Z1-'     

    if 'SIZE4BINARY' in keys2task:           
        message2print = "{(Step 34A/70) try" + "; size-try-by-create2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34) SIZE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            tpu2program4xi['SIZE4BINARY']  = task4program2tpu['SIZE']  
     
            message2print = "{(Step 34B/70) initialized" + "; size-initialized-by-create2programwithbinary4tpu to: " + str(tpu2program4xi['SIZE4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc': 1006}
            code2ack  = code2ack + '34B-'
        except: 
            message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z1-'     

    if 'LANGUAGE4BINARY' in keys2task:           
        message2print = "{(Step 35A/70) try" + "; language-try-by-create2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (35) LANGUAGE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            tpu2program4xi['LANGUAGE4BINARY']  = task4program2tpu['LANGUAGE']  
     
            message2print = "{(Step 35B/70) initialized" + "; language-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['LANGUAGE4BINARY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # Python
            code2ack  = code2ack + '35B-'
        except: 
            message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '35Z1-'     

    if 'COUNT2REFERENCE' in keys2task:                   
        message2print = "{(Step 36A/70) try" + "; count2reference-try-by-create2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (36) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
                            
            tpu2program4xi['COUNT2REFERENCE']  = task4program2tpu['COUNT2REFERENCE']  
     
            message2print = "{(Step 36B/70) initialized" + "; count2reference-created-by-create2programwithbinary4tpu: " + str(tpu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '36B-'
        except: 
            message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-create2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '36Z1-'     
        
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70/70) returning" + "; key2program4tpu-returning-by-create2programwithbinary4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # CODE2ERROR ... CODE4BINARY ...
    
    return tpu2program4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "fetch2programsource4tpu" usage = "
#   fetch2programsource4tpu(file2walk)
#"> 
# -------------------------------------------------------------------------------------------------
def fetch2programsource4tpu(program2explore4tpu):
    global fetch2code4source                                                                                        # Fetched code dictionary
    
    global kb2fetch4tpu                                                                                             # Knowledgebase for fetching
    global profile2fetch4source                                                                                     # Extension for files
    global keys2profile4source                                                                                      
    

    class2print  = "fetch2programsource4tpu"
    method2print = "fetch2programsource4tpu" 
     
    message2print = "{(Step 0A/33) passed" + "; program2explore4tpu-passed-to-fetch2programsource4tpu: " + str(program2explore4tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    #folder2lib4tpu +  tpu2buffer4xi.py; tpu2dummy4xi5test.cpp
    
    message2print = "{(Step 0B/33) using" + "; keys2profile4source-using-by-fetch2programsource4tpu: " + str(keys2profile4source)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # ['.cl', '.c', '.py', '.cpp']
                  
    # ---------------------------------------------------------------------------------------------
    # SPLIT PROGRAM NAME WITH PATH AND EXTENSION TO TWO PIECES
    # ---------------------------------------------------------------------------------------------    
    name2program4source=os.path.splitext(program2explore4tpu)

        
    if name2program4source[1] in keys2profile4source:
        try:          
            # -----------------------------------------------------------------------------------------
            # (Step 1/33) OF FETCH2SCOPE4XI - SOURCE PROGRAM
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 1A/33) analyzing" + "; program2explore4tpu-analyzing-by-fetch2programsource4tpu: " + str(program2explore4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            #folder2lib4tpu +  tpu2buffer4xi.py
                 
            path2input4source = program2explore4tpu
            message2print = "{(Step 1B/33) set" + "; path2input4source-set-by-fetch2programsource4tpu to: " + str(path2input4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #folder2lib4tpu +  tpu2buffer4xi.py 
    
            # -----------------------------------------------------------------------------------------
            # (Step 2/33) OF FETCH2SCOPE4XI - CHECKING WHETHER PROGRAM MUST BE FETCHED
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 2A/33) analyzing" + "; path2input4source-analyzing-by-fetch2programsource4tpu: " + str(path2input4source) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #folder2lib4tpu +  tpu2buffer4xi.py; tpu2dummy4xi5test.cpp
                
            keys2fetch4source = kb2fetch4tpu.keys()
            ack2match4source = False
            for key2fecth4source in keys2fetch4source:                                                              # 1,2 ...
                # -----------------------------------------------------------------------------------------
                # 
                # -----------------------------------------------------------------------------------------
                index2search=str.find(path2input4source,kb2fetch4tpu[key2fecth4source])                             # kb2fetch4tpu[1] = folder2lib4tpu + 'tpu2sampler4xi5test.py'
                if index2search == -1:                                         
                    # -----------------------------------------------------------------------------------------
                    # VALID FILE TYPE BUT NOT SELECTED TO BE FETCHED - NOT BE SKIPPED MIGHT GO THROUGH HERE TOO
                    # -----------------------------------------------------------------------------------------               
                    message2print = "{(Step 1E/33) skipped" + "; path2input4source-skipped-by-fetch2programsource4tpu: " + str(path2input4source)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                      
                    #folder2lib4tpu +  tpu2buffer4xi.py 
    
                else:
                    ack2match4source = True
                    
                    message2print = "{(Step 2B/33) decided" + "; program-to-read-decided-by-fetch2programsource4tpu: " + str(path2input4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)        
                    #folder2lib4tpu +  tpu2sampler4xi5test.py; tpu2dummy4xi5test.cpp
    
                    # -----------------------------------------------------------------------------------------
                    # (Step 3/33) OF FETCH2SCOPE4XI - OPEN INPUT
                    # -----------------------------------------------------------------------------------------
                    file2input4source = open(path2input4source,'r')
                    message2print = "{(Step 3A/33) opened" + "; file2input4source-opened-by-fetch2programsource4tpu: " + str(file2input4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)     
                    # <_io.TextIOWrapper name=folder2lib4tpu + 'tpu2sampler4xi5test.py' mode='r' encoding='cp1252'> ; tpu2dummy4xi5test.cpp 
                    
                    # -----------------------------------------------------------------------------------------
                    # (Step 4/33) OF FETCH2SCOPE4XI - OUTPUT 
                    # -----------------------------------------------------------------------------------------
                    key2save4source = path2input4source
                    fetch2code4source[key2save4source] = {}                                                                # fetch2code4source['tpu2sampler4xi5test.py'] = {}
                    number2line4source = 0  
                    message2print = "{(Step 3B/33) saving" + "; key2fecth4source-saving-by-fetch2programsource4tpu: " + str(key2fecth4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)              
                    # 
             
                    while 1:
                        # -------------------------------------------------------------------------------------
                        # (Step 5/33) OF FETCH2SCOPE4XI - PROCESS FILE
                        # -------------------------------------------------------------------------------------             
                        line2process4source = file2input4source.readline()  
                        if not line2process4source:
                            # ---------------------------------------------------------------------------------
                            # (Step 30/33) OF FETCH2SCOPE4XI - ALL LINES READ, FINISH
                            # ---------------------------------------------------------------------------------             
                            break                                               
                        else:
                            # ---------------------------------------------------------------------------------
                            # (Step 7/33) OF FETCH2SCOPE4XI - SAVE LINE
                            # ---------------------------------------------------------------------------------
                            message2print = "{(Step 7A/33) read" + "; line2process4source-read-by-fetch2programsource4tpu: " + str(line2process4source) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)              
                            # ... profile = "11" />
                            # ...
                            # # end of tpu2sampler4xi5test.py 
                            # #/> 
                            try:
                                number2line4source = number2line4source + 1
                                fetch2code4source[key2save4source][number2line4source] = line2process4source
                            except:
                                message2print = "{(Step 7Z1/33) failed" + "; key2fecth4source-failed-by-fetch2programsource4tpu: " + str(key2fecth4source) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z2/33) failed" + "; line2process4source-failed-by-fetch2programsource4tpu: " + str(line2process4source) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z3/33) failed" + "; number2line4source-failed-by-fetch2programsource4tpu: " + str(number2line4source) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)                            
                                                                 
                    # -----------------------------------------------------------------------------------------
                    # (Step 31/33) OF FETCH2SCOPE4XI - CLOSE INPUT
                    # -----------------------------------------------------------------------------------------            
                    file2input4source.close()
                    message2print = "{(Step 31A/33) closed" + "; file2input4source-closed-by-fetch2programsource4tpu: " + str(file2input4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)           
                    # <_io.TextIOWrapper name=folder2lib4tpu + 'tpu2sampler4xi5test.py' mode='r' encoding='cp1252'>; tpu2dummy4xi5test.cpp
                        
            
                    message2print = "{(Step 31B/33) saved" + "; fetch2code4source-saved-by-fetch2programsource4tpu: " + str(fetch2code4source) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # {folder2lib4tpu + 'tpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: '# -- ....
                     
                    # ---------------------------------------------------------------------------------------------
                    #
                    # ---------------------------------------------------------------------------------------------                                          
        except:
            message2print = "{(Step 1Z1/33) failed" + "; name2program4source-failed-by-fetch2programsource4tpu: " + str(name2program4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
    else:
        try:
            # -----------------------------------------------------------------------------------------
            # INVALID FILE TYPE - NOT IN PROFILE
            # -----------------------------------------------------------------------------------------       
            message2print = "{(Step 1F/33) skipped" + "; name2program4source-skipped-by-fetch2programsource4tpu: " + str(name2program4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            # folder2lib4tpu + '__pycache__//tpu2buffer4xi.cpython-32', '.pyc')
            
            extension2skip = name2program4source[1]
            message2print = "{(Step 1G/33) skipped" + "; extension-skipped-by-fetch2programsource4tpu: " + str(extension2skip[1:])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # pyc
    
            message2print = "{(Step 1H/33) returning" + "; fetch2programsource4tpu-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)   
        except:
            message2print = "{(Step 1Z2/33) yielded" + "; name2program4source-yielded-by-fetch2programsource4tpu: " + str(name2program4source)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
    message2print = "{(Step 1H/33) returning" + "; fetch2programsource4tpu-returning: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)            
    #   
                                            
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2programwithsource4tpu" usage = "
#
#   ... prepare task4set2programws ...    
#
#   response4set2program = {} 
#   print('(Step 17F/700) OF TPU2PROGRAM4XI5TEST - TRYING TO SET PROGRAM WITH SOURCE: ', set2programwithsource4tpu)   
#   # <function set2programwithsource4tpu at 0x0000000003837948>
#   try:                               
#       response4set2pws = set2programwithsource4tpu(task4set2programws)
#       print('(Step 17G/700) OF TPU2PROGRAM4XI5TEST - SET PROGRAM SOURCE WITH SOURCE: ', set2programwithsource4tpu)                                                                                                           
#   except:
#       print('(Step 17Z/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM SOURCE WITH SOURCE: ', set2programwithsource4tpu)  
#   try:                                                                                          
#       keys2response = response4set2pws.keys()
#       print('COUNT2REFERENCE:', response4set2pws ['COUNT2REFERENCE'])   
#       print('COUNT4SOURCE:', response4set2pws ['COUNT4SOURCE'])
#       print('DIRECTORY4SOURCE:', response4set2pws ['DIRECTORY4SOURCE']) 
#       print('FUNCTION2CALLBACK:', response4set2pws ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4SOURCE:', response4set2pws ['LANGUAGE4SOURCE'])
#       print('SIZE4SOURCE:', response4set2pws ['SIZE4SOURCE'])                
#        keys2source = response4set2pws['CODE4SOURCE'].keys()
#       for key2source in keys2source:
#           print('SOURCE: ', key2source)
#           print(response4set2pws ['CODE4SOURCE'] [key2source] [1])
#           #
#           # COUNT2REFERENCE: 1
#           # COUNT4SOURCE: 1
#           # DIRECTORY4SOURCE: OK
#           # FUNCTION2CALLBACK: <function trace2program4tpu at 0x0000000003A28B48>
#           # LANGUAGE4SOURCE: Python
#           # SIZE4SOURCE: 1
#           # SOURCE:  ... tpu2image4xi5test.py
#           # ... profile = "11" />                  
#   except:
#       print('(Step 3Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4set2pws : ', response4set2pws)                                                   
#
#">

# -------------------------------------------------------------------------------------------------   
def set2programwithsource4tpu(task4programset2tpu=None):

    '''
    sets programs for TPU
    '''
    global tpu2program4xi  

    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2programwithsource4tpu"
    method2print = "set2programwithsource4tpu" 

    message2print = "{(Step 1A/33) started" + "; set2programwithsource4tpu-started: " + str(set2programwithsource4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2programwithsource4tpu at 0x0000000003837948> 
    
    keys2task = task4programset2tpu.keys()
    for key2task in sorted(keys2task):
        message2print = "{(Step 1B/33) passed" + "; key2task-passed-to-set2programwithsource4tpu: " + str(key2task) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        # CODE2ERROR
        # CONTEXT 
        # COUNT4SOURCE
        # DEVICE
        # DIRECTORY4SOURCE
        # LANGUAGE4SOURCE
        # PROPERTIES
        # QUEUE
          
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2program4xi is None:
            tpu2program4xi = create2programwithsource4tpu()                                                                              # INITIALIZE PROGRAM FOR TPU 
            message2print = "{(Step 1C/70) set" + "; tpu2program4xi-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2programwithsource4tpu-skipped-by-set2programwithsource4tpu: " + str(create2programwithsource4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                 
    except:
        try:
            tpu2program4xi = create2programwithsource4tpu()
            message2print = "{(Step 1E/70) set" + "; tpu2program4xi-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            tpu2program4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; tpu2program4xi-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET       
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['DATA2CALLBACK'] = task4programset2tpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-tpu2program4xi-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['FUNCTION2CALLBACK'] = task4programset2tpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-tpu2program4xi-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['PLATFORM'] = task4programset2tpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-tpu2program4xi-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2tpu-try-by-set2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['CONTEXT'] = task4programset2tpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-tpu2program4xi-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['DEVICE'] = task4programset2tpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-tpu2program4xi-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['CODE2ERROR'] = task4programset2tpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-tpu2program4xi-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['QUEUE'] = task4programset2tpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-tpu2program4xi-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['PIPE'] = task4programset2tpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-tpu2program4xi-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['COMMANDQUEUE']  = task4programset2tpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
        tpu2program4xi['BUFFER']  = task4programset2tpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2programwithsource4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['EVENT']  = task4programset2tpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '30Z1-'            
                
    message2print = "{(Step 31A/70) try" + "; count-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31)  - COUNT                           
        # ---------------------------------------------------------------------------------------------             
        tpu2program4xi['COUNT4SOURCE']  = task4programset2tpu['COUNT4SOURCE']  
 
        message2print = "{(Step 31B/70) set" + "; count-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['COUNT4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z1-'  

    message2print = "{(Step 32A/70) try" + "; directory-try-by-set2programwithsource4tpu: " + str(task4programset2tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print) 
    # {'CODE2ERROR': 0 ... 'DIRECTORY4SOURCE': '...//a' ... 

    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DIRECTORY - REQUIRED                     
        # ---------------------------------------------------------------------------------------------                                    
        tpu2program4xi['DIRECTORY4SOURCE']  = task4programset2tpu['DIRECTORY4SOURCE'] 
        
        message2print = "{(Step 32B/70) set" + "; directory-set-by-set2programwithdirectory4tpu to: " + str(tpu2program4xi['DIRECTORY4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # OK
        code2ack  = code2ack + '32B-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; directory-yielded-by-set2programwithdirectory4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z1-' 
        
    message2print = "{(Step 33A/70) try" + "; code-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['CODE4SOURCE']  = task4programset2tpu['CODE4SOURCE']  
 
        message2print = "{(Step 33B/70) set" + "; code-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['CODE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4tpu + 'tpu2image4xi5test.py': 
        #  {1: '... profile = "11" />\n', 
        #   2: '... program = "tpu2image4xi5test.py">\n'}
 
        code2ack  = code2ack + '33B-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; code-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '33Z1-'             
        
    message2print = "{(Step 34A/70) try" + "; size-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) SIZE - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['SIZE4SOURCE']  = task4programset2tpu['SIZE4SOURCE']  
 
        message2print = "{(Step 34B/70) set" + "; size-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['SIZE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34B-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-set2programwithsource4tpu: " + str(task4programset2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z1-'    
                
    message2print = "{(Step 35A/70) try" + "; language-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) LANGUAGE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['LANGUAGE4SOURCE']  = task4programset2tpu['LANGUAGE4SOURCE']  
 
        message2print = "{(Step 35B/70) set" + "; language-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['LANGUAGE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'          
                        
    message2print = "{(Step 36A/70) try" + "; count2reference-try-by-set2programwithsource4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) COUNT2REFERENCE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['COUNT2REFERENCE']  = task4programset2tpu['COUNT2REFERENCE']  
 
        message2print = "{(Step 36B/70) set" + "; count2reference-set-by-set2programwithsource4tpu to: " + str(tpu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-set2programwithsource4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '36Z1-'                           

    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack 
    
     
    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70/70) returning" + "; key2program4tpu-returning-by-set2programwithsource4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # COUNT4SOURCE ...

    return tpu2program4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "fetch2programbinary4tpu" usage = "
#   fetch2programbinary4tpu(file2walk)
#"> 
# -------------------------------------------------------------------------------------------------
def fetch2programbinary4tpu(program2explore4tpu):
    global fetch2code4binary                                                                                        # Fetched code dictionary
    
    global kb2fetch4tpu                                                                                             # Knowledgebase for fetching

    global keys2profile4binary         

        
    class2print  = "fetch2programbinary4tpu"
    method2print = "fetch2programbinary4tpu" 
 
    message2print = "{(Step 1/8) got" + "; program2explore4tpu-got-by-fetch2programbinary4tpu: " + str(program2explore4tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #folder2lib4tpu +  __pycache__//tpu2buffer4xi.cpython-33.pyc
               
    # ---------------------------------------------------------------------------------------------
    # SPLIT PROGRAM NAME WITH PATH AND EXTENSION TO TWO PIECES
    # ---------------------------------------------------------------------------------------------    
    name2program4binary=os.path.splitext(program2explore4tpu)
    
    if name2program4binary[1] in keys2profile4binary:          
        # -----------------------------------------------------------------------------------------
        # (Step 1/33) OF FETCH2SCOPE4XI - PYTHON BINARY
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 1A/33) analyzing" + "; program2explore4tpu-analyzing-by-fetch2programbinary4tpu: " + str(program2explore4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        #folder2lib4tpu +  __pycache__//tpu2buffer4xi.cpython-33.pyc 
        
        path2input4binary = program2explore4tpu                                                                          
        message2print = "{(Step 1B/33) set" + "; path2input4binary-set-by-fetch2programbinary4tpu to: " + str(path2input4binary)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        #folder2lib4tpu +  __pycache__//tpu2buffer4xi.cpython-33.pyc
        
        # 
        # -----------------------------------------------------------------------------------------
        # (Step 2/33) OF FETCH2SCOPE4XI - CHECKING WHETHER PROGRAM MUST BE FETCHED 
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A1/33) analyzing" + "; path2input4binary-analyzing-by-fetch2programbinary4tpu: " + str(path2input4binary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #folder2lib4tpu +  __pycache__//tpu2buffer4xi.cpython-33.pyc 
        # ...
        #folder2lib4tpu +  __pycache__//tpu2sampler4xi.cpython-33.pyc
        keys2fetch4binary = kb2fetch4tpu.keys()
        ack2match4binary = False
        for key2fecth4binary in keys2fetch4binary:                                                                  # 
            string2search4binary = kb2fetch4tpu[key2fecth4binary]                                                   # tpu2sampler4xi.pyc
            index2search=str.find(path2input4binary,string2search4binary)                                           # E:\Scope4xi\041721\zap\lib\a\__pycache__\tpu2sampler4xi.cpython-33.pyc
            if index2search == -1:                                         
                pass                                                                                                # not ordered to be fetched
            else:
                ack2match4binary = True                                                                             # E:\Scope4xi\041721\zap\lib\a\__pycache__\tpu2sampler4xi.cpython-33.pyc
                    
        if ack2match4binary is True:
            message2print = "{(Step 2B/33) decided" + "; program-to-read-decided-by-fetch2programbinary4tpu: " + str(path2input4binary) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)        
            # folder2lib4tpu +  __pycache__//tpu2image4xi.cpython-33.pyc


            message2print = "{(Step 2B/33) opening" + "; path2input4binary-opening-by-fetch2programbinary4tpu: " + str(path2input4binary) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
            # folder2lib4tpu +  __pycache__//tpu2image4xi.cpython-33.pyc 
            try:
                # -----------------------------------------------------------------------------------------
                # (Step 3/33) OF FETCH2SCOPE4XI - OPEN BINARY INPUT
                # -----------------------------------------------------------------------------------------
                file2input4binary = open(path2input4binary,'rb')
                message2print = "{(Step 3A/33) opened" + "; file2input4binary-opened-by-fetch2programbinary4tpu: " + str(file2input4binary) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)     
                # <_io.BufferedReader name=folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'>
            
                try:
                    # -----------------------------------------------------------------------------------------
                    # (Step 4/33) OF FETCH2SCOPE4XI - OUTPUT
                    # -----------------------------------------------------------------------------------------
                    key2save4binary = path2input4binary
                    fetch2code4binary[key2save4binary] = {}                                                                # 
                    number2line4binary = 0  
                    message2print = "{(Step 3B/33) saving" + "; key2save4binary-saving-by-fetch2programbinary4tpu: " + str(key2save4binary) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)              
                    # folder2lib4tpu +  __pycache__//tpu2sampler4xi.cpython-33.pyc
        
                    while 1:
                        # -------------------------------------------------------------------------------------
                        # (Step 5/33) OF FETCH2SCOPE4XI - PROCESS FILE
                        # -------------------------------------------------------------------------------------             
                        line2process4binary = file2input4binary.readline()  
                        if not line2process4binary:
                            # ---------------------------------------------------------------------------------
                            # (Step 30/33) OF FETCH2SCOPE4XI - ALL LINES READ, FINISH
                            # ---------------------------------------------------------------------------------             
                            break                                               
                        else:
                            try:
                                # ---------------------------------------------------------------------------------
                                # (Step 7/33) OF FETCH2SCOPE4XI - SAVE LINE
                                # ---------------------------------------------------------------------------------
                                number2line4binary = number2line4binary + 1
                                # message2print = "{(Step 7A/33) reading" + "; number2line4binary-reading-by-fetch2programbinary4tpu: " + str(number2line4binary) + "; time: " + time4xi() + "}"
                                # print4xi(class2print, method2print, message2print)              
                                # 1,2,3 .... 1006
                                                
                                fetch2code4binary[key2save4binary][number2line4binary] = line2process4binary
                            except:
                                message2print = "{(Step 7Z1/33) failed" + "; key2fecth4binary-failed-by-fetch2programbinary4tpu: " + str(key2fecth4binary) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z2/33) failed" + "; line2process4binary-failed-by-fetch2programbinary4tpu: " + str(line2process4binary) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                
                                message2print = "{(Step 7Z3/33) failed" + "; number2line4binary-failed-by-fetch2programbinary4tpu: " + str(number2line4binary) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)        
                except:        
                    message2print = "{(Step 7Z4/33) failed" + "; number2line4binary-failed-by-fetch2programbinary4tpu: " + str(number2line4binary) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 52           
                finally:
                    # -----------------------------------------------------------------------------------------
                    # (Step 31/33) OF FETCH2SCOPE4XI - CLOSE INPUT
                    # -----------------------------------------------------------------------------------------            
                    file2input4binary.close()
                    message2print = "{(Step 31A/33) closed" + "; file2input4binary-closed-by-fetch2programbinary4tpu: " + str(file2input4binary) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)           
                    # <_io.BufferedReader name=folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'>
            except:
                message2print = "{(Step 7Z4/33) failed" + "; opening-failed-by-fetch2programbinary4tpu: " + str(path2input4binary) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  

        else:
            pass
    else:
        # -----------------------------------------------------------------------------------------
        # INVALID TYPE - NOT IN PROFILE
        # -----------------------------------------------------------------------------------------       
        message2print = "{(Step 1F/33) skipped" + "; name2program4binary-skipped-by-fetch2programbinary4tpu: " + str(name2program4binary)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        # 
        
        extension2skip = name2program4binary[1]
        message2print = "{(Step 1G/33) skipped" + "; extension-skipped-by-fetch2programbinary4tpu: " + str(extension2skip[1:])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 

        
    message2print = "{(Step 33/33) returning" + "; fetch2programbinary4tpu-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 02:24:36 11/17/11 Pacific Standard Time
                             
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2programwithbinary4tpu" usage = "
#    
#    print('(Step 4B/33) OF TPU2PROGRAM4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#    platform2xi = {}
#    try:
#        platform2xi = get2platformids4xi()
#        print('(Step 4B/33) of CONTEXT4TPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#    except:
#        print('(Step 4Z/33) of CONTEXT4TPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#    
# 
#    ... other data for set ...
#    
#    response4set2program = {} 
#    print('(Step 37F/700) OF TPU2PROGRAM4XI5TEST - TRYING TO PERFORM SET2PROGRAM4TPU: ', set2programwithbinary4tpu)   
#    # 
#    try:                               
#        response4set2program = set2programwithbinary4tpu(task4set2programwithbinary)                                                                                                         
#
#        keys2response = response4set2program.keys()
#        for key2response in sorted(keys2response):
#            print(key2response, ' : ', response4set2program[key2response])
#            # CODE4BINARY  :  {folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'\x00Z\n'}}
#            # CODE4SOURCE  :  {folder2lib4tpu + 'tpu2image4xi5test.py': {1: '... profile = "11" />\n', 2: '... program = "tpu2image4xi5test.py">\n'}}
#            # COUNT4BINARY  :  1
#            # COUNT4SOURCE  :  1
#            # LANGUAGE4BINARY  :  Python
#            # LANGUAGE4SOURCE  :  Python
#            # SIZE4BINARY  :  {folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc': 1006}
#                          
#    except:
#        print('(Step 37Z/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PERFORM SET2PROGRAM4TPU: ', set2programwithbinary4tpu)                                            
#
#">

# -------------------------------------------------------------------------------------------------   
def set2programwithbinary4tpu(task4programsetwb2tpu=None):

    '''
    sets programs for TPU
    '''
    global tpu2program4xi  
    
      
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2programwithbinary4tpu"
    method2print = "set2programwithbinary4tpu"

    message2print = "{(Step 1A/33) started" + "; set2programwithbinary4tpu-started: " + str(set2programwithbinary4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2programwithbinary4tpu at 0x0000000003B43AC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4programsetwb2tpu-to-set2programwithbinary4tpu: " + str(task4programsetwb2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0, 
    #  'DIRECTORY4BINARY': ...//__pycache__//', 
    #  'LANGUAGE4BINARY': 'Python', 
    #  'SIZE4BINARY': 1, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003B19048>, 
    #  'CODE4BINARY': {'...//__pycache__//tpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'\x00Z\n'}}, 
    #   ...


    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if tpu2program4xi is None:
            tpu2program4xi = create2programwithbinary4tpu()                                                                              # INITIALIZE PROGRAM FOR TPU 
            message2print = "{(Step 1C/70) set" + "; tpu2program4xi-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2programwithbinary4tpu-skipped-by-set2programwithbinary4tpu: " + str(create2programwithbinary4tpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                 
    except:
        try:
            tpu2program4xi = create2programwithbinary4tpu()
            message2print = "{(Step 1E/70) set" + "; tpu2program4xi-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            tpu2program4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; tpu2program4xi-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['DATA2CALLBACK'] = task4programsetwb2tpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-tpu2program4xi-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['FUNCTION2CALLBACK'] = task4programsetwb2tpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-tpu2program4xi-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['PLATFORM'] = task4programsetwb2tpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-tpu2program4xi-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2tpu-try-by-set2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['CONTEXT'] = task4programsetwb2tpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-tpu2program4xi-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['DEVICE'] = task4programsetwb2tpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318 ... 
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-tpu2program4xi-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['CODE2ERROR'] = task4programsetwb2tpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-tpu2program4xi-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['QUEUE'] = task4programsetwb2tpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003DF3668>
         
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-tpu2program4xi-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['PIPE'] = task4programsetwb2tpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-tpu2program4xi-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['COMMANDQUEUE']  = task4programsetwb2tpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['BUFFER']  = task4programsetwb2tpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2programwithbinary4tpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['EVENT']  = task4programsetwb2tpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '30Z1-'            
                
    message2print = "{(Step 31A/70) try" + "; count-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) COUNT - REQUIRED                           
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['COUNT4BINARY']  = task4programsetwb2tpu['COUNT4BINARY']  
 
        message2print = "{(Step 31B/70) set" + "; count-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['COUNT4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; count-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z1-'
          
  
    message2print = "{(Step 32A/70) try" + "; directory2binary-try-by-set2programwithdirectory2binary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DIRECTORY4BINARY - REQUIRED                               
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['DIRECTORY4BINARY']  = task4programsetwb2tpu['DIRECTORY4BINARY']  
 
        message2print = "{(Step 32B/70) set" + "; directory2binary-set-by-set2programwithdirectory2binary4tpu to: " + str(tpu2program4xi['DIRECTORY4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # ...//__pycache__//

        code2ack  = code2ack + '32B-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; directory2binary-yielded-by-set2programwithdirectory2binary4tpu: " + str(directory2binary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z1-'     
    
    message2print = "{(Step 33A/70) try" + "; binary-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE4BINARY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['CODE4BINARY']  = task4programsetwb2tpu['CODE4BINARY']  
 
        message2print = "{(Step 33B/70) set" + "; binary-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['CODE4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'\x00Z\n'}}

        code2ack  = code2ack + '33B-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; binary-yielded-by-set2programwithbinary4tpu: " + str(binary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '33Z1-' 
        
    message2print = "{(Step 34A/70) try" + "; size-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) SIZE4BINARY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['SIZE4BINARY']  = task4programsetwb2tpu['SIZE4BINARY']  
 
        message2print = "{(Step 34B/70) set" + "; size-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['SIZE4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '34B-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z1-'             
        
    message2print = "{(Step 35A/70) try" + "; language-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) LANGUAGE4BINARY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['LANGUAGE4BINARY']  = task4programsetwb2tpu['LANGUAGE4BINARY']  
 
        message2print = "{(Step 35B/70) set" + "; language-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['LANGUAGE4BINARY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # Python
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'    
                
    message2print = "{(Step 35A/70) try" + "; count2reference-try-by-set2programwithbinary4tpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) COUNT2REFERENCE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        tpu2program4xi['COUNT2REFERENCE']  = task4programsetwb2tpu['COUNT2REFERENCE']  
 
        message2print = "{(Step 35B/70) set" + "; count2reference-set-by-set2programwithbinary4tpu to: " + str(tpu2program4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; count2reference-yielded-by-set2programwithbinary4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'                           

    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack 
    
     
    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70/70) returning" + "; key2program4tpu-returning-by-set2programwithbinary4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 

    return tpu2program4xi
           
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2program4tpu" usage = "
#
#   task4release2program  = {} 
#   task4release2program  ['COUNT2REFERENCE'] = 1 
#   response4release2program = {}
#   
#   print('(Step 18A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO RELEASE PROGRAM OF TPU DEVICE ', time4xi())    
#   # 
#   try:                               
#       response4release2program = release2program4tpu(task4release2program)
#       print('(Step 18B/700) OF TPU2PROGRAM4XI5TEST - RELEASED PROGRAM OF TPU DEVICE ', time4xi()) 
#   except:
#       print('(Step 18Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO RELEASE PROGRAM OF TPU DEVICE ', time4xi())                                                                                                  
#   try:
#       print('(Step 18C/700) OF TPU2PROGRAM4XI5TEST COUNT2REFERENCE: ', response4release2program ['COUNT2REFERENCE'])
#       # 0
#   except:
#       print('(Step 18Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RELEASE2PROGRAM ', response4release2program)  
#   
#">

# -------------------------------------------------------------------------------------------------   
def release2program4tpu(task4release2tpu=None):

    '''
    releases tpu2program4xi
    '''
    global tpu2program4xi  
    
             
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2program4tpu"
    method2print = "release2program4tpu" 

    message2print = "{(Step 1A/33) started" + "; release2program4tpu-started: " + str(release2program4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2program4tpu at 0x0000000003A98AC8>
    
    
    message2print = "{(Step 1B/33) passed" + "; task4release2tpu-to-release2program4tpu: " + str(task4release2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    

    code2ack  = ' '
    code2nack = ' '

    try:
        if tpu2program4xi is None:
            tpu2program4xi = create2programwithsource4tpu()                                                                       # INITIALIZE PROGRAM                 
    except:
        try:
            tpu2program4xi = create2programwithsource4tpu()
        except:
            tpu2program4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; tpu2program4xi-yielded-by-release2program4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2program4tpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2tpu is None:
        task4release2tpu = {}
        task4release2tpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        tpu2program4xi['COUNT2REFERENCE'] = tpu2program4xi['COUNT2REFERENCE'] - task4release2tpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2program4tpu: " + str(tpu2program4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
         
        if tpu2program4xi['COUNT2REFERENCE'] < 0:
            tpu2program4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2program4tpu: " + " ** " + str(tpu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # 
             
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            tpu2program4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2program4tpu: " + " ** " + str(tpu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; tpu2program4xi-yielded-by-release2program4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack  
    
    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70/70) returning" + "; key2program4tpu-returning-by-release2program4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    
    return tpu2program4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2program4tpu" usage = "
#
#   task4retain2program  = {} 
#   task4retain2program  ['COUNT2REFERENCE'] = 1 
#   response4retain2program = {}
#   
#   print('(Step 19A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO RETAIN PROGRAM OF TPU DEVICE ', time4xi())    
#   # 
#   try:                               
#       response4retain2program = retain2program4tpu(task4retain2program)
#       print('(Step 19B/700) OF TPU2PROGRAM4XI5TEST - RETAINED PROGRAM OF TPU DEVICE ', time4xi()) 
#   except:
#       print('(Step 19Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO RETAIN PROGRAM OF TPU DEVICE ', time4xi())                                                                                                  
#   try:
#       print('(Step 19C/700) OF TPU2PROGRAM4XI5TEST COUNT2REFERENCE: ', response4retain2program ['COUNT2REFERENCE'])
#       # 2
#   except:
#       print('(Step 19Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RETAIN2PROGRAM ', response4retain2program)  
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2program4tpu(task4retain2tpu=None):

    '''
    retains either tpu2program4xi
    '''
    global tpu2program4xi 
    

    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2program4tpu"
    method2print = "retain2program4tpu"

    message2print = "{(Step 1A/33) started" + "; retain2program4tpu-started: " + str(retain2program4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2program4tpu at 0x0000000004095BC8>
    
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2tpu-to-retain2program4tpu: " + str(task4retain2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 
    
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2tpu is None:
        task4retain2tpu = {}
        task4retain2tpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2tpu-set-by-retain2program4tpu to: " + str(task4retain2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass

    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2tpu.keys():
        pass
    else:
        task4retain2tpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2program4tpu to: " + str(task4retain2tpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2tpu['COUNT2REFERENCE'] < 1:
        task4retain2tpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2program4tpu to: " + str(task4retain2tpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE PROGRAM IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if tpu2program4xi is None:
            tpu2program4xi = create2programwithsource4tpu()
            message2print = "{(Step 4A/70) created" + "; tpu2program4xi-created-by-retain2program4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            tpu2program4xi = create2programwithsource4tpu()
            message2print = "{(Step 4B/70) created" + "; tpu2program4xi-created-by-retain2program4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            tpu2program4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; tpu2program4xi-yielded-by-retain2program4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        tpu2program4xi['COUNT2REFERENCE'] = tpu2program4xi['COUNT2REFERENCE'] + task4retain2tpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2program4tpu: " + str(tpu2program4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1            
        if tpu2program4xi['COUNT2REFERENCE'] <= 0:
            tpu2program4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2program4tpu: " + " ** " + str(tpu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            tpu2program4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2program4tpu: " + " ** " + str(tpu2program4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; tpu2program4xi-yielded-by-retain2program4tpu: " + str(tpu2program4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack     
     
    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70/70) returning" + "; key2program4tpu-returning-by-retain2program4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    
    return tpu2program4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2programinfo4tpu" usage = "
#
#   print('(Step 3B1/700) OF TPU2PROGRAM4XI5TEST - TRYING TO GET SOME PROGRAM BINARY INFO FOR TPU ', time4xi())     
#   task4getinfo2program = {}
#   task4getinfo2program ['FUNCTION2CALLBACK'] = 'Yes'
#   task4getinfo2program ['COUNT4BINARY'] = 'Yes'
#   task4getinfo2program ['DIRECTORY4BINARY'] = 'Yes'
#   task4getinfo2program ['CODE4BINARY'] = 'Yes'
#   task4getinfo2program ['SIZE4BINARY'] = 'Yes'
#   task4getinfo2program ['LANGUAGE4BINARY'] = 'Yes'
#   task4getinfo2program ['COUNT2REFERENCE'] = 'Yes'
#   task4getinfo2program ['CODE4BINARY'] = 'Yes'
#   try:
#       response4programinfo2tpu = get2programinfo4tpu(task4getinfo2program) 
#       print('(Step 3B2/700) OF TPU2PROGRAM4XI5TEST - GOT SOME PROGRAM BINARY INFO FOR TPU ', time4xi())
#   except:
#       print('(Step 3Z3/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO GET SOME PROGRAM BINARY INFO FOR TPU', time4xi())                  
#   try:                                                                                          
#       keys2response = response4programinfo2tpu.keys()
#       print('COUNT2REFERENCE:', response4programinfo2tpu ['COUNT2REFERENCE'])   
#       print('COUNT4BINARY:', response4programinfo2tpu ['COUNT4BINARY'])
#       print('DIRECTORY4BINARY:', response4programinfo2tpu ['DIRECTORY4BINARY']) 
#       print('FUNCTION2CALLBACK:', response4programinfo2tpu ['FUNCTION2CALLBACK'])   
#       print('LANGUAGE4BINARY:', response4programinfo2tpu ['LANGUAGE4BINARY'])
#       print('SIZE4BINARY:', response4programinfo2tpu ['SIZE4BINARY'])                
#       # 
#       keys2source = response4programinfo2tpu['CODE4BINARY'].keys()
#       for key2source in keys2source:
#           print('BINARY: ', key2source)
#           print(response4programinfo2tpu ['CODE4BINARY'] [key2source] [1])
#           # COUNT2REFERENCE: 1
#           # COUNT4BINARY: 1
#           # DIRECTORY4BINARY: ...//__pycache__//
#           # FUNCTION2CALLBACK: <function trace2program4tpu at 0x0000000004144AC8>
#           # LANGUAGE4BINARY: Python
#           # SIZE4BINARY: {'...//__pycache__//tpu2image4xi.cpython-33.pyc': 1006}
#           # BINARY:  ...//__pycache__//tpu2image4xi.cpython-33.pyc
#           # b'l\x0c\r\n'                       
#   except:
#       print('(Step 3Z4/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4programinfo2tpu : ', response4programinfo2tpu)
#   
#">

# -------------------------------------------------------------------------------------------------   
def get2programinfo4tpu(task4getinfo2program=None):

    '''
    gets tpu2program4xi and tpu2program4xi
    '''
    
    global tpu2program4xi

    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2programinfo4tpu"
    method2print = "get2programinfo4tpu"

    message2print = "{(Step 0A/33) started" + "; get2programinfo4tpu-started: " + str(get2programinfo4tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2programinfo4tpu at 0x0000000003A98BC8>
                                                                
    
    message2print = "{(Step 0B/33) passed" + "; task4getinfo2program-passed-to-get2programinfo4tpu: " + str(task4getinfo2program) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # 
    
    
    tpu2program = {}
    try: 
        if tpu2program4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE SOURCE DATA FOR PROGRAM FOR TPU           
            # -------------------------------------------------------------------------------------           
            tpu2program = create2programwithsource4tpu()
            message2print = "{(Step 1B2/33) initialized" + "; tpu2program-initialized-by-get2programinfo4tpu to: " + str(tpu2program)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B3/33) using" + "; tpu2program4xi-using-by-get2programinfo4tpu: " + str(tpu2program4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF PROGRAM INFO OF TPU          
        # -------------------------------------------------------------------------------------     
        if task4getinfo2program is None:
            task4getinfo2program = {}
            tpu2program = tpu2program4xi
            message2print = "{(Step 1C/33) using" + "; tpu2program-got-by-get2programinfo4tpu: " + str(tpu2program)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys2task4programinfo = task4getinfo2program.keys()
        keys2task = task4getinfo2program.keys()
            
        # -------------------------------------------------------------------------------------
        # SOURCE OR BINARY PROGRAM INFO OF TPU          
        # -------------------------------------------------------------------------------------         
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2program4tpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                 
                tpu2program [key2task] = tpu2program4xi[key2task]
                
                message2print = "{(Step 2D3/33) set" + "; info-of-tpu2program-set-by-get2programinfo4tpu to: " + str(tpu2program [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2program4tpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2program-using-by-get2program4tpu: " + str(task4getinfo2program)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; tpu2program4xi-using-by-get2program4tpu: " + str(tpu2program4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE PROGRAM WITH SOURCE DATA           
            # -------------------------------------------------------------------------------------           
            tpu2program = create2programwithsource4tpu()
            
            message2print = "{(Step 1F/33) initialized" + "; tpu2program-initialized-by-get2programinfo4tpu to: " + str(tpu2program)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if task4getinfo2program is None:
                tpu2program = tpu2program4xi
                message2print = "{(Step 1G/33) set" + "; tpu2program-set-by-get2programinfo4tpu to: " + str(tpu2program)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            else:
                keys2task = task4getinfo2program.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2program4tpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        tpu2program [key2task] = tpu2program4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-tpu2program -set-by-get2programinfo4tpu to: " + str(tpu2program  [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2program4tpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                                                            
        except:
            tpu2program = {}
            message2print = "{(Step 1Z3/70) yielded" + "; tpu2program-yielded-by-get2program4tpu: " + str(tpu2program) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
     
    keys2program4tpu = tpu2program4xi.keys()
    for key2program4tpu in keys2program4tpu:
        message2print = "{(Step 70/70) returning" + "; key2program4tpu-returning-by-get2programinfo4tpu: " + str(key2program4tpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # CODE2ERROR ...
        
    return tpu2program
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2buildinfo4tpu" usage = "
#
#   response4get2buildinfo = get2buildinfo4tpu()
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2buildinfo4tpu(task4getinfo2build=None):

    '''
    gets tpu2build4xi and tpu2build4xi
    '''
    
    global tpu2build4xi

    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2buildinfo4tpu"
    method2print = "get2buildinfo4tpu"

    message2print = "{(Step 0A/33) started" + "; get2buildinfo4tpu-started: " + str(get2buildinfo4tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
                                                                
    
    message2print = "{(Step 0B/33) passed" + "; task4getinfo2build-passed-to-get2buildinfo4tpu: " + str(task4getinfo2build) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # 
    
    
    tpu2build = {}
    try: 
        if tpu2build4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE SOURCE DATA FOR PROGRAM FOR TPU           
            # -------------------------------------------------------------------------------------           
            tpu2build = {}
            message2print = "{(Step 1B2/33) initialized" + "; tpu2build-initialized-by-get2buildinfo4tpu to: " + str(tpu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B3/33) using" + "; tpu2build4xi-using-by-get2buildinfo4tpu: " + str(tpu2build4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF PROGRAM INFO OF TPU          
        # -------------------------------------------------------------------------------------     
        if task4getinfo2build is None:
            task4getinfo2build = {}
            tpu2build = tpu2build4xi
            message2print = "{(Step 1C/33) using" + "; tpu2build-got-by-get2buildinfo4tpu: " + str(tpu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys2task4buildinfo = task4getinfo2build.keys()
        keys2task = task4getinfo2build.keys()
            
        # -------------------------------------------------------------------------------------
        # BUILD INFO OF TPU          
        # -------------------------------------------------------------------------------------         
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2build4tpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                 
                tpu2build [key2task] = tpu2build4xi[key2task]
                
                message2print = "{(Step 2D3/33) set" + "; info-of-tpu2build-set-by-get2buildinfo4tpu to: " + str(tpu2build [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2build4tpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2build-using-by-get2build4tpu: " + str(task4getinfo2build)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; tpu2build4xi-using-by-get2build4tpu: " + str(tpu2build4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE BUILD           
            # -------------------------------------------------------------------------------------           
            tpu2build = {}
            message2print = "{(Step 1F/33) initialized" + "; tpu2build-initialized-by-get2buildinfo4tpu to: " + str(tpu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if task4getinfo2build is None:
                tpu2build = tpu2build4xi
                message2print = "{(Step 1G/33) set" + "; tpu2build-set-by-get2buildinfo4tpu to: " + str(tpu2build)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            else:
                keys2task = task4getinfo2build.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2build4tpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        tpu2build [key2task] = tpu2build4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-tpu2build -set-by-get2buildinfo4tpu to: " + str(tpu2build  [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2build4tpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                                                            
        except:
            tpu2build = {}
            message2print = "{(Step 1Z3/70) yielded" + "; tpu2build-yielded-by-get2build4tpu: " + str(tpu2build) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; tpu2build-returning-by-get2buildinfo4tpu: " + str(tpu2build) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 
    
    return tpu2build
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "link2program4tpu" usage = "
#     
#   print('(Step 2AB2/700) OF TPU2PROGRAM4XI5TEST - TRYING TO LINK INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())           
#   try:
#       response4link = link2program4tpu()
#       print('(Step 2BB2/700) OF TPU2PROGRAM4XI5TEST - LINKED INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi()) 
#   except:
#       print('(Step 2Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO LINK INSIDE PROGRAM OBJECT WITH FUNCTION *** ', time4xi())                                                                     
#   try:
#       print('(Step CC2/700) OF TPU2PROGRAM4XI5TEST - response4link:') 
#       keys2response = response4link.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4link[key2response])
#           # tpu2sampler4xi5test.py  :  <Process4Xi(Process4Xi-1, stopped[1])>
#   except:
#       print('(Step 2Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4link *** ', response4link)
#   tailor2trace4program(t2xi)                                                                              # TAILOR TRACE AGAIN - TARGET FUNCTIONS OF ACTIVATED PROCESSES MIGHT HAVE RESET TRACE ! 
#          
#">

# -------------------------------------------------------------------------------------------------   
def link2program4tpu(task4program2tpu=None):

    '''
    links program for TPU
    '''
    global tpu2program4xi                                                                                           # Program of TPU    
    global import2use4tpu                                                                                           # Imports of TPU
    global target2function4tpu                                                                                      # Target function of Process4Xi
       
                
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "link2program4tpu" 
    method2print = "link2program4tpu"

    message2print = "{(Step 1A/70) started" + "; link2program4tpu-started: " + str(link2program4tpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function link2program4tpu at 0x0000000004332448>
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2tpu-to-link2program4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None

    code2ack  = ' '
    code2nack = ' '    
    
    process2program4tpu = {}                                                                                       # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4program2tpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER
            # -----------------------------------------------------------------------------------------    
            pass                                                                       
        else:
            # -----------------------------------------------------------------------------------------
            # PARAMETERS GIVEN ARE OMITTED 
            # -----------------------------------------------------------------------------------------         
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2tpu-yielded-by-link2program4tpu: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
       
        code2nack  = code2nack + '1Z1-' 

    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - LINK SOURCES INSIDE PROGRAM OBJECT FOR TPU ! 
    # ---------------------------------------------------------------------------------------------    
    process2program4tpu = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; process2program4tpu-initialized-by-link2program4tpu to: " + str(process2program4tpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}

    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) - SOURCE CODE WITHOUT EOL CHARACTERS                  
        # ---------------------------------------------------------------------------------------------             
        keys2source4tpu = tpu2program4xi['CODE4SOURCE'].keys()
        message2print = "{(Step 30A/70) using" + "; keys2source4tpu-using-by-link2program4tpu: " + str(keys2source4tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # dict_keys([folder2lib4tpu + 'tpu2sampler4xi5test.py'])
        
        unit4processing = "tpu"
        
        for key2source4tpu in keys2source4tpu:
            index2search=str.find(key2source4tpu,unit4processing) 
            if index2search == -1:
                pass
            else:
                target2link = key2source4tpu[index2search:]                                                          # tpu2sampler4xi5test.py
                index2search=str.find(target2link,".py")
                if index2search == -1:
                    pass
                else:
                    target2link4tpu = target2link[:index2search]                                                  
                    message2print = "{(Step 32A/70) set" + "; target2link4tpu-set-by-link2program4tpu to: " + str(target2link4tpu)  + "; time: " + time4xi() + "}"  
                    print4xi(class2print, method2print, message2print)
                    # tpu2sampler4xi5test

                    p2source4tpu = None    
                    try:
                        # ---------------------------------------------------------------------------------------------    
                        # IMPORT TARGET FUNCTION FROM ITS MODULE AS TARGET2FUNCTION4TPU 
                        # ---------------------------------------------------------------------------------------------    
      
                        try:
                            import2use4tpu.get(target2link4tpu, handle2error4tpu) ()
                            
                            message2print = "{(Step 32B/70) imported" + "; target2link4tpu-imported-by-link2program4tpu: " + str(target2link4tpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # tpu2sampler4xi5test
                            code2ack  = code2ack + '32B-'
                        except:
                            message2print = "{(Step 32Z1/70) failed" + "; import2use4tpu-failed-by-link2program4tpu: " + str(import2use4tpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)                            
                            # 
                            message2print = "{(Step 32Z2/70) used" + "; target2link4tpu-used-by-link2program4tpu: " + str(target2link4tpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # 
                            code2nack  = code2nack + '32Z2-'
                                    
                        # ---------------------------------------------------------------------------------------------    
                        # CREATE PROCESS WITH TARGET2FUNCTION4TPU
                        # ---------------------------------------------------------------------------------------------                          
                        p2tpu = Process4Xi(target=target2function4tpu, args=())                                
                        message2print = "{(Step 33A/70) created" + "; p2tpu-created-by-link2program4tpu: " + str(p2tpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # <Process4Xi(Process4Xi-2, initial)>
                        
                        process2program4tpu[key2source4tpu] = p2tpu
                        
                        code2ack  = code2ack + '33A-'
                        try:
                            # ---------------------------------------------------------------------------------------------    
                            # START PROCESS
                            # --------------------------------------------------------------------------------------------- 
                            p2tpu.start()
                            message2print = "{(Step 34A/70) started" + "; p2tpu-started-by-link2program4tpu: " + str(p2tpu.name) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # Process4Xi-2
                            
                            try:
                                # ---------------------------------------------------------------------------------------------    
                                # STATUS OF PROCESS
                                # ---------------------------------------------------------------------------------------------                             
                                status2process4tpu = p2tpu.is_alive()
                                message2print = "{(Step 35A/70) got" + "; status-got-by-link2program4tpu: " + str(status2process4tpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                # True
                                
                                try:
                                    # ---------------------------------------------------------------------------------------------    
                                    # RUN PROCESS
                                    # ---------------------------------------------------------------------------------------------                                  
                                    p2tpu.run()
                                    message2print = "{(Step 36A/70) run" + "; p2tpu-run-by-link2program4tpu: " + str(p2tpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)
                                    # <Process4Xi(Process4Xi-1, stopped[1])>
                                except:
                                    message2print = "{(Step 36Z/70) failed" + "; run-failed-by-link2program4tpu: " + str(p2tpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)  
                                    code2nack  = code2nack + '36Z-'            
                                    # 
                            except:
                                message2print = "{(Step 35Z/70) failed" + "; status-failed-by-link2program4tpu: " + str(p2tpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)  
                                # 
                                code2nack  = code2nack + '35Z-'                                   
                                
                        except:
                            message2print = "{(Step 33Z/70) failed" + "; start-failed-by-link2program4tpu: " + str(p2tpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)  
                            code2nack  = code2nack + '33Z-'     
                        
                    except:
                        message2print = "{(Step 32Z/70) failed" + "; p2source4tpu-failed-by-link2program4tpu: " + str(p2source4tpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        code2nack  = code2nack + '32Z-' 
                                                                                    

 
        message2print = "{(Step 38A/70) set" + "; process2program4tpu-set-by-link2program4tpu to: " + str(process2program4tpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '38A-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; source-yielded-by-link2program4tpu: " + str(tpu2program4xi['CODE4SOURCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'     

                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    tpu2program4xi ['ACK2RETURN']  = code2ack
    tpu2program4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; process2program4tpu-returning-by-link2program4tpu: " + str(process2program4tpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    return process2program4tpu
            
#</function4xi> 
 
                                        
# =================================================================================================
#<class4xi class = "TPU2Program4Xi" base = "object" child = "None"
#   synopsis = "offers  methods for TPU programs" usage = "Program4Xi to TPU"
#>
# =================================================================================================
class TPU2Program4Xi(object): 
    ''' 
    TPU2Program4Xi offers  methods for TPU programs
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2tpu = cpu_corecount4xi()
    #"
    #/>
       
       
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (

            'add2programwithbinary',           # ACK 5.6.1   p132            
            'add2programwithsource',           # ACK 5.6.1   p133  
                                
            'build2program',                   # ACK 5.6.2   p137      

            'compile2program',                 # ACK 5.6.3   p139            
            'create2programwithsource',        # ACK 5.6.1   p132 
            'create2programwithbinary',        # ACK 5.6.1   p133 
            'create2programwithbuiltinkernels',# NACK 5.6.1  p135   
                       
            'delete2programwithbinary',        # ACK  
            'delete2programwithsource',        # ACK               
            
            'fetch2programbinary',             # ACK
            'fetch2programsource',             # ACK
            
            'get2programinfo',                 # ACK 5.6.7   p150 
            'get2programbuildinfo',            # NACK 5.6.7  p153
            
            'link2program',                    # ACK 5.6.3   p142                          
            'release2program',                 # ACK 5.6.1   p136                                         
            'retain2program',                  # ACK 5.6.1   p136
 
            'set2programwithsource',           # ACK
            'set2programwithbinary',           # ACK            
            'set2programcallback',             # ACK 
                              
            'trace2program',                   # ACK
            'unload2compiler',                 # NACK 5.6.6  p149 
        
            'sense',                           # ACK
            'reason',                          # ACK
            'ace')                             # ACK   
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-TPU2Program4Xi" usage = "
    # 
    #    print('(Step 110A/700) OF TPU2PROGRAM4XI5TEST - CREATING PROGRAM4TPU OBJECT ', time4xi())        
    #    try:
    #        # ---------------------------------------------------------------------------------------------
    #        # CREATE WITH OBJECT                  
    #        # ---------------------------------------------------------------------------------------------        
    #        programobject2tpu = TPU2Program4Xi()
    #        print('(Step 110B/700) OF TPU2PROGRAM4XI5TEST - CREATED PROGRAM4TPU OBJECT: ', programobject2tpu)                                  
    #        # <tpu2program4xi.TPU2Program4Xi object at 0x0000000003F69198>
    #            
    #        except:
    #    print('(Step 110Z/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM4TPU OBJECT')
    # 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4program2tpu = None):
        '''
        _init__ method initializes TPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "TPU2Program4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-TPU2Program4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 

        message2print = "{(Step 1B/33) borning" + "; tpu2program2xi-of-TPU2Program4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-tpu2program2xi-of-TPU2Program4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2tpu-sharing-by-tpu2program2xi-of-TPU2Program4Xi: " + str(TPU2Program4Xi.classvariable2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #         
                                
        message2print = "{(Step 1E/33) passed" + "; task4program2tpu-passed-to-init-of-TPU2Program4Xi: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #                       
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33) - source!              
        # -----------------------------------------------------------------------------------------
    
        message2print = "{(Step 2A/33) passing" + "; task4program2tpu-passing-to-create2programwithsource4tpu-by-init-of-TPU2Program4Xi: " + str(task4program2tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
                
        tpu2program2xi = create2programwithsource4tpu(task4program2tpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2programwithsource4tpu-performed-by-init-of-TPU2Program4Xi: " + str(create2programwithsource4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2programwithsource4tpu at 0x0000000003A98D48>
       
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; tpu2program2xi-of-TPU2Program4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # tpu2program2xi-of-TPU2Program4Xi-activated
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-tpu2program2xi-of-TPU2Program4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2tpu-shared-by-tpu2program2xi-of-TPU2Program4Xi: " + str(TPU2Program4Xi.classvariable2tpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4         
       
        message2print = "{(Step 3D/33) returning" + "; init-of-TPU2Program4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "add2programwithbinary-of-TPU2Program4Xi" usage = "
    #  
    #   print('(Step 132A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO ADD BINARY INTO PROGRAM OBJECT WITH METHOD ', time4xi())         
    #   try:
    #       task4add2program = {}
    #       task4add2program ['CODE4BINARY'] = {}
    #       task4add2program ['SIZE4BINARY'] = {}            
    #       # NOT OK - EXISTS
    #       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'] = {}
    #       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
    #       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
    #       task4add2program ['SIZE4BINARY'] [folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc'])
    #                
    #       # OK
    #       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] = {}
    #       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] [1] = b'l\x0c\r\n'
    #       task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] [2] = b'l\x0c\z\n' 
    #       task4add2program ['SIZE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] = len(task4add2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'])
    #       
    #       response4add2programwithbinary = programobject2tpu.add2programwithbinary(task4add2program) 
    #       print('(Step 132B/700) OF TPU2PROGRAM4XI5TEST - ADDED BINARY INTO PROGRAM OBJECT WITH METHOD ', time4xi())
    #   except:
    #       print('(Step 132Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO ADD BINARY INTO PROGRAM OBJECT WITH METHOD *** ', time4xi())
    #   try:                                                                                          
    #       keys2response = response4add2programwithbinary.keys()
    #       print('COUNT2REFERENCE:', response4add2programwithbinary ['COUNT2REFERENCE'])   
    #       print('COUNT4BINARY:', response4add2programwithbinary ['COUNT4BINARY'])
    #       print('DIRECTORY4BINARY:', response4add2programwithbinary ['DIRECTORY4BINARY']) 
    #       print('FUNCTION2CALLBACK:', response4add2programwithbinary ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4BINARY:', response4add2programwithbinary ['LANGUAGE4BINARY'])
    #       print('SIZE4BINARY:', response4add2programwithbinary ['SIZE4BINARY'])                
    #       # 
    #       keys2source = response4add2programwithbinary['CODE4BINARY'].keys()
    #       for key2source in keys2source:
    #           print('BINARY: ', key2source)
    #           print(response4add2programwithbinary ['CODE4BINARY'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4BINARY: 2
    #       # DIRECTORY4BINARY: ... //__pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4tpu at 0x000000000419BBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: {'...//__pycache__//tpu2image4xi.cpython-33.pyc': 1006, '...//__pycache__//tpu3image4xi.cpython-33.pyc': 2}
    #       # BINARY:  ...__pycache__//tpu2image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'
    #       # BINARY:  ...//__pycache__//tpu3image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'       
    #   except:
    #       print('(Step 132Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4ADD2PROGRAMWITHBINARY : ', response4add2programwithbinary) 
    #  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def add2programwithbinary(self, task4add2programwithbinary = None):
        '''
        sets program TPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "add2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; add2programwithbinary-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:44:02 11/22/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4add2programwithbinary-passed-to-add2programwithbinary-of-TPU2Program4Xi: " + str(task4add2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE4BINARY': {folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}, 
        #                  folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}}, 
        #  'SIZE4BINARY': {folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc': 2, 
        #                  folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc': 2}
        # }

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4add2programwithbinary-passing-to-add2programwithbinary4tpu-by-add2programwithbinary-of-TPU2Program4Xi: " + str(task4add2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE4BINARY': {folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}, 
        #                  folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc': {1: b'l\x0c\r\n', 2: b'l\x0c\\z\n'}}, 
        #  'SIZE4BINARY': {folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc': 2, 
        #                  folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc': 2}
        
        response4add2programwithbinary = add2programwithbinary4tpu(task4add2programwithbinary)

        message2print = "{(Step 2B/33) performed" + "; add2programwithbinary4tpu-performed-by-add2programwithbinary-of-TPU2Program4Xi: " + str(add2programwithbinary4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function add2programwithbinary4tpu at 0x00000000040EDCC8>
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4add2programwithbinary-returning-by-add2programwithbinary-of-TPU2Program4Xi: " + str(response4add2programwithbinary) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 1 ...
        
                
        return response4add2programwithbinary
                                
                            
    #</methodize4xi>    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "add2programwithsource-of-TPU2Program4Xi" usage = "
    #  
    #   print('(Step 122A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO ADD SOURCE INTO PROGRAM OBJECT WITH CLASS ', time4xi())          
    #   try:
    #       task4add2program = {}
    #       # 
    #       task4add2program ['CODE4SOURCE'] = {}
    #       task4add2program ['SIZE4SOURCE'] = {}
    #                   
    #       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] = {}
    #       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] [1] = '... profile = "33" />\n'
    #       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] [2] = '... program = "tpu2buffer4xi.py" />\n'
    #       task4add2program ['SIZE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'])
    #       # 
    #       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2sampler4xi5test.py'] = {}
    #       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2sampler4xi5test.py'] [1] = '... profile = "33" />\n'
    #       task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2sampler4xi5test.py'] [2] = '... program = "tpu2sampler4xi5test.py" />\n'
    #       
    #       task4add2program ['SIZE4SOURCE'] [folder2lib4tpu + 'tpu2sampler4xi5test.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2sampler4xi5test.py'])            
    #       task4add2program ['SIZE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] = len(task4add2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'])            
    #       
    #       response4add2programwithsource = programobject2tpu.add2programwithsource(task4add2program)
    #       print('(Step 122B/700) OF TPU2PROGRAM4XI5TEST - ADDED SOURCE INTO PROGRAM OBJECT WITH CLASS ', time4xi())    
    #   except:
    #       print('(Step 122Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO ADD SOURCE INTO PROGRAM OBJECT WITH CLASS *** ', time4xi())
    #   try:                                                                                      
    #       print('COUNT4SOURCE: ', response4add2programwithsource['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE: ', response4add2programwithsource['DIRECTORY4SOURCE'])
    #       print('LANGUAGE4SOURCE: ', response4add2programwithsource['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE: ', response4add2programwithsource['SIZE4SOURCE'])
    #       keys2source = response4add2programwithsource['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4add2programwithsource ['CODE4SOURCE'] [key2source] [1])
    #       # COUNT4SOURCE:  2
    #       # DIRECTORY4SOURCE:  OK
    #       # LANGUAGE4SOURCE:  Python
    #       # SIZE4SOURCE:  {' ... //tpu2buffer4xi.py': 2, '... tpu2sampler4xi5test.py': 1337}
    #       # SOURCE:  ... //tpu2buffer4xi.py
    #       # ... profile = "33" />
    #       # 
    #       # SOURCE:  ... tpu2sampler4xi5test.py
    #       # ... profile = "11" />                              
    #   except:
    #       print('(Step 1222Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4ADD2PROGRAMWITHSOURCE: ', response4add2programwithsource)  
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def add2programwithsource(self, task4add2programwithsource = None):
        '''
        sets program TPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "add2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; add2programwithsource-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:44:01 11/22/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4add2programwithsource-passed-to-add2programwithsource-of-TPU2Program4Xi: " + str(task4add2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': {1: '... profile = "33" />\n', 2: '... program = "tpu2buffer4xi.py" />\n'}, 
        #                  folder2lib4tpu + 'tpu2sampler4xi5test.py': {1: '... profile = "33" />\n', 2: '... program = "tpu2sampler4xi5test.py" />\n'}}, 
        #  'SIZE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': 2, 
        #                  folder2lib4tpu + 'tpu2sampler4xi5test.py': 2}
        #}

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4add2programwithsource-passing-to-add2programwithsource4tpu-by-add2programwithsource-of-TPU2Program4Xi: " + str(task4add2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': {1: '... profile = "33" />\n', 2: '... program = "tpu2buffer4xi.py" />\n'}, 
        #                  folder2lib4tpu + 'tpu2sampler4xi5test.py': {1: '... profile = "33" />\n', 2: '... program = "tpu2sampler4xi5test.py" />\n'}}, 
        #  'SIZE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': 2, 
        #                  folder2lib4tpu + 'tpu2sampler4xi5test.py': 2}
        #}; time: 00:44:01 11/22/11 Pacific Standard Time}
        
        response4add2programwithsource = add2programwithsource4tpu(task4add2programwithsource)

        message2print = "{(Step 2B/33) performed" + "; add2programwithsource4tpu-performed-by-add2programwithsource-of-TPU2Program4Xi: " + str(add2programwithsource4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function add2programwithsource4tpu at 0x00000000040DDD48>
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4add2programwithsource-returning-by-add2programwithsource-of-TPU2Program4Xi: " + str(response4add2programwithsource) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'LANGUAGE4SOURCE': 'Python', 
        #  'COUNT4SOURCE': 2, 
        #  'SIZE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': 2, 
        #                  folder2lib4tpu + 'tpu2sampler4xi5test.py': 1337}, 
        #  'DATA2CALLBACK': ...
         
        return response4add2programwithsource
                                
                            
    #</methodize4xi>    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "build2program-of-TPU2Program4Xi" usage = "
    # 
    #   task4build2program  = {}
    #   task4build2program  ['PROGRAM'] = folder2lib4tpu + 'tpu2sampler4xi.py'
    #   task4build2program  ['OPTIONS'] = {}
    #   task4build2program  ['OPTIONS'] ['MATH']          = 'SINGLE-PRECISION-CONSTANT'
    #   task4build2program  ['OPTIONS'] ['OPTIMIZATION']  = 'DISABLE'
    #   task4build2program  ['OPTIONS'] ['VERSION']       = 'CL1.1'
    #   task4build2program  ['OPTIONS'] ['WARNINGS']      = 'INHIBIT'
    #   print('(Step 240A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO BUILD PROGRAM OF TPU DEVICE ', time4xi())    
    #   try:                               
    #       response4build2program = programobject2tpu.build2program(task4build2program) 
    #       print('(Step 240B/700) OF TPU2PROGRAM4XI5TEST - BUILT PROGRAM OF TPU DEVICE ', time4xi())
    #   except:
    #       print('(Step 240Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO BUILD PROGRAM OF TPU DEVICE ', time4xi())
    #   try:                                                                                                     
    #       keys2response = response4build2program.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4build2program[key2response])
    #   # ACK2RETURN  :   11A1-12A1-
    #   # NACK2RETURN  :   13Z1-14Z1-
    #   # OPTIONS  :  {'VERSION': 'CL1.1', 'OPTIMIZATION': 'DISABLE', 'MATH': 'SINGLE-PRECISION-CONSTANT', 'WARNINGS': 'INHIBIT'}
    #   # PROGRAM  :  folder2lib4tpu + tpu2sampler4xi.py                
    #   except:
    #       print('(Step 240Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT ', response4build2program)              
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def build2program(self, task4build = None):
        '''
        build2program method builds program objects of TPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "build2program"
                     
        message2print = "{(Step 1A/33) started" + "; build2program-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4build-passed-to-create-of-TPU2Program4Xi: " + str(task4build) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4build-passing-to-build2program4tpu-by-build2program-of-TPU2Program4Xi: " + str(task4build) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #    
  
        tpu2program2xi = build2program4tpu(task4build)

        message2print = "{(Step 2B/33) performed" + "; build2program4tpu-performed-by-build2program-of-TPU2Program4Xi: " + str(build2program4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; tpu2program2xi-returning-by-build2program-of-TPU2Program4Xi: " + str(tpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # 

        return tpu2program2xi
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "compile2program-of-TPU2Program4Xi" usage = "
    # 
    #   print('(Step 121A1/700) OF TPU2PROGRAM4XI5TEST - COMPILING PROGRAM FOR TPU', time4xi())      
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # COMPILE                        
    #       # ---------------------------------------------------------------------------------------------        
    #       response4compile = programobject2tpu.compile2program()
    #       print('(Step 121A2/700) OF TPU2PROGRAM4XI5TEST - COMPILING PROGRAM FOR TPU', time4xi())                                   
    #   except:
    #       print('(Step 121AZ1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO COMPILE PROGRAM FOR TPU')
    #   try:
    #       keys2response4compile = response4compile.keys()
    #       for key2response4compile in keys2response4compile:
    #           print('(Step 121A3/700) OF TPU2PROGRAM4XI5TEST - KEY OF RESPONSE4COMPILE: ', key2response4compile)
    #           # tpu2sampler4xi5test.py 
    #   except:
    #       print('(Step 121AZ2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4COMPILE:' , response4compile)
    #                
    #   tailor2trace4program(t2xi)                  # <-- tailor trace again - target functions of activated processes might have reset trace !
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def compile2program(self, task4program2create = None):
        '''
        compile2program compiles for program object of TPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "compile2program"
                     
        message2print = "{(Step 1A/33) started" + "; compile2program-of-TPU2Program4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 03:09:40 11/23/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-TPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-compile2program4tpu-by-compile2program-of-TPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None
  
        tpu2program2xi = compile2program4tpu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; compile2program4tpu-performed-by-compile2program-of-TPU2Program4Xi: " + str(compile2program4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; tpu2program2xi-returning-by-compile2program-of-TPU2Program4Xi: " + str(tpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return tpu2program2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "link2program-of-TPU2Program4Xi" usage = "
    # 
    #   print('(Step 121C1/700) OF TPU2PROGRAM4XI5TEST - LINKING PROGRAM FOR TPU', time4xi())      
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # LINK                        
    #       # ---------------------------------------------------------------------------------------------        
    #       response4link = programobject2tpu.link2program()
    #       print('(Step 121C2/700) OF TPU2PROGRAM4XI5TEST - COMPILING PROGRAM FOR TPU', time4xi())                                   
    #   except:
    #       print('(Step 121CZ1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO LINK PROGRAM FOR TPU')
    #   try:
    #       keys2response4link = response4link.keys()
    #       for key2response4link in keys2response4link:
    #           print('(Step 121C3/700) OF TPU2PROGRAM4XI5TEST - KEY OF RESPONSE4LINK: ', key2response4link)
    #           # 
    #   except:
    #       print('(Step 121CZ2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4LINK:' , response4link)
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def link2program(self, task4program2create = None):
        '''
        link2program links for program object of TPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "link2program"
                     
        message2print = "{(Step 1A/33) started" + "; link2program-of-TPU2Program4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 04:02:10 11/23/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-TPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-link2program4tpu-by-link2program-of-TPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None
  
        tpu2program2xi = link2program4tpu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; link2program4tpu-performed-by-link2program-of-TPU2Program4Xi: " + str(link2program4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; tpu2program2xi-returning-by-link2program-of-TPU2Program4Xi: " + str(tpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return tpu2program2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2programwithsource-of-TPU2Program4Xi" usage = "
    # 
    #   print('(Step 190A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO CREATE PROGRAM WITH SOURCE FOR TPU ', time4xi()) 
    #   try:
    #       programobject2tpu = TPU2Program4Xi()                            
    #       response4create2programwithsource = programobject2tpu.create2programwithsource()
    #       print('(Step 190B/700) OF TPU2PROGRAM4XI5TEST - CREATED PROGRAM WITH SOURCE FOR TPU ', time4xi())
    #   except:
    #       print('(Step 190Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO CREATE PROGRAM WITH SOURCE FOR TPU ', time4xi())                                    
    #   try:                                                                                      
    #       print('COUNT4SOURCE: ', response4create2programwithsource['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE: ', response4create2programwithsource['DIRECTORY4SOURCE'])
    #       print('LANGUAGE4SOURCE: ', response4create2programwithsource['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE: ', response4create2programwithsource['SIZE4SOURCE'])
    #       keys2source = response4create2programwithsource['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [1])
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [3])
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [4])
    #           print(response4create2programwithsource ['CODE4SOURCE'] [key2source] [5])
    #           
    #       # COUNT4SOURCE:  1
    #       # DIRECTORY4SOURCE:  folder2lib4tpu
    #       # LANGUAGE4SOURCE:  Python
    #       # SIZE4SOURCE:  {folder2lib4tpu + 'tpu2sampler4xi5test.py': 1337}
    #       # SOURCE:  folder2lib4tpu + tpu2sampler4xi5test.py
    #       # ... profile = "11" />
    #       # ... program = "tpu2sampler4xi5test.py">
    #       # ... abstract = "tests tpu2sampler4xi.py"/>
    #       # ... verification = "11" target = "tpu2sampler4xi.py"/>              
    #   except:
    #       print('(Step 190Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4create2programwithsource *** ', response4create2programwithsource)  
    #           
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2programwithsource(self, task4program2create = None):
        '''
        create2programwithsource method creates program objects of TPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "create2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; create2programwithsource-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:16:42 11/17/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-TPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-create2programwithsource4tpu-by-create2programwithsource-of-TPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None   
  
        tpu2program2xi = create2programwithsource4tpu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; create2programwithsource4tpu-performed-by-create2programwithsource-of-TPU2Program4Xi: " + str(create2programwithsource4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2programwithsource4tpu at 0x0000000004024848> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; tpu2program2xi-returning-by-create2programwithsource-of-TPU2Program4Xi: " + str(tpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0, 
        #  'LANGUAGE4SOURCE': 'Python', 
        #  'COUNT4SOURCE': 1, 
        #  'SIZE4SOURCE': {folder2lib4tpu + 'tpu2sampler4xi5test.py': 1337}, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithsource4tpu-failed ; time: 05:16:42 11/17/11 Pacific Standard Time}', 
        # ...

        return tpu2program2xi
                                
    #                        
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2programwithbinary-of-TPU2Program4Xi" usage = "
    # 
    #   print('(Step 130A/700) OF TPU2PROGRAM4XI5TEST - CREATING WITH BINARY PROGRAM FOR TPU', time4xi())      
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # CREATE BINARY                         
    #       # ---------------------------------------------------------------------------------------------        
    #       response4create2programwithbinary = programobject2tpu.create2programwithbinary()
    #       print('(Step 130A/700) OF TPU2PROGRAM4XI5TEST - CREATED WITH BINARY PROGRAM FOR TPU', time4xi())
    #   except:
    #       print('(Step 130Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO CREATE  WITH BINARY PROGRAM FOR TPU')  
    #   try:                                                                                          
    #       keys2response = response4create2programwithbinary.keys()
    #       print('COUNT2REFERENCE:', response4create2programwithbinary ['COUNT2REFERENCE'])   
    #       print('COUNT4BINARY:', response4create2programwithbinary ['COUNT4BINARY'])
    #       print('DIRECTORY4BINARY:', response4create2programwithbinary ['DIRECTORY4BINARY']) 
    #       print('FUNCTION2CALLBACK:', response4create2programwithbinary ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4BINARY:', response4create2programwithbinary ['LANGUAGE4BINARY'])
    #       print('SIZE4BINARY:', response4create2programwithbinary ['SIZE4BINARY'])                
    #       #
    #       keys2source = response4create2programwithbinary['CODE4BINARY'].keys()
    #       for key2source in keys2source:
    #           print('BINARY: ', key2source)
    #           print(response4create2programwithbinary ['CODE4BINARY'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4BINARY: 1
    #       # DIRECTORY4BINARY: folder2lib4tpu + __pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4tpu at 0x000000000415CBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: {'folder2lib4tpu + '__pycache__//tpu2image4xi.cpython-33.pyc': 1006}
    #       # BINARY:  folder2lib4tpu + __pycache__//tpu2image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'
    #   except:
    #       print('(Step 130Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4CREATE2PROGRAMWITHBINARY : ', response4create2programwithbinary)              
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2programwithbinary(self, task4program2create = None):
        '''
        create2programwithbinary method creates program objects of TPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "create2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; create2programwithbinary-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:16:42 11/17/11 Pacific Standard Time
            
        message2print = "{(Step 1B/33) passed" + "; task4program2create-passed-to-create-of-TPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4program2create-passing-to-create2programwithbinary4tpu-by-create2programwithbinary-of-TPU2Program4Xi: " + str(task4program2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None   
  
        tpu2program2xi = create2programwithbinary4tpu(task4program2create)

        message2print = "{(Step 2B/33) performed" + "; create2programwithbinary4tpu-performed-by-create2programwithbinary-of-TPU2Program4Xi: " + str(create2programwithbinary4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2programwithbinary4tpu at 0x00000000040248C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; tpu2program2xi-returning-by-create2programwithbinary-of-TPU2Program4Xi: " + str(tpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0, 
        #  'COUNT2REFERENCE': 1, 
        #  'DIRECTORY4BINARY': folder2lib4tpu + '__pycache__//', 
        # ... 

        return tpu2program2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "delete2programwithbinary-of-TPU2Program4Xi" usage = "
    #  
    #   print('(Step 134A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO DELETE BINARY INSIDE PROGRAM OBJECT WITH METHOD ', time4xi())        
    #   try:
    #       task4delete2program = {}
    #       task4delete2program ['CODE4BINARY'] = {}
    #       task4delete2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] = 'Yes'
    #       task4delete2program ['CODE4BINARY'] [folder2lib4tpu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
    #                task4delete2program ['SIZE4BINARY'] = {}
    #       task4delete2program ['SIZE4BINARY'] [folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'] = 'Yes'
    #       task4delete2program ['SIZE4BINARY'] [folder2lib4tpu + '__pycache__//dummy3image4xi.cpython-33.pyc'] = 'Yes'
    #       
    #       response4delete2programwithbinary = programobject2tpu.delete2programwithbinary(task4delete2program)
    #       print('(Step 134B/700) OF TPU2PROGRAM4XI5TEST - DELETED BINARY INSIDE PROGRAM OBJECT WITH METHOD ', time4xi())                                                                  
    #   except:
    #       print('(Step 134Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO DELETE BINARY INSIDE PROGRAM OBJECT WITH METHOD *** ', time4xi())
    #   try:                                                                                          
    #       keys2response = response4delete2programwithbinary.keys()
    #       print('COUNT2REFERENCE:', response4delete2programwithbinary ['COUNT2REFERENCE'])   
    #       print('COUNT4BINARY:', response4delete2programwithbinary ['COUNT4BINARY'])
    #       print('DIRECTORY4BINARY:', response4delete2programwithbinary ['DIRECTORY4BINARY']) 
    #       print('FUNCTION2CALLBACK:', response4delete2programwithbinary ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4BINARY:', response4delete2programwithbinary ['LANGUAGE4BINARY'])
    #       print('SIZE4BINARY:', response4delete2programwithbinary ['SIZE4BINARY'])                
    #       # 
    #       keys2source = response4delete2programwithbinary['CODE4BINARY'].keys()
    #       for key2source in keys2source:
    #           print('BINARY: ', key2source)
    #           print(response4delete2programwithbinary ['CODE4BINARY'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4BINARY: 1
    #       # DIRECTORY4BINARY: ...//__pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4tpu at 0x000000000412BBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: {'...//__pycache__//tpu2image4xi.cpython-33.pyc': 1006}
    #       # BINARY:  ...__pycache__//tpu2image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'
    #                       
    #   except:
    #       print('(Step 134Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithbinary : ', response4delete2programwithbinary)         
    #                  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def delete2programwithbinary(self, task4delete2programwithbinary = None):
        '''
        sets program TPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "delete2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; delete2programwithbinary-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4delete2programwithbinary-passed-to-delete2programwithbinary-of-TPU2Program4Xi: " + str(task4delete2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4delete2programwithbinary-passing-to-delete2programwithbinary4tpu-by-delete2programwithbinary-of-TPU2Program4Xi: " + str(task4delete2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        response4delete2programwithbinary = delete2programwithbinary4tpu(task4delete2programwithbinary)

        message2print = "{(Step 2B/33) performed" + "; delete2programwithbinary4tpu-performed-by-delete2programwithbinary-of-TPU2Program4Xi: " + str(delete2programwithbinary4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4delete2programwithbinary-returning-by-delete2programwithbinary-of-TPU2Program4Xi: " + str(response4delete2programwithbinary) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return response4delete2programwithbinary
                                
                            
    #</methodize4xi>    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "delete2programwithsource-of-TPU2Program4Xi" usage = "
    #  
    #   print('(Step 124A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO DELETE SOURCE IN PROGRAM OBJECT WITH CLASS ', time4xi())        
    #   try:
    #       task4delete2program = {}
    #       task4delete2program ['CODE4SOURCE'] = {}
    #       task4delete2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] = 'Yes'
    #       task4delete2program ['CODE4SOURCE'] [folder2lib4tpu + 'tpu2dummy4xi.py'] = 'Yes'
    #                task4delete2program ['SIZE4SOURCE'] = {}
    #       task4delete2program ['SIZE4SOURCE'] [folder2lib4tpu + 'tpu2buffer4xi.py'] = 'Yes'
    #       task4delete2program ['SIZE4SOURCE'] [folder2lib4tpu + 'tpu2dummy4xi.py'] = 'Yes'
    #       
    #       response4delete2programwithsource = programobject2tpu.delete2programwithsource(task4delete2program)
    #       print('(Step 124B/700) OF TPU2PROGRAM4XI5TEST - DELETED SOURCE IN PROGRAM OBJECT WITH CLASS ', time4xi())
    #   except:
    #       print('(Step 124Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO DELETE SOURCE IN PROGRAM OBJECT WITH CLASS *** ', time4xi())    
    #   try:                                                                                      
    #       print('COUNT4SOURCE: ', response4delete2programwithsource['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE: ', response4delete2programwithsource['DIRECTORY4SOURCE'])
    #       print('LANGUAGE4SOURCE: ', response4delete2programwithsource['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE: ', response4delete2programwithsource['SIZE4SOURCE'])
    #       keys2source = response4delete2programwithsource['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4delete2programwithsource ['CODE4SOURCE'] [key2source] [1])
    #       # COUNT4SOURCE:  1
    #       # DIRECTORY4SOURCE:  OK 
    #       # LANGUAGE4SOURCE:  Python
    #       # SIZE4SOURCE:  {'... tpu2sampler4xi5test.py': 1337}
    #       # SOURCE:  ... tpu2sampler4xi5test.py
    #       # ... profile = "11" />                        
    #   except:
    #       print('(Step 124Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4delete2programwithsource *** ', response4delete2programwithsource)                            
    #                    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def delete2programwithsource(self, task4delete2programwithsource = None):
        '''
        sets program TPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "delete2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; delete2programwithsource-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:44:01 11/22/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4delete2programwithsource-passed-to-delete2programwithsource-of-TPU2Program4Xi: " + str(task4delete2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': 'Yes', 
        #                  folder2lib4tpu + 'tpu2dummy4xi.py': 'Yes'}, 
        #  'SIZE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': 'Yes', 
        #  folder2lib4tpu + 'tpu2dummy4xi.py': 'Yes'}}

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4delete2programwithsource-passing-to-delete2programwithsource4tpu-by-delete2programwithsource-of-TPU2Program4Xi: " + str(task4delete2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': 'Yes', 
        #                  folder2lib4tpu + 'tpu2dummy4xi.py': 'Yes'}, 
        #  'SIZE4SOURCE': {folder2lib4tpu + 'tpu2buffer4xi.py': 'Yes', 
        #  folder2lib4tpu + 'tpu2dummy4xi.py': 'Yes'}}
        
        response4delete2programwithsource = delete2programwithsource4tpu(task4delete2programwithsource)

        message2print = "{(Step 2B/33) performed" + "; delete2programwithsource4tpu-performed-by-delete2programwithsource-of-TPU2Program4Xi: " + str(delete2programwithsource4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function delete2programwithsource4tpu at 0x00000000040DDE48>
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4delete2programwithsource-returning-by-delete2programwithsource-of-TPU2Program4Xi: " + str(response4delete2programwithsource) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'LANGUAGE4SOURCE': 'Python' ...
        
                
        return response4delete2programwithsource
                                
                            
    #</methodize4xi>    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2programwithsource-of-TPU2Program4Xi" usage = "
    # 
    #   ... prepare task4set2program ...
    #   try: 
    #       response4set2program = programobject2tpu.set2programwithsource(task4set2program)
    #       print('(Step 158B/700) OF TPU2PROGRAM4XI5TEST - SET PROGRAM SOURCE FOR TPU: ', time4xi())    
    #   except:
    #       print('(Step 158Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM SOURCE FOR TPU: ', time4xi())
    #   try:                                                                                          
    #       keys2response = response4set2program.keys()
    #       print('COUNT2REFERENCE:', response4set2program ['COUNT2REFERENCE'])   
    #       print('COUNT4SOURCE:', response4set2program ['COUNT4SOURCE'])
    #       print('DIRECTORY4SOURCE:', response4set2program ['DIRECTORY4SOURCE']) 
    #       print('FUNCTION2CALLBACK:', response4set2program ['FUNCTION2CALLBACK'])   
    #       print('LANGUAGE4SOURCE:', response4set2program ['LANGUAGE4SOURCE'])
    #       print('SIZE4SOURCE:', response4set2program ['SIZE4SOURCE'])                
    #            keys2source = response4set2program['CODE4SOURCE'].keys()
    #       for key2source in keys2source:
    #           print('SOURCE: ', key2source)
    #           print(response4set2program ['CODE4SOURCE'] [key2source] [1])
    #       # COUNT2REFERENCE: 1
    #       # COUNT4SOURCE: 1
    #       # DIRECTORY4SOURCE: OK
    #       # FUNCTION2CALLBACK: <function trace2program4tpu at 0x0000000003AC5BC8>
    #       # LANGUAGE4SOURCE: Python
    #       # SIZE4SOURCE: 1
    #       # SOURCE:  ... //tpu2image4xi5test.py
    #       # ... profile = "11" />        
    #   except:
    #       print('(Step 158Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT response4set2program : ', response4set2program)  
    #       
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2programwithsource(self, task4set2programwithsource = None):
        '''
        sets program TPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "set2programwithsource"
                     
        message2print = "{(Step 1A/33) started" + "; set2programwithsource-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2programwithsource-passed-to-set2programwithsource-of-TPU2Program4Xi: " + str(task4set2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2programwithsource-passing-to-set2programwithsource4tpu-by-set2programwithsource-of-TPU2Program4Xi: " + str(task4set2programwithsource) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        tpu2program2xi = set2programwithsource4tpu(task4set2programwithsource)

        message2print = "{(Step 2B/33) performed" + "; set2programwithsource4tpu-performed-by-set2programwithsource-of-TPU2Program4Xi: " + str(set2programwithsource4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; tpu2program2xi-returning-by-set2programwithsource-of-TPU2Program4Xi: " + str(tpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return tpu2program2xi
                                
                            
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2programwithbinary-of-TPU2Program4Xi" usage = "
    # 
    #   ... prepare task4set2programwithbinary ...
    #   task4set2programwithbinary ['CODE4BINARY'] = None
    #   print('(Step 167A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO SET BINARY OF TASK FOR PROGRAM4TPU  ', time4xi())      
    #   try: 
    #       binary4tpu = {}
    #       key2binary4tpu = folder2lib4tpu + '__pycache__//tpu3image4xi.cpython-33.pyc'
    #       code2binary4tpu = {}
    #       code2binary4tpu [1] = b'l\x0c\r\n'
    #       code2binary4tpu [2] = b'\x00Z\n'
    #       
    #       binary4tpu [key2binary4tpu] = code2binary4tpu
    #                                 
    #       task4set2programwithbinary ['CODE4BINARY'] = binary4tpu                                                                                                              
    #       print('(Step 167B/700) OF TPU2PROGRAM4XI5TEST - SET BINARY OF TASK FOR PROGRAM4TPU TO: ', task4set2programwithbinary ['CODE4BINARY'])                           
    #   except:
    #       print('(Step 167Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO SET BINARY OF TASK FOR PROGRAM4TPU  ', time4xi()) 
    #       
    #   print('(Step 167CB/700) OF TPU2PROGRAM4XI5TEST - TRYING TO SET BINARY PROGRAM FOR TPU: ', time4xi())      
    #   try: 
    #       response4set2program = programobject2tpu.set2programwithbinary(task4set2programwithbinary)
    #       print('(Step 167D/700) OF TPU2PROGRAM4XI5TEST - SET BINARY PROGRAM FOR TPU: ', time4xi())
    #   except:
    #       print('(Step 167Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4SET2PROGRAM: ', response4set2program)                     
    #   try:                                                                                          
    #       keys2response = response4set2program.keys()
    #       print('COUNT2REFERENCE:', response4set2program ['COUNT2REFERENCE']) 
    #   except:
    #       pass
    #   try:        
    #       print('COUNT4BINARY:', response4set2program ['COUNT4BINARY'])
    #   except:
    #       pass
    #   try:        
    #       print('DIRECTORY4BINARY:', response4set2program ['DIRECTORY4BINARY']) 
    #   except:
    #       pass
    #   try:        
    #       print('FUNCTION2CALLBACK:', response4set2program ['FUNCTION2CALLBACK'])   
    #   except:
    #       pass
    #   try:        
    #       print('LANGUAGE4BINARY:', response4set2program ['LANGUAGE4BINARY'])
    #   except:
    #       pass
    #   try:        
    #       print('SIZE4BINARY:', response4set2program ['SIZE4BINARY'])                
    #   except:
    #       pass
    #   try:        
    #            keys2binary = response4set2program['CODE4BINARY'].keys()
    #       for key2binary in keys2binary:
    #           print('BINARY: ', key2binary)
    #           print(response4set2program ['CODE4BINARY'] [key2binary] [1])
    #       #    
    #   except:
    #       pass
    #       # COUNT2REFERENCE: 1
    #       # DIRECTORY4BINARY: ... //__pycache__//
    #       # FUNCTION2CALLBACK: <function trace2program4tpu at 0x0000000003B1FBC8>
    #       # LANGUAGE4BINARY: Python
    #       # SIZE4BINARY: 1
    #       # BINARY:  ... //__pycache__//tpu3image4xi.cpython-33.pyc
    #       # b'l\x0c\r\n'           
    # 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2programwithbinary(self, task4set2programwithbinary = None):
        '''
        sets program TPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "set2programwithbinary"
                     
        message2print = "{(Step 1A/33) started" + "; set2programwithbinary-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:32:44 11/17/11 Pacific Standard Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2programwithbinary-passed-to-set2programwithbinary-of-TPU2Program4Xi: " + str(task4set2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0, 
        #  'LANGUAGE4SOURCE': 'C99', 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C68EF0>, 
        #  'CONTEXT': ...

          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2programwithbinary-passing-to-set2programwithbinary4tpu-by-set2programwithbinary-of-TPU2Program4Xi: " + str(task4set2programwithbinary) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        response4set2programwithbinary = set2programwithbinary4tpu(task4set2programwithbinary)

        message2print = "{(Step 2B/33) performed" + "; set2programwithbinary4tpu-performed-by-set2programwithbinary-of-TPU2Program4Xi: " + str(set2programwithbinary4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4set2programwithbinary-returning-by-set2programwithbinary-of-TPU2Program4Xi: " + str(response4set2programwithbinary) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return response4set2programwithbinary
                                
                            
    #</methodize4xi>    
    

    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2programinfo-of-TPU2Program4Xi" usage = "
    #
    #   print('(Step 106A1/700) OF TPU2PROGRAM4XI5TEST - TRYING TO GET PROGRAM INFO ', time4xi())    
    #   try:                               
    #       response4get2programinfo = programobject2tpu.get2programinfo()                                 
    #                                
    #       keys2response = response4get2programinfo.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4get2programinfo[key2response])
    #           # CODE4SOURCE  :  {folder2lib4tpu + 'tpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: ....
    #           
    #   except:
    #       print('(Step 106Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO GET PROGRAM INFO ', time4xi()) 
    #   
    # 
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2programinfo(self, task4getinfo2program=None):
        '''
        get2programinfo method gets objects of TPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "get2programinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2programinfo-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:01:22 11/17/11 Pacific Standard Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        tpu2program2xi = get2programinfo4tpu(task4getinfo2program)
        
        message2print = "{(Step 2A/33) returning" + "; tpu2program2xi-returning-by-get2programinfo-of-TPU2Program4Xi: " + str(tpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'COUNT2REFERENCE': 1, 
        #  'DIRECTORY4BINARY': folder2lib4tpu + '__pycache__//', 
        #  'LANGUAGE4BINARY': 'Python', 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2programwithbinary4tpu-failed ; time: 05:16:42 11/17/11 Pacific Standard Time}', 
        #  'CLASS': 'create2programwithbinary4tpu', 
        #  'METHOD': 'create2programwithbinary4tpu'}, 
        #  'CODE4BINARY': ...

        return tpu2program2xi
                            
    #</methodize4xi>

   
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2buildinfo-of-TPU2Program4Xi" usage = "
    #
    #   print('(Step 106A1/700) OF TPU2BUILD4XI5TEST - TRYING TO GET BUILD INFO ', time4xi())    
    #   try:                               
    #                               
    #       response4get2buildinfo = programobject2tpu.get2buildinfo()
    #       print('(Step 106A2/700) OF TPU2BUILD4XI5TEST - GOT BUILD INFO ', time4xi())    
    #   except:
    #       print('(Step 106Z1/700) OF TPU2BUILD4XI5TEST *** FAILED TO GET BUILD INFO ', time4xi()) 
    # 
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2buildinfo(self, task4getinfo2build=None):
        '''
        get2buildinfo method gets objects of TPU2Program4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "get2buildinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2buildinfo-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        tpu2build2xi = get2buildinfo4tpu(task4getinfo2build)
        
        message2print = "{(Step 2A/33) returning" + "; tpu2build2xi-returning-by-get2buildinfo-of-TPU2Program4Xi: " + str(tpu2build2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        return tpu2build2xi
                            
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2program-of-TPU2Program4Xi" usage = "
    #
    #   print('(Step 170A/700) OF TPU2PROGRAM4XI5TEST - RELEASING PROGRAM', time4xi())  
    #   task4release2program  = {}
    #   task4release2program  ['COUNT2REFERENCE'] = 1
    #   try:
    #       response4release2program = programobject2tpu.release2program(task4release2program)
    #       print('(Step 170B/700) OF TPU2PROGRAM4XI5TEST - RELEASED PROGRAM', time4xi())
    #   except:
    #       print('(Step 170Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO RELEASE PROGRAM', time4xi())                        
    #       keys2response = response4release2program.keys()
    #   try:
    #       print('(Step 170C/700) OF TPU2PROGRAM4XI5TEST - COUNT2REFERENCE: ', response4release2program['COUNT2REFERENCE'] )
    #       # 0
    #   except:
    #       print('(Step 170Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RELEASE2PROGRAM', response4release2program)  
    #  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2program(self, task4release2program = None):
        '''
        releases TPU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "release2program"
                     
        message2print = "{(Step 1A/33) started" + "; release2program-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2program-passed-to-release2program-of-TPU2Program4Xi: " + str(task4release2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2program-passing-to-release2program4tpu-by-release2program-of-TPU2Program4Xi: " + str(task4release2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        tpu2program2xi = release2program4tpu(task4release2program)

        message2print = "{(Step 2B/33) performed" + "; release2program4tpu-performed-by-release2program-of-TPU2Program4Xi: " + str(release2program4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; tpu2program2xi-returning-by-release2program-of-TPU2Program4Xi: " + str(tpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #               
                
        return tpu2program2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2program-of-TPU2Program4Xi" usage = "
    #
    #   print('(Step 180A/700) OF TPU2PROGRAM4XI5TEST - RETAINING PROGRAM ', time4xi())
    #   task4retain2program  = {}
    #   task4retain2program  ['COUNT2REFERENCE'] = 1
    #   try:
    #       response4retain2program = programobject2tpu.retain2program(task4retain2program)
    #       print('(Step 180B/700) OF TPU2PROGRAM4XI5TEST - RETAINED PROGRAM ', time4xi())
    #   except:
    #       print('(Step 180Z1/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO RETAIN PROGRAM ', time4xi())                    
    #   try:
    #       print('(Step 180C/700) OF TPU2PROGRAM4XI5TEST COUNT2REFERENCE: ', response4retain2program ['COUNT2REFERENCE'])
    #       # 2
    #   except:
    #       print('(Step 180Z2/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO PRINT RESPONSE4RETAIN2PROGRAM ', response4retain2program)
    # 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2program(self, task4retain2program = None):
        '''
        retains TPU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "retain2program"
                     
        message2print = "{(Step 1A/33) started" + "; retain2program-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2program-of-TPU2Program4Xi: " + str(task4retain2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2program4tpu-by-retain-of-TPU2Program4Xi: " + str(task4retain2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
  
        tpu2program2xi = retain2program4tpu(task4retain2program)

        message2print = "{(Step 2B/33) performed" + "; retain2program4tpu-performed-by-retain2program-of-TPU2Program4Xi: " + str(retain2program4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; tpu2program2xi-returning-by-retain2program-of-TPU2Program4Xi: " + str(tpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
            
                
        return tpu2program2xi
                                                           
    #</methodize4xi>   
    
 
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2program-of-TPU2Program4Xi" usage = "
    #     
    #   print('(Step 210A/25) OF TPU2PROGRAM4XI5TEST - TRYING TO TRACE TPU: ', time4xi())      
    #   trace2tpu = None
    #   try:
    #       trace2tpu = programobject2tpu.trace2program() 
    #       print('(Step 210B/25) OF TPU2PROGRAM4XI5TEST - TRACE2TPU: ', trace2tpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2program4tpu-failed {}; time: 04:40:24 11/16/11 Pacific Standard Time}', 'CLASS': 'trace2program4tpu', 'METHOD': 'trace2program4tpu'}
    #   except:
    #       print('(Step 210Z1/25) OF TPU2PROGRAM4XI5TEST *** FAILED TO TRACE TPU: ', trace2tpu) 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2program(self, task4trace2program = None):
        '''
        traces TPU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "trace2program"
                     
        message2print = "{(Step 1A/33) started" + "; trace2program-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2program-passed-to-trace-of-TPU2Program4Xi: " + str(task4trace2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
         
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2program-passing-to-trace2program4tpu-by-trace2program-of-TPU2Program4Xi: " + str(task4trace2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
         
  
        trace2xi = trace2program4tpu(task4trace2program)

        message2print = "{(Step 2B/33) performed" + "; trace2program4tpu-performed-by-trace2program-of-TPU2Program4Xi: " + str(trace2program4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2program-of-TPU2Program4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #       
                
        return trace2xi
                                                           
    #</methodize4xi>       

                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "unload2compiler-of-TPU2Program4Xi" usage = "
    #        
    #   print('(Step 220A/25) OF TPU2PROGRAM4XI5TEST - TRYING TO UNLOAD COMPILER OF TPU: ', time4xi())      
    #   try:
    #       response4unload2compiler = programobject2tpu.unload2compiler() 
    #       print('(Step 220B/25) OF TPU2PROGRAM4XI5TEST - UNLOADED COMPILER OF TPU: ', response4unload2compiler)  
    #       # 
    #   except:
    #       print('(Step 220Z1/25) OF TPU2PROGRAM4XI5TEST *** FAILED TO TO UNLOAD COMPILER OF TPU: ', time4xi()) 
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def unload2compiler(self, task4unload2compiler = None):
        '''
        unloads compiler
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "unload2compiler"
                     
        message2print = "{(Step 1A/33) started" + "; unload2compiler-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
            
        message2print = "{(Step 1B/33) passed" + "; task4unload2compiler-passed-to-trace-of-TPU2Program4Xi: " + str(task4unload2compiler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
         
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4unload2compiler-passing-to-unload2compiler4tpu-by-unload2compiler-of-TPU2Program4Xi: " + str(task4unload2compiler) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
         
  
        trace2xi = unload2compiler4tpu(task4unload2compiler)

        message2print = "{(Step 2B/33) performed" + "; unload2compiler4tpu-performed-by-unload2compiler-of-TPU2Program4Xi: " + str(unload2compiler4tpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-unload2compiler-of-TPU2Program4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #       
                
        return trace2xi
                                                           
    #</methodize4xi>       

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2programcallback-of-TPU2Program4Xi" usage = "
    #
    #   print('(Step 200A/700) OF TPU2PROGRAM4XI5TEST - TRYING TO SET PROGRAM CALLBACK WITH METHOD OF OBJECT XI2TPU: ', programobject2tpu) 
    #   try:
    #       callback2tpu = programobject2tpu.set2programcallback()                                  
    #       print('(Step 200B/700) OF TPU2PROGRAM4XI5TEST - SET PROGRAM WITH CALLBACK TO: ', callback2tpu)
    #       # <function trace2program4tpu at 0x00000000038E02C8>
    #   except:
    #       print('(Step 200Z/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO SET PROGRAM CALLBACK WITH METHOD ***', time4xi())
    #
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2programcallback(self, task4set2programcallback = None): 
        '''
        set2programcallback of TPU2Program4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "TPU2Program4Xi"
        method2print = "set2programcallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2programcallback-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
            
        message2print = "{(Step 1B/33) passed" + "; task4set2programcallback-passed-to-set2programcallback-of-TPU2Program4Xi: " + str(task4set2programcallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
           
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2programcallback-passing-to-set2programcallback-by-set2programcallback-of-TPU2Program4Xi: " + str(task4set2programcallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        
        callback2tpu4program = {} 
        try:
            callback2tpu4program = set2programcallback4tpu (task4set2programcallback)

            
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2tpu4program-failed-by-set2programcallback-of-TPU2Program4Xi: " + str(callback2tpu4program) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2tpu4program-returning-by-set2programcallback-of-TPU2Program4Xi: " + str(callback2tpu4program) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2program4tpu at 0x00000000038CF5C8> 
        
                                             
        return callback2tpu4program
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-TPU2Program4Xi" usage = "
    #
    #   print('(Step 310A/700) OF TPU2PROGRAM4XI5TEST - SENSING ', time4xi())  
    #   try:
    #       print(programobject2tpu.sense())
    #       print('(Step 310B/700) OF TPU2PROGRAM4XI5TEST - SENSED ', time4xi()) 
    #       # {'object': <tpu2program4xi.TPU2Program4Xi object at 0x0000000003E00AC8>, 'classvariable2tpu-of-TPU2Program4Xi': 4}
    #   except:
    #       print('(Step 310Z/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO SENSE ', time4xi())     
    #
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses TPU2Program4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'TPU2Program4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; TPU2Program4Xi-sensing-by-sense-of-TPU2Program4Xi: " + str(TPU2Program4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 
            
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-TPU2Program4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2tpu-of-TPU2Program4Xi'] = TPU2Program4Xi.classvariable2tpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-TPU2Program4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; TPU2Program4Xi-sensed-by-sense-of-TPU2Program4Xi: " + str(TPU2Program4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-TPU2Program4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
                   
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-TPU2Program4Xi" usage = "
    # 
    #   print('(Step 320A/700) OF TPU2PROGRAM4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(programobject2tpu.reason())
    #       print('(Step 320B/700) OF TPU2PROGRAM4XI5TEST - REASONED ', time4xi())
    #       # TPU2Program4Xi-reasoned-by-reason-of-TPU2Program4Xi: <class 'tpu2program4xi.TPU2Program4Xi'>; time: 04:40:25 11/16/11 Pacific Standard Time}
    #   except:
    #       print('(Step 320Z/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO REASON ', time4xi()) 
    #
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons TPU2Program4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'TPU2Program4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; TPU2Program4Xi-reasoning-by-reason-of-TPU2Program4Xi: " + str(TPU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-TPU2Program4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
             
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-TPU2Program4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-TPU2Program4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; TPU2Program4Xi-reasoned-by-reason-of-TPU2Program4Xi: " + str(TPU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-TPU2Program4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        #           
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-TPU2Program4Xi" usage = "
    #
    #   print('(Step 330A/700) OF TPU2PROGRAM4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(programobject2tpu.ace())
    #       print('(Step 330B/700) OF TPU2PROGRAM4XI5TEST - ACED ', time4xi())                                                 
    #       # {'object-reason-object4xi': <tpu2program4xi.TPU2Program4Xi object at 0x0000000003E00AC8>, 'time-reason-of-TPU2Program4Xi': '04:40:25 11/16/11 Pacific Standard Time'}
    #   except:
    #       print('(Step 330Z/700) OF TPU2PROGRAM4XI5TEST *** FAILED TO ACE ', time4xi())                                 
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces TPU2Program4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'TPU2Program4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-TPU2Program4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; TPU2Program4Xi-acing-by-ace-of-TPU2Program4Xi: " + str(TPU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #      
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-TPU2Program4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-TPU2Program4Xi'] = TPU2Program4Xi.classvariable2tpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-TPU2Program4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; TPU2Program4Xi-aced-by-ace-of-TPU2Program4Xi: " + str(TPU2Program4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'tpu2sourceprogram4xi.TPU2Program4Xi'> 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-TPU2Program4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>

# =================================================================================================
#<class4xi class = "DirectoryWalker" base = "None" child = "None" synopsis = "a forward iterator that traverses a directory tree" usage = "
#   to review all tests"
#>
# =================================================================================================
class DirectoryWalker:

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init" usage = "initializes DirectoryWalker"> 
    # ---------------------------------------------------------------------------------------------
    def __init__(self, directory):
        self.stack = [directory]
        self.files = []
        self.index = 0
        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "getitem" usage = "getting item"> 
    # ---------------------------------------------------------------------------------------------
    def __getitem__(self, index):
        while 1:
            # ------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------          
            try:
                # --------------------------------------------------------------------------------- 
                #
                # ---------------------------------------------------------------------------------             
                file = self.files[self.index]
                self.index = self.index + 1
            except IndexError:
                # --------------------------------------------------------------------------------- 
                # pop next directory from stack
                # ---------------------------------------------------------------------------------             
                self.directory = self.stack.pop()
                self.files = os.listdir(self.directory)
                self.index = 0
            else:
                # --------------------------------------------------------------------------------- 
                # got a filename
                # ---------------------------------------------------------------------------------             
                fullname = os.path.join(self.directory, file)
                if os.path.isdir(fullname) and not os.path.islink(fullname):
                    self.stack.append(fullname)
                return fullname
    #</methodize4xi>
#</class4xi>

#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   tpu2sourceprogram4xi5test.txt
#   sense-tpu2sourceprogram4xi5test.xml
#   reason-tpu2sourceprogram4xi5test.xml
#   ace-tpu2sourceprogram4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of tpu2program4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
